function amoeba, p, fun=fun, plim=plim, silent=silent, itm=itm
;
; Simplex routine for minimization of function
;
;   CALL :    p    array ndim+1 x ndim where ndim is number of
;                     parameters for the function to minimize
;                     ndim parameters in ndim+1 points - the simplex
;  KEYWORDS:  fun  name of function to return single value
;                       y = fun(params) where params is array
;                                       with parameters
;             plim array 2 x ndim are the limits for the ndim
;                     parameters  plim(0,i) < p(j,i) < plim(1,i)
;
;  RETURN:    array with best fit parameters
;
sp = size(p)
ndim = sp(2)
if( sp(0) ne 2 ) then begin
   print,' p array does not have 2 dimensions'
   return, -999.
endif
if( sp(1) ne 1+sp(2) ) then begin
   print,' p array dimensions do not match internally'
   return, -999.
endif
if keyword_set(fun) then fun_name = fun else fun_name='funk'
if keyword_set(silent) then verbose = 0 else verbose = 1
if keyword_set(itm) then itmax=itm else itmax=60
if keyword_set(plim) then begin
   sl = size(plim)
   if( sl(0) ne 2 ) then begin
      print,' plim array does not have 2 dimensions'
      return, -999.
   endif
   if( sl(2) ne sp(2) ) then begin
      print,' plim array dimensions do not match the p dimension'
      return, -999.
   endif
   if( sl(1) ne 2 ) then begin
      print,' plim array first dimension must be 2'
      return, -999.
   endif
endif else begin
   plim = replicate(1.e31,2,ndim)
   plim(0,*) = -1.e31
endelse
;
amin = fltarr(2)
alpha = 1.0
beta = 0.5
gamma = 2.0
it = 0
ftol = 1.e-4
pr = fltarr(ndim)
prr = fltarr(ndim)
pbar = fltarr(ndim)
; define y before starting iterations
y = fltarr(ndim+1)
for i = 0, ndim  do y(i) = call_function(fun_name, p(i,*))

while( 1 gt 0) do begin

; Get highest, next highest and lowest point
   ymax = max(y,ihi)
   ymin = min(y,ilo)
   y(ilo) = ymin - 1.
   y(ihi) = ymin - 1.
   ynhi = max(y,inhi)
   y(ihi) = ymax
   y(ilo) = ymin
;cx   print,ihi,inhi,ilo
;cx  if verbose then print,it, ymax, ynhi, ymin
   if verbose then print,it, p(0,*), ymin

; Test for satisfactory approximation
   denom = abs(ymax) + abs(ymin)
   if( denom lt ftol ) then begin
      return, p(ilo,*)
   endif else begin
      rtol = 2.*abs(ymax-ymin) / denom
      if( rtol lt ftol ) then return, p(ilo,*)
   endelse

; Test for number of iterations
   if( it eq itmax ) then begin
      if verbose then print, itmax,' max no. of iterations - stop'
      return, p(ilo,*)
   endif
   it = it + 1

   pbar(*) = 0.
   for i = 0, ndim do begin
      if( i ne ihi ) then pbar(*) = pbar(*) + p(i,*)
   endfor
   pbar = pbar / ndim
   pr = (1.+alpha)*pbar - alpha*p(ihi,*)
   reset = where( plim(0,*) gt pr , stat0)
   if( stat0 gt 0 ) then begin
      if verbose then print,'Hit floor status = ', reset
      alfas = (plim(0,reset) - pbar(reset))/(pbar(reset) - p(ihi,reset))
      amin(0) = min(alfas)
      if verbose then print,' New alpha suggested: ', amin(0)
   endif
   reset = where( plim(1,*) lt pr , stat1)
   if( stat1 gt 0 ) then begin
      if verbose then print,'Hit ceiling status = ', reset
      alfat = (plim(1,reset) - pbar(reset))/(pbar(reset) - p(ihi,reset))
      amin(1) = min(alfat)
      if verbose then print,' New alpha suggested: ', amin(1)
   endif
   if( stat0 or stat1 ) then begin
      alfa = min(amin)
      pr = (1.+alfa)*pbar - alfa*p(ihi,*)
   endif
   ypr = call_function(fun_name, pr)   ; evaluate the function at the reflected point
   case 1 of
      (ypr le ymin) : begin
         prr = gamma*pr + (1. - gamma)*pbar
         reset = where( plim(0,*) gt prr , status)
         if( status gt 0 ) then prr(reset) = plim(0,reset)
         reset = where( plim(1,*) lt prr , status)
         if( status gt 0 ) then prr(reset) = plim(1,reset)
         yprr = call_function(fun_name, prr)
         if( yprr lt ymin ) then begin
            p(ihi,*) = prr
            y(ihi) = yprr
         endif else begin  ; extra long step failed - use first step
            p(ihi,*) = pr
            y(ihi) = ypr
         endelse
      end
      (ypr ge ynhi) : begin
         if( ypr lt ymax ) then begin
            p(ihi,*) = pr
            y(ihi) = ypr
         endif
         prr = beta*pr + (1. - beta)*pbar
         reset = where( plim(0,*) gt prr , status)
         if( status gt 0 ) then prr(reset) = plim(0,reset)
         reset = where( plim(1,*) lt prr , status)
         if( status gt 0 ) then prr(reset) = plim(1,reset)
         yprr = call_function(fun_name, prr)
         if( yprr lt y(ihi) ) then begin
            p(ihi,*) = prr
            y(ihi) = yprr
         endif else begin
            for i = 0, ndim do begin
               if( i ne ilo ) then begin
                  pr = 0.5*(p(i,*) + p(ilo,*))
                  reset = where( plim(0,*) gt pr , status)
                  if( status gt 0 ) then pr(reset) = plim(0,reset)
                  reset = where( plim(1,*) lt pr , status)
                  if( status gt 0 ) then pr(reset) = plim(1,reset)
                  p(i,*) = pr
                  y(i) = call_function(fun_name, pr)
               endif
            endfor
         endelse
      end
      else : begin
         p(ihi,*) = pr
         y(ihi) = ypr
      end
   endcase
endwhile
end
