pro collect_spectra, ref_sourceid, source_path, srcid2=srcid2, njmx=njmx, $
                     flist=flist, outspe=outspe, outarf=outarf, noplot=noplot
;
;   Get several spectra combined from data
;
;    2003-04-11/NJW
;    2003-05-14/NJW, update
;    2003-06-10/NJW, update
;    2003-10-29/NJW, update
;    2004-02-06/NJW, update, include possibility to give a list of files
;    2004-06-18/NJW, update, take RMF.-GRP from /r6/jemx/ic/rmf_grp
;    2005-05-10/NJW, update for njw-notebook(laptop)
;    2005-08-17/NJW, update, allow keywords for output names
;
;ref_sourceid = 'J195821.7+351206' ; Cyg X-1
;ref_sourceid = 'J171958.0-314648'  ; XTE1720-318
;ref_sourceid2 = 'XTE1720-318'

nparams = n_params()
if nparams lt 1 then begin
   print,'Syntax: collect_spectra, ref_sourceid[, source_path][, srcid2=], $'
   print,'         [,flist=][, njmx=][, outspe=, outarf=][,/noplot]'
   print
   print,'   ref_sourceid is source id such as J195821.7+351206'
   print,'        (special case: ''crab'' yields J053432.0+22052)'
   print,'   source_path is a path under which all jmxi_srcl_spe.fits files'
   print,'         can be found'
   print,'  Keyword   srcid2 : The user can give a second name for the source'
   print,'  Keyword   njmx   : JEMX instrument number (defaults to 2)'
   print,'  Keyword   flist  : If present the string array with file names will'
   print,'                     override those found through source_path'
   print,'  Keyword   outspe, outarf : File names for the resulting spectral file and'
   print,'                    ARF file. Both must be present or none'
   print,'  Keyword   noplot : Prevent plotting on the screen'
   print
   print,' The output files will be placed in the current directory as jmxi_0000_spe.fits etc.'
   print,'    except if _both_ these are set:
   print
   print,'  Keyword  outspe  : Name of output spectral file'
   print,'  Keyword  outarf  : Name of output ARF file'
   print
   print,'A logfile by name of collect_spectra_0000.log will be made'
   print
   return
endif

numjemx = 2
if keyword_set(njmx) then numjemx = njmx
if numjemx ne 1 and numjemx ne 2 then begin
   print,'Illegal JEMX number'
   return
endif
snj = stri(numjemx)

if not keyword_set(flist) and nparams eq 1 then begin
   print,'There is no indication of where to find the spectra ...'
   return
endif

; See if both keywords 'outspe' and 'outarf' are set

default_out_names = 1
if keyword_set(outspe) then begin
   if keyword_set(outarf) then begin
      default_out_names = 0  ; don't use default output names
   endif else begin
      print,'Warning: only ''outspe'' has been set - use default output names'
   endelse
endif

pflag = 1 - keyword_set(noplot)

; Get list of spectral files to include

if keyword_set(flist) then begin

   list = flist

endif else begin

   print,'Start building list ..... (takes some time) .....'
   spawn,'find '+source_path+' -name jmx'+snj+'_srcl_spe.fits -print',list
   ; Avoid lines in 'list' that are not file names
   p = strpos(list,'srcl_spe')
   w = where(p ge 0, nw)
   if nw eq 0 then begin
      print,'No spectrum files found'
      return
   endif
   list = list(w)

endelse

; Apply special case for Crab

if strlowcase(ref_sourceid) eq 'crab' then ref_sourceid = 'J053432.0+220052'

nlist = n_elements(list)
list = list(sort(list))
list_used = list
nlist_used = 0


; Prepare the output files jmxi_nnnn_spe.fits and jmxi_nnnn_arf.fits
; in the current directory but only if keywords outspe and outarf are set

if default_out_names then begin

   fname_spe = get_next_filename('jmx'+snj+'_????_spe.fits')
   strnum = strmid(fname_spe,5,4)
   fname_arf = fname_spe
   strput,fname_arf,'arf',10
   logfile = 'collect_spectra_'+strnum+'.log'

endif else begin

   fname_spe = outspe
   fname_arf = outarf
   logfile = get_next_filename('collect_spectra_????.log')

endelse

spawn,'cp /home/njw/jemx/spectra/jmx'+snj+'_coll_spe.fits '+fname_spe
spawn,'cp /home/njw/jemx/spectra/jmx'+snj+'_coll_arf.fits '+fname_arf

tot_arf  = fltarr(500)
tot_spec = fltarr(256)
tot_err2 = fltarr(256)
tot_expo = 0.0

print,'Output logging data in '+logfile
openw,log,logfile,/get_lun
printf,log,'  Logging of collect_spectra   '+systime()
printf,log
printf,log,'Resulting SPE file: '+fname_spe
printf,log,'Resulting ARF file: '+fname_arf

for i = 0, nlist-1 do begin

   print,'Analyzing '+stri(i+1)+' of '+stri(nlist)
   printf,log,'-------------------------------'
   printf,log,list(i)
   ; Note if path is absolute
   first_char = strmid(list(i),0,1) ; to be used for ARF name
   ; Get header
   if i gt 0 then respfile_prev = respfile_ori
   hdr = headfits(list(i),exten=1)
   nsources = fxpar(hdr,'NAXIS2')
   if nsources le 0 then begin
      print,'Skip - no sources'
      printf,log,'Skip this one, there are no sources'
      continue
   endif
   respfile_ori = fxpar(hdr,'RESPFILE')
   tok = strsplit(respfile_ori,'/',/extract)
   ntok = n_elements(tok)
   jj = -1
   for j = 0, ntok-1 do begin
      if strlen(tok(j)) ge 18 then begin
         p1 = strpos(tok(j),'jmx')
         p2 = strpos(tok(j),'rmf')
         p3 = strpos(tok(j),'fits')
         if p1 ge 0 and p2 ge 0 and p3 ge 0 then begin
            jj = j
            break
         endif
      endif
   endfor
   if jj ge 0 then begin
      respfile = tok(jj)
      printf,log,'RESPFILE = '+respfile
   endif else begin
      printf,log,'RESPFILE is unidentified'
      print,'Problem finding the RESPFILE'
   endelse
   if i gt 0 then begin
      if respfile_ori ne respfile_prev then begin
         print,'Warning - change of response file'
         printf,log,'Warning - change of response file'
      endif
   endif
   ; read the source identifications
   rdfitscol,list(i)+'+1',sourceid,'ROWID',/sil
   rdfitscol,list(i)+'+1',exposure,'EXPOSURE',/sil
   rdfitscol,list(i)+'+1',ancrfile,'ANCRFILE',/sil
   ancrfile = strtrim(ancrfile,2)
   sourceid = strtrim(sourceid,2)
   printf,log,'Source IDs: ', sourceid
   ; locate actual source
   if keyword_set(srcid2) then begin
      w = where( sourceid eq ref_sourceid or sourceid eq srcid2, n_w)
   endif else begin
      w = where( sourceid eq ref_sourceid, n_w)
   endelse

   if n_w eq 1 then begin

      ; include this one in the list of used spectral files
      list_used(nlist_used) = list(i)
      nlist_used = nlist_used + 1

      printf,log,'Actual source is number ', stri(w(0)+1),' (starting with 1)'

      pick_swid_str, list(i), swid
      szw = size(swid)
      if szw(0) eq 0 then swid = 'Unknown SWID' else swid = swid(0)

      tok = strsplit(list(i),'/',/extract)
      ntok = n_elements(tok)
      ;* ; Find in which 'tok' the SWID is hidden
      ;* jj = -1
      ;* for j = 0, ntok-1 do begin
         ;* if strlen(tok(j)) ge 12 then begin
            ;* if strmid(tok(j),0,1) eq '0' and $
               ;* strmid(tok(j),4,1) eq '0' and $
               ;* strmid(tok(j),8,1) eq '0' then jj = j
         ;* endif
      ;* endfor
      ;* if jj ge 0 then $
         ;* printf,log,tok(jj)+'  '+stri(nsources)+' sources'
      printf,log,swid+'  '+stri(nsources)+' sources'
      printf,log,'Exposure time is '+string(exposure(w(0)),format='(f6.0)')+' s'

      ; Get the spectrum etc.

      rdfitscol,list(i)+'+1',ra_obj,'RA_OBJ',/sil
      rdfitscol,list(i)+'+1',dec_obj,'DEC_OBJ',/sil
      rdfitscol,list(i)+'+1',rate,'RATE',/sil
      rdfitscol,list(i)+'+1',stat_err,'STAT_ERR',/sil
      if pflag then begin
	 dataplot,findgen(256),rate(*,w(0)),stat_err(*,w(0))

         ;* if jj ge 0 then xyouts,0.2,0.85,tok(jj),charsize=1.3,/norm
         if jj ge 0 then xyouts,0.2,0.85,swid,charsize=1.3,/norm
         xyouts,0.15,0.75,'Exposure '+string(exposure(w(0)), $
                   format='(f5.0," s")'),charsize=1.3,/norm

         if !d.name ne 'PS' then wait,1.
         if !d.name eq 'PS' then erase
      endif

      ; Get the corresponding ARF file

      ;tok(ntok-1) = 'jmx'+snj+'_srcl_arf.fits'
      tok(ntok-1) = strip_curly_br(ancrfile(w(0)))
      arffile = collate_path(tok)
      if first_char eq '/' then arffile = '/'+arffile
      errmsg = ''
      rdfitscol,arffile+'+1',arf,'SPECRESP',errmsg=errmsg,/sil
      if errmsg ne '' then begin
         print,'rdfitscol error: '+errmsg
         print,'DOL: '+arffile+'+1'
         free_lun, log
         return
      endif
      tot_arf  = tot_arf  + reform(arf(*,w(0)),500)*exposure(w(0))
      tot_spec = tot_spec + reform(rate(*,w(0)),256)*exposure(w(0))
      tot_err2 = tot_err2 + (reform(stat_err(*,w(0)),256)*exposure(w(0)))^2
      tot_expo = tot_expo + exposure(w(0))
   endif else printf,log,'Source was not found'

endfor

printf,log,'// total_exposure = ', tot_expo,' ; s'
printf,log,'//  --- Normal exit ---'

free_lun,log

tot_arf  = tot_arf  / tot_expo
tot_spec = tot_spec / tot_expo
tot_err  = sqrt(tot_err2) / tot_expo

if pflag then dataplot,findgen(256),tot_spec,tot_err

;  Update standard file with collected spectrum

print,'New spectrum file: '+fname_spe
fxbopen, unit, fname_spe, 1, access='RW'
fxbwrite, unit, sourceid(w(0)), fxbcolnum(unit,'ROWID'), 1
fxbwrite, unit, ra_obj(w(0)), fxbcolnum(unit,'RA_OBJ'), 1
fxbwrite, unit, dec_obj(w(0)), fxbcolnum(unit,'DEC_OBJ'), 1
fxbwrite, unit, float(tot_spec), fxbcolnum(unit,'RATE'), 1
fxbwrite, unit, float(tot_err), fxbcolnum(unit,'STAT_ERR'), 1
fxbwrite, unit, double(tot_expo), fxbcolnum(unit,'EXPOSURE'), 1
fxbwrite, unit, fname_arf+'{1}', fxbcolnum(unit,'ANCRFILE'), 1
fxbfinish, unit

;  Update the RESPFILE keyword and add the list
;  of input files

openw,lut,'tmpfil',/get_lun
printf,lut,'RESPFILE = ''/r6/jemx/ic/rmf_grp/'+respfile+''''
printf,lut,'SOURCEID = '''+ref_sourceid+''''
for i = 0, nlist_used - 1 do begin
   s = list_used(i)
   len = strlen(s)
   while len gt 70 do begin
      scut = strmid(s,0,70)+' $'
      printf,lut,'COMMENT '+scut
      s = strmid(s,70,9999)
      len = strlen(s)
   endwhile
   printf,lut,'COMMENT '+s
endfor

free_lun,lut

spawn,'fmodhead '+fname_spe+'+1 tmpfil=tmpfil'

;  Update the checksum

spawn,'fchecksum '+fname_spe+' update=yes'

;  Update file with collected average ARF


print,'New ARF file: '+fname_arf
fxbopen, unit, fname_arf, 1, access='RW'
fxbwrite, unit, float(tot_arf), fxbcolnum(unit,'SPECRESP'), 1
fxbfinish, unit

;  Update the checksum

spawn,'fchecksum '+fname_arf+' update=yes'


end
