pro dataplot, x, y, dy, xrange=xrg, yrange=yrg, psym=psym, $
     xbar=xbar, fxbar=fxbar, itype=itype, nodata=nodata, _extra=e
;
; Plot of datapoints with 'errorbars' in x-direction defined
; by channel limits derived from spacing between points.
; In the y-direction the length of the error bars are given by '+-dy'
; In the x-direction bars only appear if keyword 'xbar' has been
; specified
;
; The type of plot is indicated with the keyword 'itype'
;   itype = 0  :    x lin - y lin (default)
;   itype = 1  :    x lin - y log
;   itype = 2  :    x log - y lin
;   itype = 3  :    x log - y log
;
;   ~1995/NJW
;   2004-09-28/NJW  Updated with better error message
;   2005-05-27/NJW  Included keyword 'fxbar': fixed x-bar
;
if( n_params() ne 3 ) then begin
   print,'dataplot, x, y, dy[, xrange=][, yrange=][, psym=] $'
   print,'    [,/xbar][,fxbar=][,/nodata][, itype=]'
   return
endif
nx = n_elements(x)
if( (nx ne n_elements(y)) or (nx ne n_elements(dy)) ) then begin
  print,'DATAPLOT error: The arrays have different dimensions !'
  return
endif

; Define the plot type (lin-lin, lin-log, log-lin, log-log)

it = 0
if keyword_set(itype) then it = itype
xlog = 0
ylog = 0
if it gt 1 then xlog = 1
if (it mod 2) eq 1 then ylog = 1

; Define boundaries between abscissa values

nx = n_elements(x)
sx = shift(x,1)
sxl = fltarr(nx+1)
if xlog then begin
   sx = sqrt(x*sx)
   sxl(0:nx-1) = sx
   sxl(0) = sqrt(x(0)^3 / x(1))
   sxl(nx) = sqrt(x(nx-1)^3 / x(nx-2))
endif else begin
   sx = 0.5*(x + sx)
   sxl(0:nx-1) = sx
   sxl(0) = (3*x(0) - x(1))*0.5
   sxl(nx) = (3*x(nx-1) - x(nx-2))*0.5
endelse

; Define abscissa plot range

xdmin = min(x)
xdmax = max(x)
if keyword_set(xbar) then begin
   xdmin = min(sxl)
   xdmax = max(sxl)
endif
if keyword_set(fxbar) then begin
   xdmin = xdmin - fxbar
   xdmax = xdmax + fxbar
endif
if not keyword_set(xrg) then begin
   if xlog then begin
      xmin = xdmin^1.05 / xdmax^0.05
      xmax = xdmax^1.05 / xdmin^0.05
   endif else begin
      xmin = xdmin - 0.05 * (xdmax - xdmin)
      xmax = xdmax + 0.05 * (xdmax - xdmin)
   endelse
   xrg = [xmin,xmax]
endif

; Define ordinate plot range

if not keyword_set(yrg) then begin
   if ylog then begin
      positive = where( y - dy gt 0 )
      ymin = min(y(positive)-dy(positive))^1.05 / max(y+dy)^0.05
      ymax = max(y+dy)^1.05 / min(y(positive)-dy(positive))^0.05
   endif else begin
      ymin = min(y-dy) - 0.05 * (max(y+dy) - min(y-dy))
      ymax = max(y+dy) + 0.05 * (max(y+dy) - min(y-dy))
   endelse
   yrg = [ymin,ymax]
endif

; Set up plot

if xlog then begin
   if ylog then begin 
      plot_oo,[1],[1],xr=xrg,yr=yrg,/nodata,_extra=e
   endif else begin
      plot_oi,[1],[1],xr=xrg,yr=yrg,/nodata,_extra=e
   endelse
endif else begin
   if ylog then begin 
      plot_io,[1],[1],xr=xrg,yr=yrg,/nodata,_extra=e
   endif else begin
      plot,[1],[1],xr=xrg,yr=yrg,/nodata,_extra=e
   endelse
endelse

if keyword_set(nodata) then return

; plot first horizontal piece
if keyword_set(xbar) then oplot,sxl(0:1),[y(0),y(0)],_extra=e
; plot first vertical piece
oplot,[x(0),x(0)],[y(0)-dy(0),y(0)+dy(0)],_extra=e
for i = 1L, nx do begin
  ;  horizontal pieces (only if requested)
  if keyword_set(xbar) then oplot,sxl(i-1:i),[y(i-1),y(i-1)],_extra=e
  ; vertical pieces
  oplot,[x(i-1),x(i-1)],[y(i-1)-dy(i-1),y(i-1)+dy(i-1)],_extra=e
endfor

; Plot fixed horizontal piece if requested

if keyword_set(fxbar) then begin
   for i = 0, nx-1 do begin
      oplot,[x(i)-fxbar,x(i)+fxbar],[y(i),y(i)],_extra=e
   endfor
endif

if keyword_set(psym) then oplot,x,y,psym=psym,_extra=e

end
