pro dattim2ijd, dattim, ijd
;
;  2003-01-27/NJW
;
;  Convert a (array of) date/time string (such as 2003-01-27T10:48:17) to
;  IJD i.e. number of days since Jan 01, 00:00, 2000
;
;   2006-12-20/NJW Updated to take into account the 64.184 or 65.184 s
;   difference between UTC and IJD
;   Updated with two more leap seconds: end of 2008 and mid 2012
;
;  IJD is in TT: Terrestrial Time with no leap seconds
;  and starts in 1999-12-31T23:58:55.816 UTC
;
if n_params() ne 2 then begin
   print,'Syntax: dattim2ijd, dattim, ijd'
   return
endif

; Initial data definition

days_in_month = long([31,28,31,30,31,30,31,31,30,31,30,31])
days_in_month_leap = long([31,29,31,30,31,30,31,31,30,31,30,31])
ijd_tmp = -1.0;
ijd = -1.0;

acc_days = lonarr(12)
acc_days_leap = lonarr(12)
for i = 0, 10 do begin
   acc_days(i+1) = acc_days(i) + days_in_month(i)
   acc_days_leap(i+1) = acc_days_leap(i) + days_in_month_leap(i)
endfor

first = 1

; Analyze input (array of) string(s) and check format

sz = size(dattim)
if sz(sz(0)+1) ne 7 then begin
   print,'1. argument is not of type string'
   return
endif
nelem = n_elements(dattim)

for i = 0, nelem-1 do begin

   curstr = strcompress(dattim(i),/remove_all)
   len = strlen(curstr)

   if len lt 13 then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (too short)'
      goto, next
   endif

;  Requirements   2002-11-16T23:45:56
;            or   2002-11-16T23:45
;            or   2002-11-16T23
;      One 'T', two '-'s

   ch = strmid(curstr,10,1)

   if ch ne 'T' then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (no ''T'' in pos 10)'
      goto, next
   endif

   ch = strmid(curstr,4,1)

   if ch ne '-' then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (no ''-'' in pos 4)'
      goto, next
   endif

   ch = strmid(curstr,7,1)

   if ch ne '-' then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (no ''-'' in pos 7)'
      goto, next
   endif

   year = strmid(curstr,0,4)

   if not is_digit(year) then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (year is not a number)'
      goto, next
   endif

   iyear = fix(year)
   if iyear lt 2000 or iyear gt 2099 then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' requirement: 2000 <= year <= 2100'
      goto, next
   endif

   if iyear mod 4 eq 0 then cur_year_is_leap = 1 else cur_year_is_leap = 0

   month = strmid(curstr,5,2)

   if not is_digit(month) then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (month is not a number)'
      goto, next
   endif
   imonth = fix(month)

   if imonth lt 1 or imonth gt 12 then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' illegal month number'
      goto, next
   endif

   day = strmid(curstr,8,2)

   if not is_digit(day) then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (day is not a number)'
      goto, next
   endif

   daymax = days_in_month(imonth-1)
   if cur_year_is_leap and imonth eq 2 then daymax = daymax + 1

   if fix(day) lt 1 or fix(day) gt daymax then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid day number'
      goto, next
   endif

   hour = strmid(curstr,11,2)

   if not is_digit(hour) then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (hour is not a number)'
      goto, next
   endif

   if fix(hour) lt 0 or fix(hour) gt 23 then begin
      print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid hour number'
      goto, next
   endif

   minute = '0'
   second = '0'

   if len ge 16 then begin ; The minute is given as well

      ch = strmid(curstr,13,1)
      if ch ne ':' then begin
         print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (no '':'' in pos 13)'
         goto, next
      endif

      minute = strmid(curstr,14,2)

      if not is_digit(minute) then begin
         print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (minute is not a number)'
         goto, next
      endif

      if fix(minute) lt 0 or fix(minute) gt 59 then begin
         print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid minute number'
         goto, next
      endif

      if len ge 19 then begin ; The second is given as well

         ch = strmid(curstr,16,1)
         if ch ne ':' then begin
            print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (no '':'' in pos 13)'
            goto, next
         endif

         second = strmid(curstr,17,2)

         if not is_digit(second) then begin
            print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid format (second is not a number)'
            goto, next
         endif

         if fix(second) lt 0 or fix(second) gt 59 then begin
            print,'#'+strtrim(i,2)+' '+dattim(i)+' invalid second number'
            goto, next
         endif

      endif

   endif

   num_years = long(iyear - 2000)
   num_leap_years = (num_years+3L)/4L ; before current year

   num_days = num_years * 365L + num_leap_years

   if cur_year_is_leap then begin
      num_days = num_days + acc_days_leap(fix(month)-1) + fix(day) - 1
   endif else begin
      num_days = num_days + acc_days(fix(month)-1) + fix(day) - 1
   endelse
   num_seconds = double(hour)*3600d0 + double(minute)*60d0 + double(second)

   ;* ijd_tmp = double(num_days) + num_seconds/(24d0*3600d0)
   t_dif_s = 64.184d0
   if iyear ge 2006 then t_dif_s = 65.184d0
   if iyear ge 2009 then t_dif_s = 66.184d0
   if iyear eq 2012 and imonth gt 6 then t_dif_s = 67.184d0
   if iyear ge 2013 then t_dif_s = 67.184d0

   t_dif_ijd = t_dif_s / 86400.0d0
   ijd_tmp = double(num_days) + num_seconds/(24d0*3600d0) + t_dif_ijd

   if first then begin
      ijd = ijd_tmp
      first = 0
   endif else begin
      ijd = [ijd, ijd_tmp]
   endelse




   next:
endfor

end
