pro ebm_get_child_by_name, file_extno, child_name, $
                   outfile_extno_res, errmsg, $
                   nchild=nchild, nhits=nhits, grpname=grpname, bygrp=bygrp, $
                   verb=verb
;
;   Get the file with extension number for a given child
;   of a grouping table
;
;   Call:   file_extno   File name with extension number appended,
;                          e.g.  myfile.fits[6]
;                          Counting starts with 0 (zero) so that
;                          the primary HDU has extno 0
;           child_name   Name of child (EXTNAME) in grouping table
;
;  Return:  outfile_extno_res  File name with extension number appended,
;                              The primary HDU has number 0. If several
;                              children are found a string array is returned
;
;  Keywords:  nchild     If set, returns number of children in
;                                    called grouping table
;             grpname   If set, returns GRPNAME of called grouping table
;
;             nhits     If set, returns number of hits. If larger than 1
;                       outfile_extno_res will be a string array with nhits elements
;
;             bygrp     GRPNAME is used instead of EXTNAME for the search
;

np = n_params()
if n_params() lt 3 then begin
   print,'Syntax: ebm_get_child_by_name, file_extno, child_name,$'
   print,'        outfile_extno [, errmsg][,nchild=][,nhits=][,grpname=][,/bygrp][,/verb]'
   return
endif

; Set returned value in case of malfunction:

outfile_extno_res = '?'
grpname = ''
nchild = -1
n_missing = 0
errmsg = ''

; debug
openw,lun,'output',/get_lun
printf,lun,file_extno
printf,lun,'## child_name:  ', child_name

; Separate filename and extension number

if strmid(file_extno,0,1) eq '/' then doinit = 1 else doinit = 0
get_exten_no, file_extno, file, extno
tok = sep_filename(file)
n_tok = n_elements(tok)

; Find out if the file exists

l = findfile(file,count=n_l)
if n_l ne 1 then begin
   print,file,' does not exist'
   errmsg = 'Input file not found'
   return
endif

; Make sure that the input is a grouping table

hdr = headfits(file,exten=extno)

extname = fxpar(hdr,'EXTNAME')
grpname = fxpar(hdr,'GRPNAME')

if extname ne 'GROUPING' then begin
   if np ge 4 then begin
      errmsg = 'Not a grouping table'
   endif else begin
      print,'EBM_GET_CHILD_BY_NAME error: Not a grouping table'
   endelse
   return
endif

rdfitscol,file_extno, mempos, 4, /silent
rdfitscol,file_extno, memloc, 5, /silent

n_children = n_elements(mempos)
nchild = n_children

;debug
printf,lun,'n_children = ', n_children

if n_children eq 0 then begin
   if np ge 4 then begin
      errmsg = 'No children in grouping table'
   endif else begin
      print,'EBM_GET_CHILD_BY_NAME error: No children in grouping table'
   endelse
   return
end


; Search for the requested child

nhits = 0
for index = 0,  n_children-1 do begin

   outextno = mempos(index) - 1

   ;debug
   printf,lun,'index=',index,'  ',outextno,'  ',memloc(index)

   if strlen(memloc(index)) eq 0 then begin ; in same file
      outfile = file
   endif else begin ; filename given
      tok(n_tok-1) = strtrim(memloc(index),2)
      outfile = collate_path(tok,init=doinit)
   endelse
   outfile_extno = outfile+'['+strtrim(outextno,2)+']'

   ;debug
   printf,lun,'outfile_extno: ', outfile_extno

   ; Make sure that the requested file exists. If not then skip
   ; write warning and continue

   list = findfile(outfile, count=n_list)

   if n_list gt 0 then begin ;   The file does exist!

      ; Get the header

      hdr = headfits( outfile, exten=outextno)

      if keyword_set(bygrp) then begin
         compname = fxpar(hdr,'GRPNAME')
         sz = size(compname)
         if sz(1) ne 7 then compname = '!!!!!!!'
       endif else begin
         compname = fxpar(hdr,'EXTNAME')
       endelse
   ;debug
   printf,lun,'compname: ', compname

      if compname eq child_name then begin
         nhits = nhits + 1
         ;debug
         printf,lun,'Agree! hit #', nhits

         if np ge 4 then errmsg = ''
         if nhits eq 1 then begin
            outfile_extno_res = outfile_extno
         endif else begin
            outfile_extno_res = [outfile_extno_res,outfile_extno]
         endelse
      endif

   endif else begin

      n_missing = n_missing + 1
      if keyword_set(verb) then begin
         print,'Warning: file in group: ', outfile
         print,'does not exist. Skip and continue ...'
      endif

   endelse

endfor

if nhits eq 0 then begin
   if np ge 4 then begin
      errmsg = 'Child does not exist in group'
   endif else begin
      print,'EBM_GET_CHILD_BY_NAME error: Requested child was'
      print,'         not found in grouping table'
   endelse
endif

; Report on number of missing elements

if n_missing gt 0 then begin
   print,'Warning: '+strtrim(n_missing,2)+' files are missing'
endif

;debug
free_lun,lun

return

end
