pro filesync, mode=mode
;
;  A procedure to synchronize files in two parallel (twin) directories
;  on two different systems
;
;   2003-06-02/NJW
;   2003-09-09/NJW revised with 'skip_sync.list'
;
;----------------------------------------------------------------------
; Procedure (default, mode==0) :
;
;   1) Go to directory (directory A) that is to be moved to other
;      system (directory B).
;
;      > fsync
;
;      will produce a tar file 'sync.tar' with all regular
;      files* in the directory plus 'synclist' and 'synctarlist'
;      'synctarlist' is a list of files in directory (including
;                    synctarlist and synclist)
;      'synclist' is the same list but with appended modification time
;                 and sorted in reverse time order
;
;      * except those listed in 'skip_sync.list' (This is useful for
;        files like 'Makefile' that are usually not identical copies
;        from system to system
;
;   2) Transfer 'sync.tar' to directory B
;
;   3) Go to directory B
;
;      > xfsync
;
;----------------------------------------------------------------------
;
; Procedure (mode==1) :
;
;   1) Go to directory (directory A) that is to be moved to other
;      system (directory B).
;
;      > gsync
;
;      will produce an ASCII file 'synclist' with all regular
;      files in the directory with appended modification time
;
;   2) Transfer 'synclist' to directory B
;
;   3) Go to directory B
;
;      > xgsync
;
;      that produces a list, 'syncupdates' of files that need
;      update from directory A
;
;   4) Transfer 'syncupdates' to directory A
;
;   5) Go to directory A
;
;      > tar cf sync.tar -[I or T] syncupdates
;
;   6) Transfer 'sync.tar' to directory B
;
;   7) Go to directory B
;
;      > tar xf sync.tar
;
;----------------------------------------------------------------------

if not keyword_set(mode) then begin

   ; Define output files
   
   outfilename = 'synclist'
   openw,lulist,outfilename,/get_lun
   outtarfilename = 'synctarlist'
   openw,lutar,outtarfilename,/get_lun
   
   ; Get list of files in current directory
   ; with information on modification time

   skip_list = read_slist('skip_sync.list')
   list = file_search('*',/test_regular,count=nlist)
   list = filter_done( skip_list, list )
   nlist = n_elements(list)
   
   for i = 0, nlist-1 do begin
      openr,lun,list(i),/get_lun
      a = fstat(lun)
      free_lun,lun
      printf,lulist,list(i)+' '+stri(a.mtime)+'  '+systime(0,a.mtime)
      printf,lutar,list(i)
   endfor
   
   free_lun, lulist
   free_lun, lutar
   
   ; Sort the 'synclist' file

   spawn,'sort -nr +1 synclist > __++__++__'
   spawn,'mv __++__++__ synclist'

   ; Create the tar file. The 'tar' option to get files from
   ; list differs between OSs but also between sites.
   ; In fact DSRI is the only SunOS that requires -I
   
   if !version.os eq 'sunos' then begin
      if '1' eq getenv('DSRIENV') then begin
         spawn,'tar cf sync.tar -I '+outtarfilename
      endif else spawn,'tar cf sync.tar -T '+outtarfilename
   endif
   if !version.os eq 'linux' then $
      spawn,'tar cf sync.tar -T '+outtarfilename
   
   ; Clean up by removing 'synclist' and 'synctarlist'
   
   spawn,'\rm synclist'
   spawn,'\rm synctarlist'
   
   print,'Now you are ready to transfer sync.tar to twin dir'
   print,'Remember to do ''xfsync'' there before calling ''fsync'''
   print

endif else begin

   ; Define output files
   
   outfilename = 'synclist'
   openw,lulist,outfilename,/get_lun
   
   ; Get list of files in current directory
   ; with information on modification time
   
   list = file_search('*',/test_regular,count=nlist)
   
   for i = 0, nlist-1 do begin
      openr,lun,list(i),/get_lun
      a = fstat(lun)
      free_lun,lun
      printf,lulist,list(i)+' '+stri(a.mtime)+'  '+systime(0,a.mtime)
   endfor
   
   free_lun, lulist
   
   print,'Now you are ready to transfer ''synclist'' to twin dir'
   print,'Next action is to do ''xgsync'' from twin dir'
   print

endelse

end
