function find_cfslide_sources, dol_of_image, thr=thr,$
             userms=userms,$
             nodisp=nodisp, maxnum=maxnum, $
             numjemx=numjemx,$
             radiuslimit=radiuslimit
;
; Find the sources in the a FITS image where the C-program 'cfslide' has
; been run
;
; Return an array with source positions, peak size, and detection significance
;

if n_params() eq 0 then begin
   print,'res = FIND_CFSLIDE_SOURCES(dol_of_image $'
   print,'  [,thr=][,/userms][,/nodisp][,maxnum=]'
   print,'res is an array res(4,nsources_found) with position, peaksize'
   print,'     and detection level for each accepted source'
   return, -1
endif

out_file_name = get_next_filename('fcs_????.scm')
strnum = strmid(out_file_name,4,4)

; Define radius limit

rlim = 1.e20
if keyword_set(radiuslimit) then rlim = radiuslimit



; Separate file name and extension number

get_exten_no, dol_of_image, im_file_name, im_exten_no

; Get the data

im_orig = nanclean(readfits(im_file_name, hdr_orig, exten_no=im_exten_no))

; Avoid sources on the edge

w = where( im_orig eq 0.0, nw )
sz = size(im_orig)
idimx = sz(1)
idimy = sz(2)
b = fltarr(idimx, idimy)
n_wz = 0
if nw gt 0 then begin
   b(w) = 1.
   b = smooth(b,15)
   wz = where(b gt 0.05,n_wz)
endif

;  Analyse the FITS file to see if IMAGEFLAT, K_IMAGE, and CHI2R_IMAGE exist

num_ext = nfits_extens( im_file_name )

k_ext = -1
chi2r_ext = -1
flat_ext = -1
lrms_ext = -1
for i = 1, num_ext do begin
   hdr = headfits( im_file_name, ext=i)
   imatype = fxpar(hdr,'IMATYPE')
   si = size(imatype)
   if si(0) eq 0 then begin
	 if si(1) eq 7 then begin
            imatype = strtrim(imatype,2)
	    if imatype eq 'K_IMAGE' then k_ext = i
	    if imatype eq 'CHI2R_IMAGE' then chi2r_ext = i
	    if imatype eq 'IMAGEFLAT' then flat_ext = i
	    if imatype eq 'LOCALRMS' then lrms_ext = i
      endif
   endif
endfor
if k_ext lt 0 then begin
   print,'No K_IMAGE extension'
endif else begin
   print,'K_IMAGE in extension# '+stri(k_ext)
endelse
if chi2r_ext lt 0 then begin
   print,'No CHI2R_IMAGE extension'
endif else begin
   print,'CHI2R_IMAGE in extension# '+stri(chi2r_ext)
endelse
if flat_ext lt 0 then begin
   print,'No IMAGEFLAT extension'
endif else begin
   print,'IMAGEFLAT in extension# '+stri(flat_ext)
endelse
if lrms_ext lt 0 then begin
   print,'No LOCALRMS extension'
endif else begin
   print,'LOCALRMS in extension# '+stri(lrms_ext)
endelse

; Terminate if required auxiliary extensions are not all found

if k_ext lt 0 or chi2r_ext lt 0 or flat_ext lt 0 then begin
   print,'At least one of the required extensions was missing'
   return,-1
endif
if keyword_set(userms) and lrms_ext lt 0 then begin
   print,'RMS reduction was requested but no LOCALRMS found'
   return,-1
endif


openw,lun,out_file_name,/get_lun

; Interpret keywords

;   'nodisp'
display = 1
if keyword_set(nodisp) then display = 0

;  'thr'
threshold = 3.0    ; default value
if keyword_set(thr) then threshold = thr

; 'maxnum'
maxnum_sources = 3
if keyword_set(maxnum) then maxnum_sources = maxnum
res_array = fltarr(6,maxnum_sources) ; prepare for returning the results

; Start output til log-file

printf,lun,'// Result of ''find_cfslide_sources'' '+systime()
printf,lun,'// dol_of_image = '+dol_of_image
printf,lun,'// Significance Threshold = ', threshold
printf,lun,'// maxnum_sources = ', maxnum_sources
if keyword_set(swid) then begin
   printf,lun,'// Coordinates calculated as NL image with:'
   printf,lun,'// SWID = ', swid
endif

im_flat = nanclean(readfits( im_file_name, ext=flat_ext ))

if display then disp,im_flat,fac,ma=400,tit=dol_of_image+' Flatfielded'; Left window

k_image = nanclean(readfits(im_file_name,ext=k_ext))
if n_wz gt 0 then k_image(wz) = 0.0
greg = where(k_image ne 0.0, n_greg)
if n_greg eq 0 then begin
   print,'K_IMAGE is always zero'
   free_lun,lun
   return,-1
endif

if keyword_set(userms) then begin
   local_rms = nanclean(readfits(im_file_name,ext=lrms_ext))
   div = where( local_rms gt 0.0, n_div)
   zer = where( local_rms le 0.0, n_zer)
   if n_div eq 0 then begin
      print,'LOCALRMS is always zero'
      free_lun,lun
      return,-1
   endif
   k_image(div) = k_image(div) / local_rms(div)
   if n_zer gt 0 then k_image(zer) = 0.0
   printf,lun,'// k_image has been reduced with local_rms'
endif

s_image = nanclean(readfits(im_file_name,ext=chi2r_ext))


; Define the distribution of pixel values in the K image

histo,k_image(greg), hist_k, xk,nbins=200
; Accumulate to find 99.6% point
acc = accumulate(hist_k, /norm)
klim = interpol(xk,acc,0.996)
if display then begin
   window,2,title=dol_of_image
   plot,xk,hist_k,ps=10
   maxy = max(hist_k)
   draw_arrow, klim, 0.5*maxy, klim, 0.1*maxy,0,0.05,0.07
endif


k_accept_limit = threshold*klim
printf,lun,'// klim = ', klim
printf,lun,'// k_accept_limit = ', k_accept_limit



if display then disp,k_image,fac,ma=400,pa=1,tit=dol_of_image+' K-image'; Right window

printf,lun,'//'
printf,lun,'// Column 1: local source number or JEMX number'
printf,lun,'// Column 2: image x-value in pixels'
printf,lun,'// Column 3: image y-value in pixels'
printf,lun,'// Column 4: RA of src (deg) or same as col 2'
printf,lun,'// Column 5: DEC of src (deg) or same as col 3'
printf,lun,'// Column 6: K value'
printf,lun,'// Column 7: S value'
printf,lun,'//'

;--------------------------------------------------------------
;  Loop for searching for sources. The candidates are the maxima
;           in the K matrix
;--------------------------------------------------------------

im_flat_max = max(im_flat)
k_image_max = max(k_image)
im_flat_disp = im_flat
k_image_disp = k_image

n_candidate_srcs = 0
n_accepted_srcs = 0
while n_candidate_srcs lt maxnum_sources do begin

; Find the maximum value pixel in the K image

   maxim,k_image,mmmx,xc,yc,/silent
   peak_x = float(xc)
   peak_y = float(yc)

   n_candidate_srcs = n_candidate_srcs + 1


   print,'Peak number ', n_candidate_srcs,'  ----------------------------'
   printf,lun,'Peak number ', n_candidate_srcs,'  ----------------------------'

   ;  Determine better position by centroiding

   xspan = indgen(7) + xc - 3
   xvalid = where( xspan ge 0 and xspan lt idimx )
   yspan = indgen(7) + yc - 3
   yvalid = where( yspan ge 0 and yspan lt idimy )
   xw = total(im_orig(min(xspan(xvalid)):max(xspan(xvalid)), $
                      min(yspan(yvalid)):max(yspan(yvalid))),2)
   peak_x = total(xw*xspan(xvalid)) / total(xw)
   yw = total(im_orig(min(xspan(xvalid)):max(xspan(xvalid)), $
                      min(yspan(yvalid)):max(yspan(yvalid))),1)
   peak_y = total(yw*yspan(yvalid)) / total(yw)

   print,'Pos pix centr  : ', xc, yc, peak_x, peak_y
   printf,lun,'Pos pix centr  : ', xc, yc, peak_x, peak_y
   k_val = k_image(xc,yc)

   ; Derive the RA and Dec

   ra_src = peak_x
   dec = peak_y
   skypos_fits, dol_of_image, peak_x+1, peak_y+1, ra_src, dec_src,/sil
   print,'RADEC : ', ra_src, dec_src
   printf,lun,'RADEC : ', ra_src, dec_src


   ; Check with s_image

   s_val = s_image(xc,yc)
   print,'K and Chi2r :                  = ', k_val, s_val
   printf,lun,'K and Chi2r :                  = ', k_val, s_val

   if k_val gt k_accept_limit and s_val lt 1.5 then begin
      ; Store results in array to be returned
      res_array(0,n_accepted_srcs) = peak_x
      res_array(1,n_accepted_srcs) = peak_y
      res_array(2,n_accepted_srcs) = ra_src
      res_array(3,n_accepted_srcs) = dec_src
      res_array(4,n_accepted_srcs) = k_val
      res_array(5,n_accepted_srcs) = s_val

      n_accepted_srcs = n_accepted_srcs + 1
      if keyword_set(numjemx) then numid = numjemx else numid = n_candidate_srcs
      printf,lun,numid,peak_x,peak_y,ra_src,dec_src, k_val,s_val,$
         format='(i4,2f8.2,2f9.3,f12.3,f10.4)'
      print,numid,peak_x,peak_y,ra_src,dec_src, k_val,s_val,$
         format='(i4,2f8.2,2f9.3,f12.3,f10.4)'

      im_flat_disp = circle(im_flat_disp,peak_x,peak_y,7.,val=im_flat_max)
      k_image_disp = circle(k_image_disp,peak_x,peak_y,7.,val=k_image_max)

   endif


    k_image(xc-3:xc+3, yc-3:yc+3) = 0


endwhile


free_lun,lun

if display then disp,im_flat_disp,fac,ma=400,tit=dol_of_image+' Flatfielded'; Left window
if display then disp,k_image_disp,fac,ma=400,pa=1,tit=dol_of_image+' K-image'; Right window

print
print,'Output to '+out_file_name
print,'Number of accepted sources: ',n_accepted_srcs

if n_accepted_srcs gt 0 then begin
   return, res_array(*,0:n_accepted_srcs-1)
endif else return,0


end

