function find_mosaic_sources, image_definition, thr=thr, blevel=blevel, $
             nodisp=nodisp, maxnum=maxnum, savres=savres,$
             bkg_definition=bkg_definition,swid=swid,numjemx=numjemx,$
             radiuslimit=radiuslimit, s_psf=s_psf, s_flat=s_flat, $
             noflat=noflat,nofits=nofits
;
; Find the sources in the an image where the PSF is known
;
; Return an array with source positions, peak size, and detection significance
;
; 'image_definition' is either 1) a DOL i.e. a FITS file with extension number appended
;                           or 2) an array
;
; thr:
;
; If the keyword 'thr' has been set then only the results from
; sources with greater significance will be saved;
; the default value is 9
;
; nodisp:
;
; If the keyword 'nodisp' (no display) has been set then the display
; is suppressed
; 
; noflat
; suppress flatfielding
;
; maxnum:
;
; The optional keyword 'maxnum' sets the maximum number of sources to
; look for.
;
; savres:
;
; The keyword 'savres' will cause the intermediate image arrays to
; be saved (overridden the next time 'find_sources' is run with this
; option).
;
; bkg_definition:
;
; If a background is to be subtracted (instead of a smoothed version of
; thi image itself) then it must be given with keyword 'bkg_definition'
;  - either a DOL or array
;
; swid:
;
; The 'swid' keyword is a 12 character string with actual SWID
; If set the position will be calculated according to midisky
; or refined_recon conventions
;
; s_psf:
;
; Sigma [pixel] of PSF to be used. Default value is 1.5
;
; s_flat:
;
; Sigma [pixel] for flat fielding proces. Default value is
; 3 * s_psf
;

if n_params() eq 0 then begin
   print,'res = FIND_MOSAIC_SOURCES(image_definition $'
   print,'  [,thr=][,nodisp=][,maxnum=][,savres=]$'
   print,'  [,bkg_definition=][,swid=][,s_psf=][,s_flat=][,/noflat])'
   print,'res is an array res(4,nsources_found) with position, peaksize'
   print,'     and detection level for each accepted source'
   return, -1
endif

out_file_name = get_next_filename('fms_????.scm')
strnum = strmid(out_file_name,4,4)

; Define radius limit

rlim = 1.e20
if keyword_set(radiuslimit) then rlim = radiuslimit

; Define sigma PSF

if not keyword_set(s_psf) then s_psf = 1.5

; Define sigma flat

if not keyword_set(s_flat) then s_flat = 3.*s_psf


; Is parameter for image definition and Array or a DOL ?

sz = size(image_definition)
type = sz(sz(0)+1)
ndim = sz(0)
im_is_dol = 0
im_is_array = 0
if ndim eq 0 and type eq 7 then im_is_dol = 1
if ndim eq 2 and type ne 7 then im_is_array = 1
if im_is_dol+im_is_array eq 0 then begin
   print,'Illegal image definition'
   return, -1
endif

; Set Boolean for saving intermediate results

sav = 0
if keyword_set(savres) then sav = 1

if im_is_dol then begin

   ; Separate file name and extension number

   get_exten_no, image_definition, im_file_name, im_exten_no
   im_dol = image_definition

   ; Get the data

   im_orig = readfits(im_file_name, hdr_orig, exten_no=im_exten_no)
   wnan = wherenan(im_orig, nwnan)
   if nwnan gt 0 then im_orig(wnan) = 0.0
   ;** im_orig = im_orig * 1.e-6

   ;  Analyse the FITS file to see if K_IMAGE and CHI2R_IMAGE exist

   num_ext = nfits_extens( im_file_name )

   k_ext = -1
   chi2r_ext = -1
   for i = 1, num_ext do begin
      hdr = headfits( im_file_name, ext=i)
      imatype = fxpar(hdr,'IMATYPE')
      si = size(imatype)
      if si(0) eq 0 then begin
	 if si(1) eq 7 then begin
	    if imatype eq 'K_IMAGE' then k_ext = i
	    if imatype eq 'CHI2R_IMAGE' then chi2r_ext = i
         endif
      endif
   endfor
   if k_ext lt 0 then begin
      print,'No K_IMAGE extension'
   endif else begin
      print,'K_IMAGE in extension# '+stri(k_ext)
   endelse
   if chi2r_ext lt 0 then begin
      print,'No CHI2R_IMAGE extension'
   endif else begin
      print,'CHI2R_IMAGE in extension# '+stri(chi2r_ext)
   endelse

endif else begin

   im_orig = image_definition
   wnan = wherenan(im_orig, nwnan)
   if nwnan gt 0 then im_orig(wnan) = 0.0

   ; Save input array

   im_save_file = 'fms_i'+strnum+'.fits'
   writefits,im_save_file,im_orig
   im_dol = im_save_file+'[0]'

endelse

openw,lun,out_file_name,/get_lun

; Interpret keywords

;   'nodisp'
display = 1
if keyword_set(nodisp) then display = 0

;  'thr'
threshold = 3.0    ; default value
if keyword_set(thr) then threshold = thr

;  'blevel'
if not keyword_set(blevel) then blevel = 0.1

; 'maxnum'
maxnum_sources = 3
if keyword_set(maxnum) then maxnum_sources = maxnum
res_array = fltarr(6,maxnum_sources) ; prepare for returning the results

; Subtract a given background?

use_bkg = 0
if keyword_set(bkg_definition) then begin
   use_bkg = 1

   ; Array or DOL ?

   sz = size(bkg_definition)
   type = sz(sz(0)+1)
   ndim = sz(0)
   bkg_is_dol = 0
   bkg_is_array = 0
   if ndim eq 0 and type eq 7 then bkg_is_dol = 1
   if ndim eq 2 and type ne 7 then bkg_is_array = 1
   if bkg_is_dol+bkg_is_array eq 0 then begin
      print,'Illegal bkg definition'
      return, -1
   endif

   if bkg_is_dol then begin

      ; Separate file name and extension number

      get_exten_no, bkg_definition, bkg_file_name, bkg_exten_no
      bkg_dol = bkg_definition

      ; Get the data

      bkg_array = readfits(bkg_file_name, hdr_bkg, exten_no=bkg_exten_no)
      wnan = wherenan(bkg_array, nwnan)
      if nwnan gt 0 then bkg_array(wnan) = 0.0

   endif else begin

      bkg_array = bkg_definition
      wnan = wherenan(bkg_array, nwnan)
      if nwnan gt 0 then bkg_array(wnan) = 0.0
      bkg_save_file = 'fms_b'+strnum+'.fits'
      writefits,bkg_save_file,bkg_array
      bkg_dol = bkg_save_file+'[0]'

   endelse

endif

; Start output til log-file

printf,lun,'// Result of ''find_mosaic_sources'' '+systime()
printf,lun,'// im_dol = '+im_dol
if use_bkg then begin
   printf,lun,'// bkg_dol = '+bkg_dol
endif else begin
   printf,lun,'// No external background used'
endelse
printf,lun,'// Significance Threshold = ', threshold
printf,lun,'// maxnum_sources = ', maxnum_sources
printf,lun,'// s_psf = ', s_psf
if keyword_set(noflat) then begin
   printf,lun,'// No flatfielding'
endif else begin
   printf,lun,'// s_flat = ', s_flat
endelse
if keyword_set(swid) then begin
   printf,lun,'// Coordinates calculated as NL image with:'
   printf,lun,'// SWID = ', swid
endif

; Subtract background according to chosen method

if use_bkg then begin

   im_flat = bkg_subtract(im_orig, bck_array,kres=krim)

endif else begin

   ; Do a standard flatfielding

   if not keyword_set(noflat) then begin
      im_flat = flatfield(im_orig, sigma=s_flat, type=1)
   endif else im_flat = im_orig

endelse

if display then disp,im_flat,fac,ma=400,tit=im_dol+' Flatfielded'; Left window

flat_save_file = 'fms_f'+strnum+'.fits'
if not keyword_set(nofits) then writefits,flat_save_file,im_flat

;  Define the PSF

psf = def_psf(11,[0.,0.],s_psf)

;  Produce K_IMAGE if not already in file

if k_ext ge 0 then begin
   k_image = readfits(im_file_name,ext=k_ext)
   greg = where(k_image ne 0.0)
endif
if chi2r_ext ge 0 then s_image = readfits(im_file_name,ext=chi2r_ext)

if k_ext lt 0 then begin

   ;  Slide PSF kernel across image

   print,'Start sliding ...'
   slide, im_flat, psf, k_image, s_image, greg=greg, blevel=blevel
   ;                             (greg = good region)

endif

k_save_file = 'fms_k'+strnum+'.fits'
if not keyword_set(nofits) then writefits,k_save_file,k_image

s_save_file = 'fms_s'+strnum+'.fits'
if not keyword_set(nofits) then writefits,s_save_file,s_image

; Define the distribution of pixel values in the K image

histo,k_image(greg), hist_k, xk,nbins=200
; Accumulate to find 99.6% point
acc = accumulate(hist_k, /norm)
klim = interpol(xk,acc,0.996)
if display then begin
   window,2,title=im_dol
   plot,xk,hist_k,ps=10
   maxy = max(hist_k)
   draw_arrow, klim, 0.5*maxy, klim, 0.1*maxy,0,0.05,0.07
endif

;** yfit = gaussfit(xk,hist_k,coeff,nterms=3)
;** oplot,xk,yfit
;** k_accept_limit = threshold*coeff(2) + coeff(1)
;** printf,lun,'// k_fit_ampl = ', coeff(0)
;** printf,lun,'// k_fit_peakpos = ', coeff(1)
;** printf,lun,'// k_fit_sigma = ', coeff(2)

k_accept_limit = threshold*klim
printf,lun,'// klim = ', klim
printf,lun,'// k_accept_limit = ', k_accept_limit

; Here all the intermediate results are saved - if requested

if sav then begin
   save,im_orig,file='find_mosaic_sources.im_orig'
   save,im_flat,file='find_mosaic_sources.im_flat'
   save,psf,file='find_mosaic_sources.psf'
   save,k_image,file='find_mosaic_sources.k_image'
   save,s_image,file='find_mosaic_sources.s_image'
endif


if display then disp,k_image,fac,ma=400,pa=1,tit=im_dol+' K-image'; Right window

printf,lun,'//'
printf,lun,'// Column 1: local source number or JEMX number'
printf,lun,'// Column 2: image x-value in pixels'
printf,lun,'// Column 3: image y-value in pixels'
printf,lun,'// Column 4: RA of src (deg) or same as col 2'
printf,lun,'// Column 5: DEC of src (deg) or same as col 3'
printf,lun,'// Column 6: K value'
printf,lun,'// Column 7: S value'
printf,lun,'//'

;--------------------------------------------------------------
;  Loop for searching for sources. The candidates are the maxima
;           in the K matrix
;--------------------------------------------------------------

im_flat_max = max(im_flat)
k_image_max = max(k_image)
im_flat_disp = im_flat
k_image_disp = k_image

n_candidate_srcs = 0
n_accepted_srcs = 0
while n_candidate_srcs lt maxnum_sources do begin

; Find the maximum value pixel in the K image

   maxim,k_image,mmmx,xc,yc,/silent
   peak_x = float(xc)
   peak_y = float(yc)

   n_candidate_srcs = n_candidate_srcs + 1


   print,'Peak number ', n_candidate_srcs,'  ----------------------------'
   printf,lun,'Peak number ', n_candidate_srcs,'  ----------------------------'
   print,'Peak pos pixels            = ', xc, yc
   printf,lun,'Peak pos pixels            = ', xc, yc

   ;  Determine better position by centroiding

   xw = total(im_orig(xc-3:xc+3,yc-3:yc+3),2)
   peak_x = total(xw*findgen(7)) / total(xw) + xc - 3
   yw = total(im_orig(xc-3:xc+3,yc-3:yc+3),1)
   peak_y = total(yw*findgen(7)) / total(yw) + yc - 3

   print,'Peak position              = ', peak_x, peak_y
   printf,lun,'Peak position              = ', peak_x, peak_y
   k_val = k_image(xc,yc)

   ; Derive the RA and Dec if input is FITS file

   ra_src = peak_x
   dec = peak_y
   if im_is_dol then begin
      skypos_fits, im_dol, peak_x+1, peak_y+1, ra_src, dec_src,/sil
      print,'Right Ascension           = ', ra_src
      printf,lun,'Right Ascension           = ', ra_src
      print,'Declination               = ', dec_src
      printf,lun,'Declination               = ', dec_src
   endif

   print,'K value                    = ', k_val
   printf,lun,'K value                    = ', k_val

   ; Check with s_image

   s_val = s_image(xc,yc)
   print,'S value                    = ', s_val
   printf,lun,'S value                    = ', s_val

   if k_val gt k_accept_limit and s_val lt 1.5 then begin
      ; Store results in array to be returned
      res_array(0,n_accepted_srcs) = peak_x
      res_array(1,n_accepted_srcs) = peak_y
      res_array(2,n_accepted_srcs) = ra_src
      res_array(3,n_accepted_srcs) = dec_src
      res_array(4,n_accepted_srcs) = k_val
      res_array(5,n_accepted_srcs) = s_val

      n_accepted_srcs = n_accepted_srcs + 1
      if keyword_set(numjemx) then numid = numjemx else numid = n_candidate_srcs
      printf,lun,numid,peak_x,peak_y,ra_src,dec_src, k_val,s_val,$
         format='(i4,2f8.2,2f9.3,f10.3,f10.4)'

      im_flat_disp = circle(im_flat_disp,peak_x,peak_y,7.,val=im_flat_max)
      k_image_disp = circle(k_image_disp,peak_x,peak_y,7.,val=k_image_max)

   endif


    k_image(xc-3:xc+3, yc-3:yc+3) = 0


endwhile


free_lun,lun

if display then disp,im_flat_disp,fac,ma=400,tit=im_dol+' Flatfielded'; Left window
if display then disp,k_image_disp,fac,ma=400,pa=1,tit=im_dol+' K-image'; Right window

print
print,'Output to '+out_file_name
print,'Number of accepted sources: ',n_accepted_srcs

if n_accepted_srcs gt 0 then begin
   return, res_array(*,0:n_accepted_srcs-1)
endif else return,0


end

