function find_sources, image_definition, thr=thr,$
             nodisp=nodisp, maxnum=maxnum, savres=savres,$
             bkg_definition=bkg_definition,swid=swid,numjemx=numjemx,$
             radiuslimit=radiuslimit, nofiles=nofiles
;
; Find the surces in the output file of 'j_ima_basic_recon' (JMXi-RSTI-IMA)
;
; Return an array with source positions, peak size, and detection significance
;
; 'image_definition' is either 1) a DOL i.e. a FITS file with extension number appended
;                           or 2) an array
;
; thr:
;
; If the keyword 'thr' has been set then only the results from
; sources with greater significance will be saved;
; the default value is 9
;
; nodisp:
;
; If the keyword 'nodisp' (no display) has been set then the display
; is suppressed
;
; nofiles:
; If the keyword 'nofiles' has been set then no files with extra information
; are written
;
; maxnum:
;
; The optional keyword 'maxnum' sets the maximum number of sources to
; look for.
;
; savres:
;
; The keyword 'savres' will cause the intermediate image arrays to
; be saved (overridden the next time 'find_sources' is run with this
; option).
;
; bkg_definition:
;
; If a background is to be subtracted (instead of a smoothed version of
; thi image itself) then it must be given with keyword 'bkg_definition'
;  - either a DOL or array
;
; swid:
;
; The 'swid' keyword is a 12 character string with actual SWID
; If set the position will be calculated according to midisky
; or refined_recon conventions

if n_params() eq 0 then begin
   print,'res = FIND_SOURCES(image_definition $'
   print,'  [,thr=][,/nodisp][,maxnum=][,savres=]$'
   print,'  [,bkg_definition=][,swid=][,/nofiles])'
   print,'res is an array res(4,nsources_found) with position, peaksize'
   print,'     and detection level for each accepted source'
   return, -1
endif

; Define radius limit

rlim = 1.e20
if keyword_set(radiuslimit) then rlim = radiuslimit

; Array or DOL ?

sz = size(image_definition)
type = sz(sz(0)+1)
ndim = sz(0)
im_is_dol = 0
im_is_array = 0
if ndim eq 0 and type eq 7 then im_is_dol = 1
if ndim eq 2 and type ne 7 then im_is_array = 1
if im_is_dol+im_is_array eq 0 then begin
   print,'Illegal image definition'
   return, -1
endif

; Set Boolean for saving intermediate results

sav = 0
if keyword_set(savres) then sav = 1
nof = 0
if keyword_set(nofiles) then nof = 1

if im_is_dol then begin

   ; Separate file name and extension number

   get_exten_no, image_definition, im_file_name, im_exten_no
   im_dol = image_definition

   ; Get the data

   im_orig = readfits(im_file_name, hdr_orig, exten_no=im_exten_no)

endif else begin

   im_orig = image_definition

   ; Save input array

   im_save_file = get_next_filename('fs_i?????.fits')
   if not nof then writefits,im_save_file,im_orig
   im_dol = im_save_file+'[0]'

endelse

szimorig = size(im_orig)

; Define output file name

if not nof then begin

   out_file_name = get_next_filename('fs_r?????.fs')
   imb_file_name = get_next_filename('fs_imb?????.fits')
   sig_file_name = get_next_filename('fs_sig?????.fits')

   openw,lun,out_file_name,/get_lun

endif

; Interpret keywords

;   'nodisp'
display = 1
if keyword_set(nodisp) then display = 0

;  'thr'
threshold = 9.0    ; default value
if keyword_set(thr) then threshold = thr

; 'maxnum'
maxnum_sources = 20
if keyword_set(maxnum) then maxnum_sources = maxnum
res_array = fltarr(4,maxnum_sources) ; prepare for returning the results

; Subtract a given background?

use_bkg = 0
if keyword_set(bkg_definition) then begin
   use_bkg = 1

   ; Array or DOL ?

   sz = size(bkg_definition)
   type = sz(sz(0)+1)
   ndim = sz(0)
   bkg_is_dol = 0
   bkg_is_array = 0
   if ndim eq 0 and type eq 7 then bkg_is_dol = 1
   if ndim eq 2 and type ne 7 then bkg_is_array = 1
   if bkg_is_dol+bkg_is_array eq 0 then begin
      print,'Illegal bkg definition'
      return, -1
   endif

   if bkg_is_dol then begin

      ; Separate file name and extension number

      get_exten_no, bkg_definition, bkg_file_name, bkg_exten_no
      bkg_dol = bkg_definition

      ; Get the data

      bkg_array = readfits(bkg_file_name, hdr_bkg, exten_no=bkg_exten_no)

   endif else begin

      bkg_array = bkg_definition
      bkg_save_file = get_next_filename('fs_b?????.fits')
      if not nof then writefits,bkg_save_file,bkg_array
      bkg_dol = bkg_save_file+'[0]'

   endelse

endif

; Start output til log-file

if not nof then begin

   printf,lun,'// Result of ''find_sources'' '+systime()
   printf,lun,'// im_dol = '+im_dol
   if use_bkg then begin
      printf,lun,'// bkg_dol = '+bkg_dol
   endif else begin
      printf,lun,'// No external background used'
   endelse
   printf,lun,'// Significance Threshold = ', threshold
   printf,lun,'// maxnum_sources = ', maxnum_sources
   printf,lun,'// imb_file = '+imb_file_name
   printf,lun,'// sig_file = '+sig_file_name
   if keyword_set(swid) then begin
      printf,lun,'// Coordinates calculated as NL image with:'
      printf,lun,'// SWID = ', swid
   endif

endif


; First step is to remove particularly strong sources

im = rem_extremes(im_orig,nsig=3)

; Subtract background according to chosen method

if use_bkg then begin

   bck = bkg_array

endif else begin

   ; Determine the background and the noise in the background
   ; based on this 'no source' image

   parm = 2.5 ; Standard Dev for convolution kernel
              ; Previous value (i.e. before 000727) was 4.0 (with ks=25)
   bck = gauss_convol_2d(im, parm, val='s', ks=15)

endelse

; Get the difference between the de-sourced image (im) and the smooth
;    background (bck)

dif = bkg_subtract(im, bck,kres=krim)

; Find the level of the fluctuations in the image

rim = local_rms(dif,step=5,hbs=8)

; Background subtracted image; use same factor as derived above

imb = bkg_subtract(im_orig, bck, fac=krim, kres=kimb)
if not nof then writefits,imb_file_name,imb


; let im_4pix be the image with 4 pixels combined for source finding
; making sure that sources divided over several pixels can be found

im_4pix = imb+ shift(imb,0,1)+shift(imb,1,0)+shift(imb,1,1)

; The significance matrix is now

sig = im_4pix / ( (2*rim)>1 )
if not nof then writefits,sig_file_name,sig

; Here all the intermediate results are saved - if requested

if sav then begin
   save,im_orig,file='find_sources.im_orig'
   save,sig,file='find_sources.sig'
   save,im,file='find_sources.im'
   save,bck,file='find_sources.bck'
   save,dif,file='find_sources.dif'
   save,rim,file='find_sources.rim'
   save,im_4pix,file='find_sources.im_4pix'
   save,imb,file='find_sources.imb'
endif

; Compose name of FITS file for saving significance matrix

;del kot = reverse(str_sep(file_name,'/'))
;del kot0 = kot(0)
;del strput, kot0, 's'
;del kot(0) = kot0
;del sig_file_name = collate_path(reverse(kot))
;del fxaddpar, hdr_orig, 'COMMENT',' Significance matrix from find_sources'
;del fxwrite,sig_file_name(0),hdr_orig,sig


if display then disp,imb,fac,ma=612,tit='Image - Bkg'; Left window
if display then disp,sig,fac,ma=612,pa=1,tit='Significance'; Right window

if not nof then begin

   printf,lun,'// bkg_subtract_k_rim = ', krim
   printf,lun,'// bkg_subtract_k_imb = ', kimb
   printf,lun,'//'
   printf,lun,'// Column 1: local source number or JEMX number'
   printf,lun,'// Column 2: image x-value in pixels'
   printf,lun,'// Column 3: image y-value in pixels'
   printf,lun,'// Column 4: RA of src (deg) or same as col 2'
   printf,lun,'// Column 5: DEC of src (deg) or same as col 3'
   printf,lun,'// Column 6: Peak size'
   printf,lun,'// Column 7: Background level'
   printf,lun,'// Column 8: Local RMS'
   printf,lun,'// Column 9: Detection level'
   printf,lun,'//'

endif


;--------------------------------------------------------------
;  Loop for searching for sources. The candidates are the maxima
;           in the significance matrix
;--------------------------------------------------------------

n_candidate_srcs = 0
n_accepted_srcs = 0
while n_candidate_srcs lt maxnum_sources do begin

; Find the maximum value pixel in the significance matrix

   maxim,sig,mmmx,xc,yc,/silent
   if sqrt((1.*xc-szimorig(1))^2 + (1.*yc-szimorig(2))^2) gt rlim then begin
      sig(xc,yc) = 0
      ;  Insert here more zeroing of pixels if required
      ;sig(xc-1:xc+1,yc-1:yc+1) = 0
      continue
   endif

   n_candidate_srcs = n_candidate_srcs + 1

; Define region for background determination
   xb = xc + 8
   yb = yc + 8
   dx1 = abs(xb-xc)
   dx2 = dx1
   dy1 = abs(yb-yc)
   dy2 = dy1

   ; Adjust center to avoid index errors

   if xc + dx2 ge szimorig(1) then dx2 = szimorig(1)-1-xc
   if xc - dx1 lt   0   then dx1 = xc
   if yc + dy2 ge szimorig(2) then dy2 = szimorig(2)-1-yc
   if yc - dy1 lt   0   then dy1 = yc

   if display then begin
      ; Show selection in image (i.e. on the screen)

      wset,0
      bar = intarr(fac*(dx1+dx2),1)
      tv,bar+100,fac*(xc-dx1),fac*(yc+dy2)
      tv,bar+100,fac*(xc-dx1),fac*(yc-dy1)
      bar = intarr(1, fac*(dy1+dy2))
      tv,bar+100,fac*(xc-dx1),fac*(yc-dy1)
      tv,bar+100,fac*(xc+dx2),fac*(yc-dy1)
      wait,3
   endif

   ; Find the peak in the selected area in the background
   ; subtracted image

   im_region = imb(xc-dx1:xc+dx2, yc-dy1:yc+dy2)
   szim = size(im_region)
   aux = im_region*0
   maxim,im_region,mx,xm,ym,/silent

   ; Determine which neighbor pixels contribute

   z = fltarr(2,2) - 1.e30 ; start with this number to avoid
                           ; problems near the edge
   for p = 0, 1 do begin
      for q = 0, 1 do begin
         if xm-p ge 0 and xm+1-p lt szim(1) and ym-q ge 0 $
           and ym+1-q lt szim(2) then $
            z(p,q) = total(im_region(xm-p:xm+1-p, ym-q:ym+1-q))
      endfor
   endfor
   maxim,z,mzx,p,q,/silent
   peak = im_region(xm-p:xm+1-p, ym-q:ym+1-q)
   posx = (peak(1,0)+peak(1,1))/mzx
   posy = (peak(0,1)+peak(1,1))/mzx
   aux(xm-p:xm+1-p, ym-q:ym+1-q) = 1
   i_outside = where(aux eq 0)
   i_inside  = where(aux eq 1)
   bkg_level = avg(im_region(i_outside))
   bkg_sigma = rms(im_region(i_outside))

   ; show result

   peak_x = xc - dx1 + xm - p + posx
   peak_y = yc - dy1 + ym - q + posy

   ra_src = peak_x
   dec_src = peak_y
   ;pos_conv,sat_y,sat_z,to='sat' ; convert to satellite positions
   if keyword_set(swid) then begin
      ref_recon_fs2skycoord, swid, peak_x, peak_y, ra_src, dec_src
   endif

   peak_sz = mzx - 4*bkg_level
   print,'Peak number ', n_candidate_srcs,'  ----------------------------'
   print,'Peak position              = ', peak_x, peak_y
   print,'Peak size (bkg subtracted) = ', peak_sz
   print,'Bkg_level                  = ', bkg_level
   print,'Bkg RMS                    = ', bkg_sigma
   det_lev = peak_sz / bkg_sigma
   print,'Detection level            = ',det_lev
   if det_lev gt threshold then begin
      ; Store results in array to be returned
      res_array(0,n_accepted_srcs) = peak_x
      res_array(1,n_accepted_srcs) = peak_y
      res_array(2,n_accepted_srcs) = peak_sz
      res_array(3,n_accepted_srcs) = det_lev

      n_accepted_srcs = n_accepted_srcs + 1
      if keyword_set(numjemx) then numid = numjemx else numid = n_candidate_srcs
      if not nof then printf,lun,numid,peak_x,peak_y,ra_src,dec_src, peak_sz,bkg_level,$
         bkg_sigma,det_lev,format='(i4,2f8.2,2f9.3,f10.1,f10.1,f10.2,f7.2)'
   endif

   if det_lev lt 3 then goto, slut

;*      new_value = max(im_orig)
      new_value = 0
;*      print,'First removal : ', xc-dx1+xm-p, xc-dx1+xm-p+1
;*      print,'              : ', yc-dy1+ym-q, yc-dy1+ym-q+1
      imb(xc-dx1+xm-p:xc-dx1+xm-p+1, yc-dy1+ym-q:yc-dy1+ym-q+1) $
                                                         = new_value
      sig(max([xc-1,0]):min([xc+1,szimorig(1)-1]), $
         max([yc-1,0]):min([yc+1,szimorig(2)-1])) = new_value

      if display then disp,imb,fac,ma=612,tit='Image - Bkg'
      if display then disp,sig,fac,ma=612,pa=1,tit='Significance'

endwhile

slut:


if not nof then free_lun,lun
if display then wdelete,0,1

if not nof then begin
   print
   print,'Output to '+out_file_name
endif

if n_accepted_srcs gt 0 then begin
   return, res_array(*,0:n_accepted_srcs-1)
endif else return, 0

end

