function find_strongest_source, image_definition, thr=thr, blevel=blevel, $
             nodisp=nodisp, savres=savres,$
             radiuslimit=radiuslimit, s_psf=s_psf, s_flat=s_flat, $
             noflat=noflat,nofits=nofits,nolog=nolog
;
; Find the strongest source in the an image where the PSF is approximately known
;
; Return an array with source position, peak size, and detection significance
;
; 'image_definition' is either 1) a DOL i.e. a FITS file with extension number appended
;                           or 2) an array
;
; thr:
;
; If the keyword 'thr' has been set then only the results from
; sources with greater significance will be saved;
; the default value is 9
;
; nodisp:
;
; If the keyword 'nodisp' (no display) has been set then the display
; is suppressed
; 
; noflat
; suppress flatfielding
;
; savres:
;
; The keyword 'savres' will cause the intermediate image arrays to
; be saved (overridden the next time 'find_sources' is run with this
; option).
;
; s_psf:
;
; Sigma [pixel] of PSF to be used. Default value is 1.5
;
; s_flat:
;
; Sigma [pixel] for flat fielding proces. Default value is
; 3 * s_psf
;

if n_params() eq 0 then begin
   print,'res = FIND_STRONGEST_SOURCE(image_definition $'
   print,'  [,thr=][,nodisp=][,savres=]$'
   print,'  [,s_psf=][,s_flat=][,/noflat][,/nofits][,/nolog])'
   print,'res is an array res(4) with position, peaksize'
   print,'     and detection level for strongest source'
   return, -1
endif

out_file_name = get_next_filename('fss_????.scm')
strnum = strmid(out_file_name,4,4)

; Define radius limit

if not keyword_set(radiuslimit) then radiuslimit = 1.e30

; Define sigma PSF

if not keyword_set(s_psf) then s_psf = 1.5

; Define sigma flat

if not keyword_set(s_flat) then s_flat = 3.*s_psf


; Is parameter for image definition and Array or a DOL ?

sz = size(image_definition)
type = sz(sz(0)+1)
ndim = sz(0)
im_is_dol = 0
im_is_array = 0
if ndim eq 0 and type eq 7 then im_is_dol = 1
if ndim eq 2 and type ne 7 then im_is_array = 1
if im_is_dol+im_is_array eq 0 then begin
   print,'Illegal image definition'
   return, -1
endif

; Set Boolean for saving intermediate results

sav = 0
if keyword_set(savres) then sav = 1

if im_is_dol then begin

   ; Separate file name and extension number

   get_exten_no, image_definition, im_file_name, im_exten_no
   im_dol = image_definition

   ; Get the data

   im_orig = readfits(im_file_name, hdr_orig, exten_no=im_exten_no)
   szr = size(im_orig)
   if szr(0) ne 2 then begin
      print,'%FIND_STRONGEST_SOURCE error, could not read 2D image: '+image_definition
      return, -1
   endif
   wnan = wherenan(im_orig, nwnan)
   if nwnan gt 0 then im_orig(wnan) = 0.0

endif else begin

   im_orig = image_definition
   wnan = wherenan(im_orig, nwnan)
   if nwnan gt 0 then im_orig(wnan) = 0.0

   ; Save input array

   if not keyword_set(nofits) then begin
      im_save_file = 'fss_i'+strnum+'.fits'
      writefits,im_save_file,im_orig
      im_dol = im_save_file+'[0]'
   endif

endelse

if not keyword_set(nolog) then $
      openw,lun,out_file_name,/get_lun

; Interpret keywords

;   'nodisp'
display = 1
if keyword_set(nodisp) then display = 0

;  'thr'
threshold = 3.0    ; default value
if keyword_set(thr) then threshold = thr

;  'blevel'
if not keyword_set(blevel) then blevel = 0.1

res_array = fltarr(8) ; prepare for returning the results


if not keyword_set(nolog) then begin

   ; Start output til log-file

   printf,lun,'// Result of ''find_strongest_source'' '+systime()
   if not keyword_set(nofits) then $
         printf,lun,'// im_dol = '+im_dol
   printf,lun,'// Significance Threshold = ', threshold
   printf,lun,'// s_psf = ', s_psf
   if keyword_set(noflat) then begin
      printf,lun,'// No flatfielding'
   endif else begin
      printf,lun,'// s_flat = ', s_flat
   endelse

endif

; Do a standard flatfielding

if not keyword_set(noflat) then begin
   im_flat = flatfield(im_orig, sigma=s_flat, type=1)
endif else im_flat = im_orig

if display then disp,im_flat,fac,ma=400,tit=im_dol+' Flatfielded'; Left window

flat_save_file = 'fss_f'+strnum+'.fits'
if not keyword_set(nofits) then writefits,flat_save_file,im_flat

;  Define the PSF

psf = def_psf(11,[0.,0.],s_psf)


; Here all the intermediate results are saved - if requested

if sav then begin
   save,im_orig,file='find_strongest_source.im_orig'
   save,im_flat,file='find_strongest_source.im_flat'
   save,psf,file='find_strongest_source.psf'
   save,k_image,file='find_strongest_source.k_image'
   save,s_image,file='find_strongest_source.s_image'
endif

if not keyword_set(nolog) then begin

   printf,lun,'//'
   printf,lun,'// Column 1: image x-value in pixels'
   printf,lun,'// Column 2: image y-value in pixels'
   printf,lun,'// Column 3: RA of src (deg) or same as col 2'
   printf,lun,'// Column 4: DEC of src (deg) or same as col 3'
   printf,lun,'// Column 5: Peak size'
   printf,lun,'// Column 6: Peak volume'
   printf,lun,'// Column 7: Peak sigma x'
   printf,lun,'// Column 8: Peak sigma y'
   printf,lun,'//'

endif

;--------------------------------------------------------------
;  Get source position
;--------------------------------------------------------------

im_flat_max = max(im_flat)
im_flat_disp = im_flat


; Find the maximum value pixel in the flat image

maxim,im_flat,mmmx,xc,yc,/silent
peak_x = float(xc)
peak_y = float(yc)

print,'Peak pos pixels            = ', xc, yc
if not keyword_set(nolog) then $
   printf,lun,'Peak pos pixels            = ', xc, yc

;  Make a gauss fit in two directions

gx = gaussfit(findgen(15),im_flat(xc-7:xc+7,yc),coefx,nterms=4)
peakx = coefx(0)
posx = coefx(1) + xc - 7
sigx = coefx(2)
blevx = coefx(3)

gy = gaussfit(findgen(15),im_flat(xc,yc-7:yc+7),coefy,nterms=4)
peaky = coefy(0)
posy = coefy(1) + yc - 7
sigy = coefy(2)
blevy = coefy(3)

peaksize = 0.5*(peakx + peaky)
volume = peaksize*2*!pi*sigx*sigy

if not keyword_set(nodisp) then begin
   window,1,xs=600,ys=800,xp=0
   !p.multi = [0,1,2,0]
   plot,findgen(15),im_flat(xc-7:xc+7,yc),ps=10,$
       title='Peak in x-direction'
   oplot,findgen(15),gx,li=2
   plot,findgen(15),im_flat(xc,yc-7:yc+7),ps=10,$
       title='Peak in y-direction'
   oplot,findgen(15),gy,li=2
   !p.multi = 0
endif

;  Determine better position by centroiding

q = 4
xw = total(im_flat(xc-q:xc+q,yc-q:yc+q),2)
peak_x = total(xw*findgen(2*q+1)) / total(xw) + xc - q
yw = total(im_flat(xc-q:xc+q,yc-q:yc+q),1)
peak_y = total(yw*findgen(2*q+1)) / total(yw) + yc - q

print,'CM Peak position           = ', peak_x, peak_y
if not keyword_set(nolog) then $
   printf,lun,'CM Peak position           = ', peak_x, peak_y
print,'Fit Peak position          = ', posx, posy
if not keyword_set(nolog) then $
   printf,lun,'Fit Peak position          = ', posx, posy
print,'Peak volume          = ', volume
if not keyword_set(nolog) then $
   printf,lun,'Peak volume          = ', volume

; Derive the RA and Dec if input is FITS file

ra_src = 0.0
dec_src = 0.0
if 1 eq 1 then begin
   peak_x = posx
   peak_y = posy
endif

if im_is_dol then begin
   skypos_fits, im_dol, peak_x+1, peak_y+1, ra_src, dec_src, dir='sky',/sil
   print,'Right Ascension           = ', ra_src
   if not keyword_set(nolog) then $
      printf,lun,'Right Ascension           = ', ra_src
   print,'Declination               = ', dec_src
   if not keyword_set(nolog) then $
      printf,lun,'Declination               = ', dec_src
endif

; Store results in array to be returned
res_array(0) = peak_x
res_array(1) = peak_y
res_array(2) = ra_src
res_array(3) = dec_src
res_array(4) = peaksize
res_array(5) = volume
res_array(6) = sigx
res_array(7) = sigy

if not keyword_set(nolog) then $
   printf,lun,peak_x,peak_y,ra_src,dec_src, peaksize, volume, $
         sigx, sigy,$
         format='(2f8.2,2f9.3,f9.2,f10.2,2f7.3)'

im_flat_disp = circle(im_flat_disp,peak_x,peak_y,7.,val=im_flat_max)



if not keyword_set(nolog) then free_lun,lun

if display then disp,im_flat_disp,fac,ma=400,tit=im_dol+$
   ' Flatfielded'; Left window

print
if not keyword_set(nolog) then print,'Output to '+out_file_name

return, res_array


end

