pro find_swid_radec, ra0, dec0, rad1, rad2, offax=offax, $
       rev=rev, tstart=tstart, tstop=tstop, list=list, $
       nof=nof, silent=silent, lst=lst, jemx=jemx, mod1=mod1, mod2=mod2
;
;  Find the INTEGRAL SWIDs where the pointing is less than 'rad1'
;  away from (ra0,dec0) or - if 'rad2' is given - greater than 'rad1'
;  and less than 'rad2' away
;
;   Ca. 2003-05/NJW
;  2004-10-22/NJW updated to produce jemx.lst style string array by
;      keyword 'lst'
;  2005-04-25/NJW updated with keywords 'jemx' and 'mode' for a selection
;                 on instrument number (1 or 2) and data mode
;
if n_params() lt 3 then begin
   print
   print,'Syntax: find_swid_radec, ra0, dec0, rad1[, rad2][,offax=] $'
   print,'        [,rev=][,tstart=][,tstop][,list=][,lst=][,/nof] $'
   print,'        [,jemx=][,mod1=][,mod2=]
   print
   print,'     If rad2 (in degrees) is not present then the pointings with'
   print,'     distance < rad1 (in degrees) will be presented'
   print,'     else the pointings with distance rad1 < dist < rad2'
   print,'     will be presented'
   print,'   Keywords:'
   print,'             offax:  Returns the off-axis angles'
   print,'             rev:    Either scalar integer or 2 element integer'
   print,'                     array with min and max revolution number'
   print,'                  or string such as ''45-48,56,70-72'''
   print,'             tstart: Time in IJD for start of interval'
   print,'             tstop:  Time in IJD for end of interval'
   print,'             list :  Returns list of SWIDs'
   print,'             lst  :  Returns list of SWIDs jemx.lst format'
   print,'             nof  :  Flag to avoid file writing'
   print,'             jemx :  1 or 2, default is both'
   print,'             mod1 :  Datataking mode for jemx1'
   print,'             mod2 :  Datataking mode for jemx2'
   print,'                     ''F'', ''R'', or ''S'', defaults to no selection'
   return
endif
;
;  Initialize
;
first = 1
revstrarr = strarr(20)
if keyword_set(tstart) then tbegin = tstart else tbegin = 0.0d0
if keyword_set(tstop) then tend = tstop else tend = 1.0d9
;
; Get list of pointing files
;
base = '/r6/jemx/pointings/'

revstrarr = strarr(20)

if keyword_set(rev) then begin
   szrev = size(rev)
   nrev = n_elements(rev)
   if nrev eq 1 and szrev(1) ne 7 then begin ;    we have a single integer
      ;  reformat to RRRR
      revstrarr(0) = string(rev,format='(i4.4)')
   endif else begin       ;  we have an array of integers or a string
      ;  prepare the syntax for a file search
      revstr = '{'
      if szrev(1) eq 7 then begin
         ;  change string representation to array of integers
         revnums = str2arr(rev)
      endif else begin
         ;  for an array of integers the two first ones span the range
         revnums = rev(0) + indgen(rev(1)-rev(0)+1)
      endelse

      ;   'revnums' is now an array of integers - produce 20 search strings
      ;   such as revstrarr(2) = '{0123,0645,1050}'
      ;   since 'file_search' cannot handle a very, very long search string

      nrevnums = n_elements(revnums)
      nrevpart = (nrevnums-1)/20+1   ; calculate number of revols per search string
      print,'nrevpart = ', nrevpart
      ndone = 0
      ;   form the 20 search strings
      for k = 0, 20-1 do begin
         revstrarr(k) = '{'
         for i = 0, nrevpart-1 do begin
            if ndone lt nrevnums then begin
               revstrarr(k) = revstrarr(k)+string(revnums(ndone),format='(i4.4)')
               if i lt nrevpart-1 and ndone lt nrevnums-1 then revstrarr(k) = revstrarr(k)+','
               ndone = ndone + 1
            endif
         endfor
         revstrarr(k) = revstrarr(k)+'}'
      endfor
   endelse
   frist = 1
   for k = 0, 20-1 do begin
      if strlen(revstrarr(k)) gt 2 then begin
         print,'revstrarr(k) = ', revstrarr(k)
         if frist eq 1 then begin
            pf_list = file_search(base+'pointings_'+revstrarr(k)+'p.dat',count=npf_list)
            frist = 0
         endif else begin
            pf_llll = file_search(base+'pointings_'+revstrarr(k)+'p.dat',count=npf_list)
            pf_list = [pf_list,pf_llll]
         endelse
         print,'Found ',npf_list,' files'
      endif
   endfor
endif else begin
   pf_list = file_search(base+'pointings_*p.dat',count=npf_list)
endelse

npf_list = n_elements(pf_list)

print,'In total ',n_elements(pf_list),' files'



if not keyword_set(nof) then begin

   ; Open output file

   outfilename = get_next_filename('find_swid_radec_???.dat')
   openw,lun,outfilename,/get_lun
   printf,lun,'//              '+ndate(2)
   printf,lun,'// ra0 = ', ra0
   printf,lun,'// dec0 = ', dec0
   if n_params() lt 4 then begin
      printf,lun,'// radius = ', rad1,' ; deg'
   endif else begin
      printf,lun,'// selection in annulus'
      printf,lun,'// rad1 = ', rad1,' ; deg'
      printf,lun,'// rad2 = ', rad2,' ; deg'
   endelse
   if keyword_set(tstart) then begin
      printf,lun,'// tstart = ', tstart
   endif
   if keyword_set(tstop) then begin
      printf,lun,'// tstop = ', tstop
   endif
   if keyword_set(rev) then begin
      if nrev eq 1 then begin
         printf,lun,'// rev_begin = ', rev
         printf,lun,'// rev_end   = ', rev
      endif else begin
         printf,lun,'// rev_begin = ', rev(0)
         printf,lun,'// rev_end   = ', rev(1)
      endelse
   endif

endif

ntotal = 0
for ipf_list = 0, npf_list-1 do begin

   rscolm,pf_list(ipf_list),swid,1,/str,/sil
   rscolm,pf_list(ipf_list),ra,2,/sil
   rscolm,pf_list(ipf_list),dec,3,/sil
   rscolm,pf_list(ipf_list),posangle,4,/sil
   rscolm,pf_list(ipf_list),ut,5,/str,/sil
   rscolm,pf_list(ipf_list),ijd,6,/sil
   rscolm,pf_list(ipf_list),telapse,7,/sil
   rscolm,pf_list(ipf_list),mode1,8,/str,/sil
   rscolm,pf_list(ipf_list),mode2,9,/str,/sil

   dista = arcdist(ra0, dec0, ra, dec, /deg)

   if n_params() eq 3 then begin

      w = where( dista lt rad1, nw)

   endif else begin

      w = where( dista gt rad1 and dista lt rad2, nw)

   endelse

   if nw gt 0 then begin

      if first then begin
         first = 0
         swid_all = swid(w)
         ra_all = ra(w)
         dec_all = dec(w)
         posangle_all = posangle(w)
         ut_all = ut(w)
         ijd_all = ijd(w)
         telapse_all = telapse(w)
         mode1_all = mode1(w)
         mode2_all = mode2(w)
         dist_all = dista(w)
      endif else begin
         swid_all = [swid_all,swid(w)]
         ra_all = [ra_all,ra(w)]
         dec_all = [dec_all,dec(w)]
         posangle_all = [posangle_all,posangle(w)]
         ut_all = [ut_all,ut(w)]
         ijd_all = [ijd_all,ijd(w)]
         telapse_all = [telapse_all,telapse(w)]
         mode1_all = [mode1_all,mode1(w)]
         mode2_all = [mode2_all,mode2(w)]
         dist_all = [dist_all,dista(w)]
      endelse

      ntotal = ntotal + nw

   endif

endfor

if not keyword_set(silent) then print,'Revol. and radius selection leaves '+stri(ntotal)+' SWIDs'

list = ''
lst = ''
if ntotal eq 0 then begin
   if not keyword_set(nof) then free_lun, lun
   return
endif

; Select on JEMX number if requested

if keyword_set(jemx) then begin
   if jemx eq 1 then begin
      w = where( mode1_all ne '*', nw )
   endif else begin
      w = where( mode2_all ne '*', nw )
   endelse
   if nw eq 0 then begin
      if not keyword_set(silent) then print,'No SWIDs found on selection for jemx unit'
      if not keyword_set(nof) then free_lun, lun
      return
   endif
   dist_all = dist_all(w)
   swid_all = swid_all(w)
   ra_all = ra_all(w)
   dec_all = dec_all(w)
   posangle_all = posangle_all(w)
   ut_all = ut_all(w)
   ijd_all = ijd_all(w)
   telapse_all = telapse_all(w)
   mode1_all = mode1_all(w)
   mode2_all = mode2_all(w)
   ntotal = nw
   if not keyword_set(silent) then print,'Instrument selection leaves '+stri(ntotal)+' SWIDs'
endif

; Select on JEMX 1 mode if requested

if keyword_set(mod1) then begin
   w = where( mode1_all eq mod1, nw )
   if nw eq 0 then begin
      if not keyword_set(silent) then print,'No SWIDs found on selection for JEMX 1 mode'
      if not keyword_set(nof) then free_lun, lun
      return
   endif
   dist_all = dist_all(w)
   swid_all = swid_all(w)
   ra_all = ra_all(w)
   dec_all = dec_all(w)
   posangle_all = posangle_all(w)
   ut_all = ut_all(w)
   ijd_all = ijd_all(w)
   telapse_all = telapse_all(w)
   mode1_all = mode1_all(w)
   mode2_all = mode2_all(w)
   ntotal = nw
   if not keyword_set(silent) then print,'JMX1 mode selection leaves '+stri(ntotal)+' SWIDs'
endif

; Select on JEMX 2 mode if requested

if keyword_set(mod2) then begin
   w = where( mode2_all eq mod2, nw )
   if nw eq 0 then begin
      if not keyword_set(silent) then print,'No SWIDs found on selection for JEMX 2 mode'
      if not keyword_set(nof) then free_lun, lun
      return
   endif
   dist_all = dist_all(w)
   swid_all = swid_all(w)
   ra_all = ra_all(w)
   dec_all = dec_all(w)
   posangle_all = posangle_all(w)
   ut_all = ut_all(w)
   ijd_all = ijd_all(w)
   telapse_all = telapse_all(w)
   mode1_all = mode1_all(w)
   mode2_all = mode2_all(w)
   ntotal = nw
   if not keyword_set(silent) then print,'JMX2 mode selection leaves '+stri(ntotal)+' SWIDs'
endif

offax = dist_all

noutput = 0
lfirst = 1
for i = 0, ntotal-1 do begin
   if ijd_all(i) gt tbegin and ijd_all(i) lt tend then begin

      if lfirst then begin
         list = [swid_all(i)]
         lfirst = 0
      endif else list = [list,swid_all(i)]
      if not keyword_set(nof) then $
       printf,lun,swid_all(i),ra_all(i),dec_all(i),posangle_all(i),$
         ut_all(i),ijd_all(i),telapse_all(i), $
         mode1_all(i),mode2_all(i),dist_all(i),$
         format='(a12,f10.4,f9.4,f9.3,a21,f10.4,f7.0,2x,a1,2x,a1,f8.2)'
      noutput = noutput + 1

   endif
endfor

if ntotal eq 0 then begin
   lst=''
   if not keyword_set(silent) then print,'No SWIDs found'
endif else begin
   lst = strarr(ntotal)
   for i = 0, ntotal-1 do begin
      lst(i) = './scw/'+strmid(swid_all(i),0,4)+'/'+swid_all(i)+'.001/swg.fits[1]'
   endfor
endelse

if not keyword_set(nof) then free_lun,lun

if not keyword_set(silent) then begin
if noutput eq 1 then begin
   if not keyword_set(silent) then print,stri(noutput)+' has been selected by time'
endif else begin
   if not keyword_set(silent) then print,stri(noutput)+' have been selected by time'
endelse
if not keyword_set(nof) then print,'Output is directed to ./'+outfilename

endif



end
