;
;  This file contains the necessary procedures to do a
;  2 Gauss functions fit with constant background on a spectrum
;  where the channel boundaries are given i.e. not necessarily
;  of same width
;
;  The use of the error function also removes the constraint to
;  have small channel widths as compared to the sigma of the
;  Gauss function
;
;   p(0)   sum peak 1
;   p(1)   mean peak 1
;   p(2)   sigma peak 1
;   p(3)   sum peak 2
;   p(4)   bkg level
;
;   mean peak 2 = mean peak 1  + 2.9 [namely 25.0 - 22.1]
;   sigma peak 2 = sigma peak 1
;

pro frssfit, pha, phaspec, p_res, phaspec_fit, itm=itm

; 'pha' is proportional to the energy
; 'phaspec' is the number of counts in each bin


; Locate the peak in the spectrum

peakval = max(phaspec, imax)

; Assume that this corresponds to 22.1 keV

factor = 22.1 / pha(imax)
print,'First conversion factor: ', factor

energies = pha * factor
e_bds = cent2bds(energies)

; Isolate the 22 keV peak

sel = where(energies gt 15. and energies lt 30., n_sel)

e_sel = energies(sel)
phaspec_sel = phaspec(sel)
e_bds_sel = e_bds(sel(0):max(sel)+1)

plot,e_sel,phaspec_sel,ps=10,tit='Isolated peak'

p_init = fltarr(5)
p_init(0) = total(phaspec_sel)
p_init(1) = 22.1
p_init(2) = 0.47 * sqrt(22.1) / 2.3548
p_init(3) = 0.2*total(phaspec_sel)
p_init(4) = 0.0
energy_dif = 2.9

p_res = g2fitb( e_bds_sel, phaspec_sel, p_init, energy_dif, phaspec_fit, itm=itm, /sil)

oplot,e_sel,phaspec_fit

print,' Resulting conversion factor = ', factor * 22.1 / p_res(1)

; Isolate the 8 keV peak

sel = where(energies gt 4. and energies lt 14., n_sel)

e_sel = energies(sel)
phaspec_sel = phaspec(sel)
e_bds_sel = e_bds(sel(0):max(sel)+1)

plot,e_sel,phaspec_sel,ps=10,tit='Isolated peak'

p_init = fltarr(5)
p_init(0) = total(phaspec_sel)
p_init(1) = 7.47
p_init(2) = 0.47 * sqrt(7.47) / 2.3548
p_init(3) = 0.1*total(phaspec_sel)
p_init(4) = 0.0
energy_dif = 8.3 - 7.47

p_res = g2fitb( e_bds_sel, phaspec_sel, p_init, energy_dif, phaspec_fit, itm=itm, /sil)

oplot,e_sel,phaspec_fit


end


function g2fitb, chan_bds, array, p_init, energy_dif, array_fit, silent=silent, itm=itm
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;       2001-11-26  Updated to include constant background
;
npar = n_params()
if npar lt 2 then begin
   print,'Syntax: res = g2fitb( chan_bds, array, p_init, energy_dif[, array_fit][/silent] )
   return,-1.
endif

common g2fitb_com, chan_bds_com, array_com, energy_dif_com

chan_bds_com = chan_bds
array_com = array
energy_dif_com = energy_dif

n = n_elements(array)
if n_elements(chan_bds) ne n+1 then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif


plim = fltarr(2,5)
plim(0,0) = 0.
plim(1,0) = total(array)
plim(0,1) = 0.7 * p_init(1)
plim(1,1) = 1.3 * p_init(1)
plim(0,2) = 0.5 * p_init(2)
plim(1,2) = 2.0 * p_init(2)
plim(0,3) = 0.
plim(1,3) = 0.4 * total(array)
plim(0,4) = 0.
plim(1,4) = total(array) / n_elements(array)



bkg_lev = 0.
avg_lev = avg(array)

;if not keyword_set(silent) then begin
   print,'Initial values : sum1 = ', p_init(0)
   print,'                mean1 = ', p_init(1)
   print,'               sigma1 = ', p_init(2)
   print,'                 sum2 = ', p_init(3)
   print,'              bkg_lev = ', p_init(4)
;endif


; Define the arrays to use with AMOEBAN fitting procedure

p = fltarr(6,5)
p(0,*) = p_init
p(1,*) = p_init * [1.05, 1.0, 1.0, 1.0, 1.0]
p(2,*) = p_init * [1.0, 1.05, 1.0, 1.0, 1.0]
p(3,*) = p_init * [1.0, 1.0, 1.05, 1.0, 1.0]
p(4,*) = p_init * [1.0, 1.0, 1.0, 1.05, 1.0]
p(5,*) = p_init + [0.0, 0.0, 0.0, 0.0, plim(1,4)*0.1]


res = amoeban(p, fun='g2fitb_func', plim=plim, itm=itm, silent=silent)

print,'  Final values : sum1 = ', res(0)
print,'                mean1 = ', res(1)
print,'               sigma1 = ', res(2)
print,'                 sum2 = ', res(3)
print,'              bkg_lev = ', res(4)



if npar gt 2 then array_fit = res(0) * gmodel(chan_bds, res(1), res(2)) $
                            + res(3) * gmodel(chan_bds, res(1)+energy_dif, res(2)) + res(4)

return, res

end

;=============================================================================

function g2fitb2, chan_lo, chan_hi, array, p_init, array_fit, silent=silent
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries defined by 'chan_lo'
;  and 'chan_hi'
;
;   NJW 2001-11-14
;
npar = n_params()
if npar lt 3 then begin
   print,'Syntax: res = g2fitb2( chan_lo, chan_hi, array, p_init[, array_fit][,/silent] )
endif


n = n_elements(array)
if n_elements(chan_lo) ne n or n_elements(chan_hi) ne n then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif

chan_bds = [chan_lo, chan_hi(n-1)]

if npar eq 3 then begin
   return, g2fitb( chan_bds, array, p_init, silent=silent )
endif else begin
   return, g2fitb( chan_bds, array, p_init, array_fit, silent=silent )
endelse

end

;=============================================================================

function gmodel, chan_bds, mean, sigma
;
;  A function to calculate a gaussian curve
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;
if n_params() ne 3 then begin
   print,'Syntax: res = gmodel( chan_bds, mean, sigma )
endif

n = n_elements(chan_bds) - 1
; 1.41421356237 == sqrt(2)
div = sigma * 1.41421356237
res = fltarr(n)

for i = 0, n-1 do begin

   a1 = (chan_bds(i) - mean) / div
   a2 = (chan_bds(i+1) - mean) / div
   res(i) = 0.5*(erf(a2) - erf(a1))

endfor

return, res

end

;=============================================================================

function g2fitb_func, p
;
;  Function to be minimized in 'g2fitb'
;
;  NJW 2001-11-14
;

common g2fitb_com, chan_bds, array, energy_dif

res1 = gmodel( chan_bds, p(1), p(2) )
res2 = gmodel( chan_bds, p(1)+energy_dif, p(2) )

return, total( (res1*p(0) + res2*p(3) + p(4) - array)^2 )

end
