;
;  This file contains the necessary procedures to do a
;  2 Gauss functions fit with constant background on a spectrum
;  where the channel boundaries are given i.e. not necessarily
;  of same width
;
;  The use of the error function also removes the constraint to
;  have small channel widths as compared to the sigma of the
;  Gauss function
;


function g2fitb, chan_bds, array, array_fit, silent=silent, itm=itm
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;       2001-11-26  Updated to include constant background
;
npar = n_params()
if npar lt 2 then begin
   print,'Syntax: res = g2fitb( chan_bds, array[, array_fit][/silent] )
   return,-1.
endif

common g2fitb_com, chan_bds_com, array_com

chan_bds_com = chan_bds
array_com = array

n = n_elements(array)
if n_elements(chan_bds) ne n+1 then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif

; Find initial values for 'array'

chan_cen = 0.5*(chan_bds(0:n-1)+chan_bds(1:n))

plot,chan_cen,array,ps=10
print,'Now define peak of first component'
curmark1, mark_p
print,'Now define left/right at half peak for component 1'
curmark,mark_fwhm

fwhm = mark_fwhm(0,1) - mark_fwhm(0,0)
bins = where(mark_fwhm(0,0) lt chan_cen and chan_cen lt mark_fwhm(0,1),nbins)
sum1 = mark_p(1) * nbins
mean1 = mark_p(0)
sigma1 = fwhm / 2.3548

plim = fltarr(2,7)
plim(0,0) = 0.
plim(1,0) = total(array)
plim(0,1) = mark_fwhm(0,0)
plim(1,1) = mark_fwhm(0,1)
plim(0,2) = 0.
plim(1,2) = fwhm

print,'Now define peak of component 2'
curmark1, mark_p
print,'Now define left/right at half peak for component 2'
curmark,mark_fwhm

fwhm = mark_fwhm(0,1) - mark_fwhm(0,0)
bins = where(mark_fwhm(0,0) lt chan_cen and chan_cen lt mark_fwhm(0,1),nbins)
sum2 = mark_p(1) * nbins
mean2 = mark_p(0)
sigma2 = fwhm / 2.3548

plim(0,3) = 0.
plim(1,3) = total(array)
plim(0,4) = mark_fwhm(0,0)
plim(1,4) = mark_fwhm(0,1)
plim(0,5) = 0.
plim(1,5) = fwhm


bkg_lev = 0.
avg_lev = avg(array)

;if not keyword_set(silent) then begin
   print,'Initial values : sum1 = ', sum1
   print,'                mean1 = ', mean1
   print,'               sigma1 = ', sigma1
   print,'                 sum2 = ', sum2
   print,'                mean2 = ', mean2
   print,'               sigma2 = ', sigma2
   print,'             bkg_lev = ', bkg_lev
;endif

ans = ''
read,'Continue ? ',ans
if ans ne 'y' then return,-1.

; Define the arrays to use with AMOEBAN fitting procedure

p = fltarr(8,7)
p(0,*) = [sum1, mean1, sigma1, sum2, mean2, sigma2, bkg_lev]
p(1,*) = [sum1*1.05, mean1, sigma1, sum2, mean2, sigma2, bkg_lev]
p(2,*) = [sum1, mean1*1.05, sigma1, sum2, mean2, sigma2, bkg_lev]
p(3,*) = [sum1, mean1, sigma1*1.05, sum2, mean2, sigma2, bkg_lev]
p(4,*) = [sum1, mean1, sigma1, sum2*1.05, mean2, sigma2, bkg_lev]
p(5,*) = [sum1, mean1, sigma1, sum2, mean2*1.05, sigma2, bkg_lev]
p(6,*) = [sum1, mean1, sigma1, sum2, mean2, sigma2*1.05, bkg_lev]
p(7,*) = [sum1, mean1, sigma1, sum2, mean2, sigma2, bkg_lev + avg_lev*0.05]


res = amoeban(p, fun='g2fitb_func', plim=plim, itm=itm, silent=silent)

if npar gt 2 then array_fit = res(0) * gmodel(chan_bds, res(1), res(2)) $
                            + res(3) * gmodel(chan_bds, res(4), res(5)) + res(6)

return, res

end

;=============================================================================

function g2fitb2, chan_lo, chan_hi, array, array_fit, silent=silent
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries defined by 'chan_lo'
;  and 'chan_hi'
;
;   NJW 2001-11-14
;
npar = n_params()
if npar lt 3 then begin
   print,'Syntax: res = g2fitb2( chan_lo, chan_hi, array[, array_fit][,/silent] )
endif


n = n_elements(array)
if n_elements(chan_lo) ne n or n_elements(chan_hi) ne n then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif

chan_bds = [chan_lo, chan_hi(n-1)]

if npar eq 3 then begin
   return, g2fitb( chan_bds, array, silent=silent )
endif else begin
   return, g2fitb( chan_bds, array, array_fit, silent=silent )
endelse

end

;=============================================================================

function gmodel, chan_bds, mean, sigma
;
;  A function to calculate a gaussian curve
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;
if n_params() ne 3 then begin
   print,'Syntax: res = gmodel( chan_bds, mean, sigma )
endif

n = n_elements(chan_bds) - 1
; 1.41421356237 == sqrt(2)
div = sigma * 1.41421356237
res = fltarr(n)

for i = 0, n-1 do begin

   a1 = (chan_bds(i) - mean) / div
   a2 = (chan_bds(i+1) - mean) / div
   res(i) = 0.5*(erf(a2) - erf(a1))

endfor

return, res

end

;=============================================================================

function g2fitb_func, p
;
;  Function to be minimized in 'g2fitb'
;
;  NJW 2001-11-14
;

common g2fitb_com, chan_bds, array

res1 = gmodel( chan_bds, p(1), p(2) )
res2 = gmodel( chan_bds, p(4), p(5) )

return, total( (res1*p(0) + res2*p(3) + p(6) - array)^2 )

end
