pro gauss_seidel, method, a, y, x, alpha, qx, niter_max,$
    bot, top, smooth=smooth
;
;  Solve the system of linear equations by the Gauss-Seidel
;  method as described by Li and Wu Astroph. and Space Sci. 215, 213
;  (1994)  equation (9, 10) with the optional constraint that in each
;  iteration the approximated solution fulfills
;
;              bot(i) < x(i) [ < top(i) ]
;
;     a # x = y,
;   if method eq  9 then the procedure returns 'x' by eq. (9)
;   if method eq 10 then the procedure returns 'x' by eq. (10)
;
;  a is a quadratic (positive definite) matrix (N x N)
;      and y a vector of size N
;
if n_params() ne 7 then begin
   print,'GAUSS_SEIDEL requires parameters: method, a, y, x, alpha,'
   print,'             ','qx, niter_max[,bot[,top]][,smooth=smooth]'
   return
endif
sz = size(a)
if sz(1) ne sz(2) then begin
   print,'GAUSS_SEIDEL requires a square matrix'
   return
endif
n = sz(1)
if sz(1) ne n_elements(y) then begin
   print,'GAUSS_SEIDEL y must have same dimension as a'
   return
endif
if sz(1) ne n_elements(x) then begin
   print,'GAUSS_SEIDEL x must have same dimension as a'
   return
endif
bset = 0                        ; logical variable for bottom set
if n_params() gt 7 then begin
   bset = 1
   if sz(1) ne n_elements(bot) then begin
      print,'GAUSS_SEIDEL bot must have same dimension as a'
      return
   endif
endif
tset = 0                        ; logical variable for top set
if n_params() gt 8 then begin
   tset = 1
   if sz(1) ne n_elements(top) then begin
      print,'GAUSS_SEIDEL top must have same dimension as a'
      return
   endif
endif

k = keyword_set(smooth)
alpha = (0.001 > alpha) < 0.999 
x1 = x*1.0
niter = 0
qx = fltarr(niter_max)

floor = 0.
ceiling = 1.e35
repeat begin
   for i = 0, n-1 do begin
      if bset then floor = bot(i)
      if tset then ceiling = top(i)
      if method eq 9 then begin
         if i gt 0 then t2 = total(a(i,0:i-1)*x(0:i-1)) else t2 = 0.
         if i lt n-1 then t3 =$
             total(a(i,i+1:n-1)*x1(i+1:n-1)) else t3 = 0.
         t4 = (1.-a(i,i)) * x1(i)
         x(i) = ((alpha*(y(i) - t2 - t3 + t4) $
             + (1.-alpha)*x1(i)) > floor) < ceiling
      endif else begin   ; method (10)
         if i gt 0 then t2 = total(a(i,0:i-1)*x(0:i-1)) else t2 = 0.
         if i lt n-1 then t3 =$
            total(a(i,i+1:n-1)*x1(i+1:n-1)) else t3 = 0.
         x(i) = (alpha*(y(i) - t2 - t3)/a(i,i) + (1.-alpha)*x1(i))
         x(i) = (x(i) > floor) < ceiling
      endelse
      if k then begin
         floor = x(i) - smooth
         ceiling = x(i) + smooth
      endif
   endfor
   x1 = x
   qx(niter) = total(abs(y-a#x))
   print,'GS iteration #', niter,' qx: ', qx(niter)
   niter = niter + 1
endrep until (niter gt niter_max-1) or (qx(niter-1) gt 1.e30)
print, 'GAUSS_SEIDEL', method,': number of iterations = ', niter
if niter lt niter_max then print,' DIVERGING ! undue termination'
niter_max = niter
return
end
