;
;  This file contains the necessary procedures to do a
;  Gauss function fit with constant background on a spectrum
;  where the channel boundaries are given i.e. not necessarily
;  of same width
;
;  The use of the error function also removes the constraint to
;  have small channel widths as compared to the sigma of the
;  Gauss function
;


function gfitb, chan_bds, array, array_fit, silent=silent
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;       2001-11-26  Updated to include constant background
;
npar = n_params()
if npar lt 2 then begin
   print,'Syntax: res = gfitb( chan_bds, array[, array_fit][/silent] )
   return,-1.
endif

common gfitb_com, chan_bds_com, array_com

chan_bds_com = chan_bds
array_com = array

n = n_elements(array)
if n_elements(chan_bds) ne n+1 then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif

; Find initial values for 'array'

chan_cen = 0.5*(chan_bds(0:n-1)+chan_bds(1:n))

sum = total(array)
mean = total(array*chan_cen) / sum
sigma = sqrt(total(array*(chan_cen-mean)^2) / sum)
bkg_lev = 0.
avg_lev = sum / n

if not keyword_set(silent) then begin
   print,'Initial values : sum = ', sum
   print,'                mean = ', mean
   print,'               sigma = ', sigma
   print,'             bkg_lev = ', bkg_lev
endif

; Define the arrays to use with AMOEBAN fitting procedure

p = fltarr(5,4)
p(0,*) = [sum, mean, sigma, bkg_lev]
p(1,*) = [sum*1.1, mean, sigma, bkg_lev]
p(2,*) = [sum, mean*1.1, sigma, bkg_lev]
p(3,*) = [sum, mean, sigma*1.1, bkg_lev]
p(4,*) = [sum, mean, sigma, bkg_lev + avg_lev*0.1]

res = amoeban(p, fun='gfitb_func', itm=100, silent=silent)

if npar gt 2 then array_fit = res(0) * gmodel(chan_bds, res(1), res(2)) + res(3)

return, res

end

;=============================================================================

function gfitb2, chan_lo, chan_hi, array, array_fit, silent=silent
;
;  A function to fit a gaussian curve to 'array' with
;  non-equidistant channel boundaries defined by 'chan_lo'
;  and 'chan_hi'
;
;   NJW 2001-11-14
;
npar = n_params()
if npar lt 3 then begin
   print,'Syntax: res = gfitb2( chan_lo, chan_hi, array[, array_fit][,/silent] )
endif


n = n_elements(array)
if n_elements(chan_lo) ne n or n_elements(chan_hi) ne n then begin
   print,'Error, dimension mismatch for arguments'
   return, -1
endif

chan_bds = [chan_lo, chan_hi(n-1)]

if npar eq 3 then begin
   return, gfitb( chan_bds, array, silent=silent )
endif else begin
   return, gfitb( chan_bds, array, array_fit, silent=silent )
endelse

end

;=============================================================================

function gmodel, chan_bds, mean, sigma
;
;  A function to calculate a gaussian curve
;  non-equidistant channel boundaries 'chan_bds'
;
;   NJW 2001-11-14
;
if n_params() ne 3 then begin
   print,'Syntax: res = gmodel( chan_bds, mean, sigma )
endif

n = n_elements(chan_bds) - 1
; 1.41421356237 == sqrt(2)
div = sigma * 1.41421356237
res = fltarr(n)

for i = 0, n-1 do begin

   a1 = (chan_bds(i) - mean) / div
   a2 = (chan_bds(i+1) - mean) / div
   res(i) = 0.5*(erf(a2) - erf(a1))

endfor

return, res

end

;=============================================================================

function gfitb_func, p
;
;  Function to be minimized in 'gfitb'
;
;  NJW 2001-11-14
;

common gfitb_com, chan_bds, array

res = gmodel( chan_bds, p(1), p(2) )

return, total( (res*p(0) + p(3) - array)^2 )

end
