function j_get_gain_for_source, swid, ra, dec, chat=chat

;  Derive the gain [linPHA per keV] by comparing PHA and PI
;  for processed data for a given science window and source
;  position. Assume that the necessary data files can be
;  found in the /r8/jemx/arc/rev_2 data archive.
;
;  2006-12-22/NJW. Based on 'j_get_used_gain.pro'
;
;  Define the local gain as the number of linPHA channels per keV:
;
;     local_gain = linPHA / Energy
;
;  The local gain is both a function of detector position and of time
;
;  Define the global gain (global_gain) as
;
;     global_gain = average[ linPHA / (Energy * f_spag) ]
;
;  where f_spag is the local spatial gain factor and the average
;  is taken over a suitable area of the detector
;
;  The global gain is a function of time i.e. 'global' refers to the
;  detector surface

if n_params() lt 3 then begin
   print,'Syntax: gain = j_get_gain_for_source( swid, ra, dec [,/chat])'
   return, -1
endif

sz = size(swid)
if sz(sz(0)+1) ne 7 then begin
   print,'The ''swid'' must be a string variable'
   return,-1
endif
if strlen(swid) ne 12 then begin
   print,'The ''swid'' must have a length of 12'
   return,-1
endif

strrev = strmid(swid,0,4)

arcdir = '/r8/jemx/arc/rev_2/scw/'+strrev+'/'+swid+'.001/'
evfile = gz_proxy_file( arcdir+'jmx1_events.fits' )

if not file_test(evfile,/reg) then begin
   print,arcdir+'jmx1_events.fits is missing'
   return, -1
endif

fhdr = headfits( evfile, ext=3 )
fswid = stri(fxpar(fhdr, 'SWID'))
finstr = stri(fxpar(fhdr, 'INSTRUME'))

if fswid ne swid then begin
   print,'Mismatching SWIDs: ', fswid+'  '+swid
   return, -1
endif

sjmx = strmid(finstr,3,1)

;  Get instances of IMOD and RMF

spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' imod', insta_imod
spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' rmf', insta_rmf
insta_imod = insta_imod(0)
insta_rmf = insta_rmf(0)

;  Get the events

errmsg = ''
extnam = finstr+'-FULL-ALL'
rdfitscol,evfile+'['+extnam+']',rawx,'RAWX',errmsg=errmsg,nrows=nrows
rdfitscol,evfile+'['+extnam+']',rawy,'RAWY',/sil
rdfitscol,evfile+'['+extnam+']',pha,'PHA',/sil
;* rdfitscol,evfile+'['+extnam+']',time,'TIME',/sil
rdfitscol,evfile+'['+extnam+']',pi,'PI',/sil
rdfitscol,evfile+'['+extnam+']',status,'STATUS',/sil

;  Define 'flag' array to make sure that each detector position is only
;  counted once when making the average

flag = intarr(256,256)

;  Get the PI energy boundaries

ebm_get_child_by_name,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+'.fits+1', $
      finstr+'-FBDS-MOD', dol_fbds
rdfitscol, dol_fbds, e_min,'e_min',/sil
rdfitscol, dol_fbds, e_max,'e_max',/sil

;  Get the PHA rebinning table i.e. from linear PHA to PHA in telemetry

rdfitscol,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+ $
      '.fits['+finstr+'-FULB-MOD]',fulb,'channel',/sil

;  Get the spatial gain array with two components

spag = readfits('/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+'.fits',ext=24)
print,'size(spag):', size(spag)

;  Make an event selection based on STATUS flag

w = where( status lt 33, nw )
print,stri(nw)+' were selected from '+stri(nrows)+' events'
rawx = rawx(w)
rawy = rawy(w)
;* time = time(w)
pha = pha(w)
pi = pi(w)
status = status(w)

energy = fltarr(nw)
lin_pha = fltarr(nw)

; Since the global gain is a function of time an array
; is made

global_gain_arr = fltarr(nw)

n_used = 0L

;   Loop over the selected events

for i = 0L, nw-1 do begin

   ;  assign the linPHA value randomly between given limits

   lin_pha_min = fulb(pha(i)-1) + 1
   lin_pha_max = fulb(pha(i))
   if pha(i) le 128 then lin_pha(i) = lin_pha_max else $
      lin_pha(i) = lin_pha_min + $
      float(fix(randomu(seed)*(lin_pha_max-lin_pha_min)+0.5))

   ;  'lin_pha' is (close to) the linPHA signal as delivered from the DFEE

   ;  assign an energy between PI limits

   energy(i) = e_min(pi(i)) + randomu(seed)*(e_max(pi(i)) - e_min(pi(i)))

   f_spag = ((1. + 0.01*spag(rawx(i),rawy(i),0))*spag(rawx(i),rawy(i),1))

   global_gain_arr(i) = lin_pha(i) / (energy(i) * f_spag)

   if keyword_set( chat ) then begin
      if i mod 7345L eq 0 then begin
         print,'i = ',i,' w(i) = ', w(i)
         print,'RAWX,RAWY = ', rawx(i), rawy(i)
         print,'PHA,PI,ENERGY = ', pha(i), pi(i), energy(i)
         print,'SPAG level 0 = ', spag(rawx(i),rawy(i),0)
         print,'SPAG level 1 = ', spag(rawx(i),rawy(i),1)
         print,'Linear PHA between ', lin_pha_min, lin_pha_max
         print,'Chosen Linear PHA = ', lin_pha(i)
         print,'Local gain = ', lin_pha(i) / energy(i)
         print,'Global gain = ', global_gain_arr(i)
      endif
   endif

endfor

return, global_gain_arr

end
