function j_get_used_gain, full_cor_file, raw_file, chat=chat

;  Derive the gain [linPHA per keV] by comparing PHA and PI
;  for processed data
;
;  2005-11-03/NJW. Based on 'predict_arf'


;  Define the local gain as the number of linPHA channels per keV:
;
;     local_gain = linPHA / Energy
;
;  The local gain is both a function of detector position and of time

;  Define the global gain (global_gain) as
;
;     global_gain = average[ linPHA / (Energy * f_spag) ]
;
;  where f_spag is the local spatial gain factor and the average
;  is taken over a suitable area of the detector
;
;  The global gain is a function of time i.e. 'global' refers to the
;  detector surface

;* s = '/net/gauss/data/njw/jemx/q019_a5_01/obs/q019_1_block_01/'
;* full_cor_file = s+'scw/036500800010.000/'+strlowcase(finstr)+'_full_cor.fits'
;* s = '/nfs/lin6/data/nrt/scw/0365/'
;* raw_file = s+'036500800010.000/'+strlowcase(finstr)+'/raw/'+strlowcase(finstr)+'_raw.fits'

;* insta_imod = '0093'
;* insta_rmf  = '0039'

if n_params() lt 2 then begin
   print,'Syntax: gain = j_get_used_gain( full_cor_file, raw_file[,/chat])'
   return, -1
endif

if not file_test(full_cor_file,/reg) then begin
   print,full_cor_file,' is missing'
   return, -1
endif
if not file_test(raw_file,/reg) then begin
   print,raw_file,' is missing'
   return, -1
endif

fhdr = headfits( full_cor_file, ext=1 )
rhdr = headfits( raw_file, ext=1 )
fswid = stri(fxpar(fhdr, 'SWID'))
rswid = stri(fxpar(rhdr, 'SWID'))
finstr = stri(fxpar(fhdr, 'INSTRUME'))
rinstr = stri(fxpar(rhdr, 'INSTRUME'))

if fswid ne rswid then begin
   print,'Mismatching SWIDs: ', fswid+'  '+rswid
   return, -1
endif
if finstr ne rinstr then begin
   print,'Mismatching INSTRUMEs: ', finstr+'  '+rinstr
   return, -1
endif

sjmx = strmid(finstr,3,1)

;  Get instances of IMOD and RMF

spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' imod', insta_imod
spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' rmf', insta_rmf
insta_imod = insta_imod(0)
insta_rmf = insta_rmf(0)

;  Get the events

errmsg = ''
extnam = finstr+'-FULL-RAW'
rdfitscol,raw_file+'['+finstr+'-FULL-RAW]',rawx,'RAWX',errmsg=errmsg,nrows=nrows
if errmsg ne '' then begin
   errmsg = ''
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',rawx,'RAWX',errmsg=errmsg,nrows=nrows
   if errmsg ne '' then begin
      print,'Emptied all options - give up!'
      return, -1
   endif
endif
if errmsg ne '' then begin
   print,'This is an events file'
   errmsg = ''
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',rawx,'RAWX',nrows=nrows,errmsg=errmsg
   if errmsg ne '' then begin
      print,'Well, it wasn''t an events file after all - I give up!'
      return,-1
   endif
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',rawy,'RAWY',/sil
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',pha,'PHA',/sil
endif else begin
   rdfitscol,raw_file+'['+finstr+'-FULL-RAW]',rawy,'RAWY',/sil
   rdfitscol,raw_file+'['+extnam+'-FULL-RAW]',pha,'PHA',/sil
endelse

;* rdfitscol,full_cor_file+'+1',time,'TIME',/sil
rdfitscol,full_cor_file+'+1',pi,'PI',/sil
rdfitscol,full_cor_file+'+1',status,'STATUS',/sil

;  Define 'flag' array to make sure that each detector position is only
;  counted once when making the average

flag = intarr(256,256)

;  Get the PI energy boundaries

ebm_get_child_by_name,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+'.fits+1', $
      finstr+'-FBDS-MOD', dol_fbds
rdfitscol, dol_fbds, e_min,'e_min',/sil
rdfitscol, dol_fbds, e_max,'e_max',/sil

;  Get the PHA rebinning table i.e. from linear PHA to PHA in telemetry

rdfitscol,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+ $
      '.fits['+finstr+'-FULB-MOD]',fulb,'channel',/sil

;  Get the spatial gain array with two components

spag = readfits('/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+'.fits',ext=24)
print,'size(spag):', size(spag)

;  Make an event selection based on STATUS flag

w = where( status lt 33, nw )
print,stri(nw)+' were selected from '+stri(nrows)+' events'
rawx = rawx(w)
rawy = rawy(w)
;* time = time(w)
pha = pha(w)
pi = pi(w)
status = status(w)

energy = fltarr(nw)
lin_pha = fltarr(nw)

; Since the global gain is a function of time an array
; is made

global_gain_arr = fltarr(nw)

n_used = 0L

;   Loop over the selected events

for i = 0L, nw-1 do begin

   ;  assign the linPHA value randomly between given limits

   lin_pha_min = fulb(pha(i)-1) + 1
   lin_pha_max = fulb(pha(i))
   if pha(i) le 128 then lin_pha(i) = lin_pha_max else $
      lin_pha(i) = lin_pha_min + $
      float(fix(randomu(seed)*(lin_pha_max-lin_pha_min)+0.5))

   ;  'lin_pha' is (close to) the linPHA signal as delivered from the DFEE

   ;  assign an energy between PI limits

   energy(i) = e_min(pi(i)) + randomu(seed)*(e_max(pi(i)) - e_min(pi(i)))

   f_spag = ((1. + 0.01*spag(rawx(i),rawy(i),0))*spag(rawx(i),rawy(i),1))

   global_gain_arr(i) = lin_pha(i) / (energy(i) * f_spag)

   if keyword_set( chat ) then begin
      if i mod 7345L eq 0 then begin
         print,'i = ',i,' w(i) = ', w(i)
         print,'RAWX,RAWY = ', rawx(i), rawy(i)
         print,'PHA,PI,ENERGY = ', pha(i), pi(i), energy(i)
         print,'SPAG level 0 = ', spag(rawx(i),rawy(i),0)
         print,'SPAG level 1 = ', spag(rawx(i),rawy(i),1)
         print,'Linear PHA between ', lin_pha_min, lin_pha_max
         print,'Chosen Linear PHA = ', lin_pha(i)
         print,'Local gain = ', lin_pha(i) / energy(i)
         print,'Global gain = ', global_gain_arr(i)
      endif
   endif

endfor

return, global_gain_arr

end
