function j_make_bkg_shd, e1, e2, trouble_shd
;
;  Produce a shadowgram with expected background in a given energy interval
;
;  At DSRI we have the IC data in /r2/jemx/ic in three subdirectories:
;             bkg/, imod_grp/, and rmf_grp/
;
;      For 'bkg' the contents of ISDC /isdc/testdata/unit_test/ic_tree/4.50/ic/jmx2/bkg
;      have been copied
;
;  The njw-notebook (Linux) version has the data from Sami Maisala 2003-08-14
;  in /mnt/win_f/r2/jemx/ic/bkg/bkg_scw_models (SCAL, SIDX, SPAT) and
;  in /mnt/win_f/r2/jemx/ic/bkg/new_ic_bkg (SCAT, SIDX-IDX, SPAT-IDX, (+ASPE, ATTI
;     ORBI,RADL))
;

;* e1 = 10 ; keV only for main program version
;* e2 = 20 ; keV only for main program version

if n_params() ne 3 then begin
   print,'Syntax: bkg_image = j_make_bkg_shd( e1, e2, trouble_shd)
   print,'   bkg_image has dimensions 2x256x256 where layer 0 is'
   print,'   instrument (particle) background and layer 1 is'
   print,'   diffuse X-ray background.'
   print,'   trouble_shd is a report on missing pixels in shadowgrams'
   return,-1
endif

backg_shd = fltarr(2,256,256) ; to store the result for bkg and dbg
trouble_shd = intarr(2,256,256) ; to store the pixels where trouble occurred

; Get the PI energy table

if !version.os eq 'sunos' then begin
   imod_file = '/r2/jemx/ic/imod_grp/jmx2_imod_grp_0034.fits'
endif else begin
   imod_file = '/mnt/win_f/r2/jemx/ic/imod_grp/jmx2_imod_grp_0034.fits'
endelse
rdfitscol,imod_file+'+19',e_min,'e_min',nrows=nrows
rdfitscol,imod_file+'+19',e_max,'e_max'

e = 0.5*(e_min + e_max)
de = e_max - e_min

e_sel = where( e gt e1 and e lt e2, n_e_sel)
if n_e_sel lt 1 then begin
   print,'Your energy interval is too narrow or outside limits'
;   return,-1
   stop
endif

; Get the BKG and DBG distribution map
; Use the data supplied by Sami Maisala 2003-08-14

if !version.os eq 'sunos' then begin
   spat_bkg = readfits('/r2/jemx/ic/bkg/jmx2_spat_bkg_0006.fits',exten=6)
   spat_dbg = readfits('/r2/jemx/ic/bkg/jmx2_spat_dbg_0004.fits',exten=8)
endif else begin
   scw_dir = '/mnt/win_f/r2/jemx/ic/bkg/bkg_scw_models/'
   spat_bkg = readfits(scw_dir+'JMX2-SPAT-BKG.fits',exten=1)
   spat_dbg = readfits(scw_dir+'JMX2-SPAT-DBG.fits',exten=1)
endelse

; Get the BKG and DBG index arrays

if !version.os eq 'sunos' then begin
   sidx_bkg = readfits('/r2/jemx/ic/bkg/jmx2_sidx_bkg_0003.fits',exten=4)
   sidx_dbg = readfits('/r2/jemx/ic/bkg/jmx2_sidx_dbg_0002.fits',exten=2)
endif else begin
   scw_dir = '/mnt/win_f/r2/jemx/ic/bkg/bkg_scw_models/'
   sidx_bkg = readfits(scw_dir+'JMX2-SIDX-BKG.fits',exten=1)
   sidx_dbg = readfits(scw_dir+'JMX2-SIDX-DBG.fits',exten=1)
endelse

; Get the BKG and DBG selection of spectra from the IC data

if !version.os eq 'sunos' then begin
   rdfitscol,'/r2/jemx/ic/bkg/jmx2_scat_bkg_0003.fits+1',$
          spe_inum_bkg,'spe_inum',nrows=nrows_bkg
   rdfitscol,'/r2/jemx/ic/bkg/jmx2_scat_bkg_0003.fits+1',spectrum_bkg,'spectrum'
   rdfitscol,'/r2/jemx/ic/bkg/jmx2_scat_dbg_0003.fits+1',$
          spe_inum_dbg,'spe_inum',nrows=nrows_dbg
   rdfitscol,'/r2/jemx/ic/bkg/jmx2_scat_dbg_0003.fits+1',spectrum_dbg,'spectrum'
endif else begin
   ic_dir = '/mnt/win_f/r2/jemx/ic/bkg/new_ic_bkg/'
   rdfitscol,ic_dir+'JMX2-SCAT-BKG.fits+1',spe_inum_bkg,'spe_inum',nrows=nrows_bkg
   rdfitscol,ic_dir+'JMX2-SCAT-BKG.fits+1',spectrum_bkg,'spectrum'
   rdfitscol,ic_dir+'JMX2-SCAT-DBG.fits+1',spe_inum_dbg,'spe_inum',nrows=nrows_dbg
   rdfitscol,ic_dir+'JMX2-SCAT-DBG.fits+1',spectrum_dbg,'spectrum'
endelse

; Get the average scaling factor from SCW data products

if !version.os eq 'sunos' then begin
   print,'Use default scaling factor of 20.0 (BKG) and 5.0 (DBG)'
   avg_scalfact_bkg = 20.0
   avg_scalfact_dbg = 5.0
endif else begin
   scw_dir = '/mnt/win_f/r2/jemx/ic/bkg/bkg_scw_models/'
   rdfitscol,scw_dir+'JMX2-SCAL-BKG.fits+1',scalfact_bkg,'scalfact'
   rdfitscol,scw_dir+'JMX2-SCAL-DBG.fits+1',scalfact_dbg,'scalfact'
   avg_scalfact_bkg = avg(scalfact_bkg)
   avg_scalfact_dbg = avg(scalfact_dbg)
endelse
   
; Get the fraction of counts from each spectrum

norm_cts_bkg = total(spectrum_bkg(e_sel,*),1)
norm_cts_dbg = total(spectrum_dbg(e_sel,*),1)

; Calculate the background shadowgram

for i = 0, 256-1 do begin
for j = 0, 256-1 do begin

   index_bkg = sidx_bkg(i,j)
   pos_bkg = where(index_bkg eq spe_inum_bkg, npos)
   if npos eq 1 then begin
      backg_shd(0,i,j) = norm_cts_bkg(pos_bkg) * spat_bkg(i,j)
   endif else trouble_shd(0, i, j) = 1

   index_dbg = sidx_dbg(i,j)
   pos_dbg = where(index_dbg eq spe_inum_dbg + 1, npos)
   if npos eq 1 then begin
      backg_shd(1,i,j) = norm_cts_dbg(pos_dbg) * spat_dbg(i,j)
   endif else trouble_shd(1, i, j) = 1

endfor
endfor

; Include the scaling factors so that the shadowgram ends up
; having the unit cts/s/pixel

backg_shd(0,*,*) = backg_shd(0,*,*) * avg_scalfact_bkg
backg_shd(1,*,*) = backg_shd(1,*,*) * avg_scalfact_dbg

return, backg_shd

end
