pro jsim_vs_input_osim, swg_base_name, dol_of_find, noplot=noplot, outdir=outdir
;
; Compare simulation results with input data
;  using data generated by OSIM
;
;    NJW/2002-08-02
;

if n_params() lt 2 then begin
   print,'Syntax: JSIM_VS_INPUT, swg_base_name, dol_of_find$'
   print,'           [,/noplot][,outdir=]'
   return
endif


get_exten_no, dol_of_find, file_of_find, find_extno
extno = find_extno
str_extno = strtrim(extno,2)

if keyword_set(outdir) then begin ; output files directed here
   out_file = outdir+'/'+basename(swg_base_name)+'_'+str_extno+'.svi'
endif else out_file = swg_base_name+'_'+str_extno+'.svi'


fhdr = headfits(file_of_find, exten=find_extno)
shdiden = fxpar(fhdr,'SHDIDEN')
mindetlv = fxpar(fhdr,'MINDETLV')

openw,lun,out_file,/get_lun

printf,lun,'//            ', systime()
printf,lun,'// Report on: ', swg_base_name+'.fits'
printf,lun,'//     together with: ', dol_of_find
printf,lun,'// shdiden = ', shdiden

swg_file = swg_base_name+'.fits'
dol_of_swg = swg_file+'+1'

; 1)  Find satellite pointing and roll

hdr = headfits(swg_file, exten=1)
ra_scx = fxpar(hdr,'RA_SCX')
dec_scx = fxpar(hdr,'DEC_SCX')
roll = fxpar(hdr,'DROLLANG')

printf,lun,'// RA_SCX = ',ra_scx
printf,lun,'// DEC_SCX = ',dec_scx
printf,lun,'// DROLLANG = ', roll
printf,lun,'//'
printf,lun,'//    List of the input sources:'
printf,lun,'// block = -1'
printf,lun,'// #       RA          DEC         Sx         NH         param'

; 2)  Find the input source list

; Use SWG grouping table as entrance; the location of
;  the srcl index table is already known

ebm_get_child_by_name, dol_of_swg, 'JMX1-SRCL-CAT',dol_of_cat


; Read the input source data

errmsg = ''
rdfitscol,dol_of_cat,ra_input,'RA_OBJ',err=errmsg,/sil
rdfitscol,dol_of_cat,dec_input,'DEC_OBJ',err=errmsg,/sil,nrows=nrows

if nrows eq 0 then begin
   printf,lun,'//  No input sources found'
   n_input_srcs = 0
endif else begin
   sz = size(ra_input)
   if sz(0) eq 0 then ra_input = replicate(ra_input,1)
   sz = size(dec_input)
   if sz(0) eq 0 then dec_input = replicate(dec_input,1)

   n_input_srcs = n_elements(ra_input)
   for i = 0, n_input_srcs-1 do begin
      printf,lun,i,ra_input(i),dec_input(i),0.0,$
            0.0, 0.0,$
            format='(i5,2f12.5, 2e12.3, f12.5)'
   endfor
endelse

; 3)  Get the derived source positions and intensities


rdfitscol,dol_of_find,ra_der,'RA_OBJ',hdr=hdr2,/sil
rdfitscol,dol_of_find,dec_der,'DEC_OBJ',/sil
rdfitscol,dol_of_find,det_lev_der,'DETSIG',/sil
rdfitscol,dol_of_find,peak_der,'PEAKSIZE',err=errmsg,/sil,nrows=nrows
if errmsg ne '' then print,errmsg
e_min = fxpar(hdr2,'E_MIN')
e_max = fxpar(hdr2,'E_MAX')
telapse = fxpar(hdr2,'TELAPSE')
n_der = nrows

printf,lun,'//'
printf,lun,'// e_min = ', e_min, ' ; keV'
printf,lun,'// e_max = ', e_max, ' ; keV'
printf,lun,'// telapse = ', telapse, ' ; seconds'
printf,lun,'//'
printf,lun,'// Sources found by j_ima_src_find in the RSTI image:'
printf,lun,'//'
printf,lun,'// block = 0'


if n_der gt 0 then begin

   printf,lun,'//'
   printf,lun,'// #    RAfound      DECfound     peak       det-level   Off-axis'

   for i = 0, n_der-1 do begin
      offaxis = arcdist(ra_der(i),dec_der(i),ra_scx,dec_scx,/deg)
      printf,lun,i,ra_der(i),dec_der(i),peak_der(i),det_lev_der(i),offaxis,$
                  format='(i5,2f12.5,3f12.3)'
   endfor
endif else begin
   printf,lun,'//'
   printf,lun,'// No sources above detection level ', mindetlv
endelse

; Plot inputs

want_to_plot = 1
if keyword_set(noplot) then want_to_plot = 0
if want_to_plot then begin
   xr = [ra_scx+6.0/cos(dec_scx*!pi/180.),$
         ra_scx-6.0/cos(dec_scx*!pi/180.)]
   yr = [dec_scx-6.0, dec_scx+6.0]
   ;-- xr = [max(ra_input)+0.3,min(ra_input)-0.3]
   !p.title = swg_file+'['+str_extno+']'
   !x.title = 'RA (2000)'
   !y.title = 'DEC (2000)'
   plot, ra_input, dec_input, ps=1,xr=xr,yr=yr,/nodata
   oplotsiz, ra_input, dec_input,replicate(1.0,n_elements(ra_input)),ps=1,zn=1.5,zx=5

   ; and add derived sources

   if n_der gt 0 then begin
     def_sym,'c'
     oplotsiz,ra_der,dec_der,det_lev_der,ps=8,zn=1.5,zx=5
   endif
endif


; Step through the derived (found) sources

if n_der gt 0 then begin
   for i = 0, n_der-1 do begin

      printf,lun,'// Found source #'+strtrim(i,2), ra_der(i), dec_der(i)
      printf,lun,'// block = ', i+1
      printf,lun,'// identified with following input sources:'
      printf,lun,'// #   RAinput   DECinput  dist(arcmin) padding...'

      ; find input sources that are very close

      dist = arcdist(ra_input,dec_input,ra_der(i),dec_der(i),/deg)
      candidates = where(dist lt 0.05, n_cand)
      if n_cand gt 0 then begin
         for j = 0, n_cand-1 do begin
            k = candidates(j)
            printf,lun,k,ra_input(k),dec_input(k),dist(k)*60,$
                      0.,  0., format='(i5,5f12.5)'
         endfor
      endif else printf,lun,'//       No candidate found'


   endfor
endif

free_lun,lun
end


