pro lcbinning, timval_in, rate_in, rate_err_in, step_in, tc, lc, errc, $
     tunit=tunit, stepunit=stepunit, mode=mode, datatype=datatype
;
;  2003-03-03/NJW
;  2006-10-04/NJW Updated with first/last bin correction
;  2006-12-14/NJW Updated to handle disjunct input rate
;
;  Binning or rebinning of event or light-curve data
;
;  'timval' is event or bin time values. The time unit
;           can be given in keyword 'tunit' as (string) 'ijd' (default) or 's'
;  'rate'   must have same dimension as 'timval' if 'datatype' is 1 or 2 (see below)
;           Else it is ignored
;  'rate_err'    must have same dimension as 'timval' if 'datatype' is 1 or 2 (see below)
;           Else it is ignored
;  'step'   The desired time step size. The unit can be given
;           in keyword 'stepunit' as (string) 'ijd' or 's' (default)
;  'tc'     (Return) time values of bin centers. Same unit as 'timval'
;  'lc'     (Return) is the resulting light-curve
;  'errc'   (Return) is the resulting error of the light-curve
;
; Keywords:
;
;  'tunit'  Unit of input times 's' or 'ijd' (default)
;  'stepunit'  Unit of step 's' (default) or 'ijd'
;  'mode'   Mode for output values '/s' (default) or 'cts'
;  'datatype' 0:  input data is an event list (default)
;             1:  input data is a contiguous lightcurve with regular time steps
;             2:  input data is a disjunct lightcurve with possibly irregular time steps
;
;-----------------------------
if n_params() lt 5 then begin
   print,'Syntax: lcbinning, timval, rate, rate_err, step, >tc, >lc, >errc $'
   print,'     [datatype=][, tunit=][, stepunit=][, mode=]'
   print,'     Keywords'
   print,'          datatype - 0: input data is an event list (default)'
   print,'                     1: contiguous lightcurve with regular time steps'
   print,'                     2: disjunct lightcurve with possibly irregular time steps'
   print,'          tunit   :  unit for input time values (''s'' or ''ijd''(default))'
   print,'          stepunit:  time unit for step value  (''s''(default) or ''ijd'')'
   print,'             mode :  Mode for output values ''/s'' (defaults) or ''cts'''
   return
endif
;
;   Get the mode
;
if not keyword_set(mode) then mode = '/s'
;
;   Get the datatype
;
if not keyword_set(datatype) then datatype = 0
;
;   Get the time units
;
orig_tunit = 'ijd'
if keyword_set(tunit) then begin
   orig_tunit = ''
   str = strlowcase(tunit)
   if strmid(str,0,1) eq 's' then begin
      orig_tunit = 's'
      print,'Convert time values to days'
      timval = double(timval_in) / 86400.d0
   endif
   if str eq 'ijd' then begin
      orig_tunit = 'ijd'
      print,'No input time value conversion'
      timval = double(timval_in)
   endif
   if orig_tunit eq '' then begin
      print,'Illegal keyword tunit'
      return
   endif
endif else timval = double(timval_in)

orig_stepunit = 's'
if keyword_set(stepunit) then begin
   orig_stepunit = ''
   str = strlowcase(stepunit)
   if str eq 'ijd' then begin
      orig_stepunit = 'ijd'
      print,'No step unit conversion'
      step = double(step_in)
   endif
   if strmid(str,0,1) eq 's' then begin
      orig_stepunit = 's'
      print,'Convert step time unit to days'
      step = double(step_in) / 86400.d0
   endif
   if orig_stepunit eq '' then begin
      print,'Illegal keyword stepunit'
      return
   endif
endif else begin
   print,'Convert step time unit to days'
   step = double(step_in) / 86400.d0 ; default conversion
endelse
;
;   Event list or light curve input ?
;
is_lc = 1
if n_elements(rate_in) eq 1 then is_lc = 0
rate = double(rate_in)
rate_err = double(rate_err_in)
;
;   Determine original binsize
;
n_input_values = n_elements(timval)

if datatype eq 1 then begin
   dt = delta(timval)
   dtmax = max(dt)
   dtmin = min(dt)
   if dtmax eq dtmin then orig_binsize = dt(0) else begin
      av = avg(dt)
      rs = rms(dt)
      sel = where( abs(dt - av) lt 0.5*rs, n_sel)
      av = avg(dt(sel))
      rs = rms(dt(sel))
      orig_binsize = av
   endelse
;
;   Find rate correction factor
;
   rate_factor = orig_binsize / step
endif
if datatype eq 0 then begin
   rate = replicate(1.d0, n_input_values)
   rate_err  = replicate(1.d0, n_input_values)
endif
;
;   Remove time values where the rate is zero
;
w = where( rate ne 0.0 , n_w)
if n_w gt 0 and n_w lt n_input_values then begin
   timval = timval(w)
   rate = rate(w)
   rate_err = rate_err(w)
   print,'Rejected '+strtrim(n_input_values-n_w,2)+' input bins where rate==0'
endif
if n_w eq 0 then begin
   print,'No non-zero rate values - terminate!'
   return
endif
;
;   Setup array to hold the resulting light curve
;
t1 = min(timval)
t2 = max(timval)
n_input_values = n_elements(timval)
nsteps = long((t2 - t1)/step) + 1
t_adjust = 0.5*(nsteps*step - (t2 - t1))
lc = fltarr(nsteps)
numc = intarr(nsteps)
sumc = fltarr(nsteps)
wsumc = fltarr(nsteps)
errc = fltarr(nsteps) ; to hold the error
tcmin = t1 - t_adjust
tcmax = t2 + t_adjust
tc = xlingen( nsteps, tcmin+0.5*step, tcmax-0.5*step)
; get bin-numbers for all events or input lightcurve bins
bin = long((timval - tcmin)/step)
for i = 0L, n_input_values-1 do begin
   j = bin(i)
   if j lt nsteps then begin
      ; handle both the case of datatype 0,1, or 2
      weight = 1.
      if rate_err(i) gt 0 then weight = 1./rate_err(i)
      wsumc(j) = wsumc(j) + weight
      numc(j) = numc(j) + 1
      sumc(j) = sumc(j) + rate(i)*weight
      lc(j) = lc(j)+rate(i)
      errc(j) = errc(j) + rate_err(i)^2
   endif else print,'Warning: j = ', j
endfor
;
;  In case of datatype 2 do the calculation for output bins
;
if datatype eq 2 then begin
   for i = 0L, nsteps-1 do begin
      if numc(i) gt 0 then begin
         lc(i) = sumc(i) / wsumc(i)
         errc(i) = sqrt(numc(i))/wsumc(i)
      endif
   endfor
endif
;
; Correction to first and last bin
;
fraction = (tcmin + step - t1)/step;
lc(0) = lc(0) / fraction;
errc(0) = errc(0) / fraction;
fraction = (t2 - tcmax + step)/step;
lc(nsteps-1) = lc(nsteps-1) / fraction;
errc(nsteps-1) = errc(nsteps-1) / fraction;
;
;  Convert to counts per second
;
if datatype ne 2 then begin
   if mode eq '/s' then begin
      if datatype eq 1 then begin
         lc = lc * rate_factor
         errc = sqrt(errc) * rate_factor
      endif else begin
         lc = lc / (step * 86400.d0)
         errc = sqrt(errc) / (step * 86400.d0)
      endelse
   endif
endif
;
;   Convert to 'seconds' if that was the input unit
;
if orig_tunit eq 's' then begin
   timval = timval * 86400.d0
   tc = tc * 86400.d0
   step = step * 86400.d0
endif

end
