pro make_arf, help=help, dialog=dialog, noplot=noplot
;
;  2004-09-17/NJW, cloned from new_arf_step1.pro, arffit_....pro, new_arf_step2.pro
;  2005-08-18/NJW  updated for higher speed and larger sigma
;  2005-08-19/NJW  updated to remove repeated code and cleaned up
;  2005-11-14/NJW  updated to make sure that the ARF is increasing below 3 keV
;                     and that the 'rxl' array is used (typo from earlier version)
;
;  
;
;  Procedure: Take a Crab spectrum and the corresponding ARF (ARFc)
;  as defined in the parameter file 'make_arf.par' i.e. _NOT_ the
;  anchor file (ANCRFILE) from column in spectral file.
;
;  Take the ARF (ARFr) from a selected RMF-GRP (e.g. jmx2_rmf_grp_0005.fits)
;  test that it matches the ARFc, at least at energies above 10 keV except
;  for a factor - the Crab could be observed off-axis.
;
;  This latter ARF (ARFc) is the starting array for the new adaption.
;
;  Create a new file with the resulting ARF (the array 'eflt' multiplied with
;  ARFc) and update the the column 'ANCRFILE' in the spectral file with the
;  new ARF file. The resulting ARF is also stored into a copy of the RMF-GRP
;  and the keyword 'RESPFILE' in the spectral file is updated to the new RMF-GRP
;
;  The names of these files are stored in 'make_arf.par'
;  and can be read from there.
;  The vignetting factor found is stored in 'make_arf.par'
;
;  Keyword 'noplot' will prevent all plotting when set
;
;  Contents of 'make_arf.par' :
;
;   crab_spe_file : spectral file such as jmx2_srcl_spe.fits
;   crab_arf_file : arf file such as jmx2_srcl_arf.fits
;                     Both of the above two files are produced by j_src_spectra
;   rmf_grp_file  : the RMF.-GRP file used by j_src_spectra
;
;   new_arf_file  : new ARF file - product of 'make_arf'
;   new_eflt_file : by-product of 'make_arf', gives the energy dependent ratio
;                   between new_arf and crab_arf
;   new_rmf_grp_file : copy of rmf_grp_file except that column SPECRESP has
;                   been exchanged with new_arf/vignetting
;

if keyword_set(help) then begin
   print,'Syntax: make_arf[,/help][,/dialog][,/noplot]'
   print,' Keyword ''dialog'' will display default filenames etc.'
   print,' Keyword ''noplot'' will prevent all plotting.'
   print
   return
endif

common instrument, e_min, e_max, matrix, arf
common observation, rate, stat_err, countspec, number
common model, elo, ehi, eline, spectrcode
common results, const, param1, nh, chi2red, sx
common rmf, rmf_grp_file

if keyword_set(dialog) then begin
   crab_spe_file = get_pars('make_arf.par','crab_spe_file')
   print
   crab_arf_file = get_pars('make_arf.par','crab_arf_file')
   print
   new_arf_file = get_pars('make_arf.par','new_arf_file')
   print
   rmf_grp_file = get_pars('make_arf.par','rmf_grp_file')
   print
   new_eflt_file = get_pars('make_arf.par','new_eflt_file')
   print
   new_rmf_grp_file = get_pars('make_arf.par','new_rmf_grp_file')
endif else begin
   crab_spe_file = comgets('make_arf.par','crab_spe_file',nval)
   crab_arf_file = comgets('make_arf.par','crab_arf_file',nval)
   new_arf_file = comgets('make_arf.par','new_arf_file',nval)
   rmf_grp_file = comgets('make_arf.par','rmf_grp_file',nval)
   new_eflt_file = comgets('make_arf.par','new_eflt_file',nval)
   new_rmf_grp_file = comgets('make_arf.par','new_rmf_grp_file',nval)
endelse

pflag = 1 - keyword_set(noplot)

p = strpos(crab_spe_file,'jmx1')
if p ge 0 then begin
   strj = '1'
endif else begin
   p = strpos(crab_spe_file,'jmx2')
   if p ge 0 then begin
      strj = '2'
   endif else begin
      strj = ''
      read,'Enter JEMX number : ... ', strj
   endelse
endelse

; Derive the vignetting factor applied by j_src_spectra
; Read the ARF in the 'crab_arf_file', compare the high
; energy values (that are unchanged in the fitting process)
; with the ones from the RMF file - also check that the
; ratio is constant in the chosen range
;
; Assume that the Crab spectrum is the first spectrum in the file

rdfitscol,crab_arf_file+'+1',arfca,'SPECRESP' ; Get the ARF
sz = size(arfca)
if sz(0) eq 2 then arfc = arfca(*,0) else arfc = arfca

rdfitscol,rmf_grp_file+'+4', arfr, 'SPECRESP' ; Get ARF from RMF file

; Determine the ratio between the arfr and arfc

ratio = arfc(205:*) / arfr(205:*)
avgrat = avg(ratio)
dmax =  max(abs(ratio-avgrat))
if dmax gt 1.e-3 then begin
   print,'Warning: deviation from constant ratio is ', dmax
endif else begin
   print,'Constant ratio has been verified'
endelse

; Update the parameter file 'make_arf.par'

res = comreplace('make_arf.par','vignetting',avgrat)






; Read the spectral information

rdfitscol,crab_spe_file+'+1',rate,'RATE'
rdfitscol,crab_spe_file+'+1',stat_err,'STAT_ERR'

; Define energy boundaries:

rmfgroup = rmf_grp_file+'+1'
ebm_get_child_by_name,rmfgroup,'JMX'+strj+'-FBDS-MOD',fbds_name
rdfitscol,fbds_name,e_min,'E_MIN',/sil,errmsg=errmsg
rdfitscol,fbds_name,e_max,'E_MAX',/sil,errmsg=errmsg
if errmsg ne '' then begin
   print,'##7##'
   print,errmsg
   return
endif
ebm_get_child_by_name,rmfgroup,'JMX'+strj+'-RMF.-RSP',rmf_name
rdfitscol,rmf_name,elo,'ENERG_LO',/sil,nrows=n_eline
rdfitscol,rmf_name,ehi,'ENERG_HI',/sil
rdfitscol,rmf_name,matrix,'MATRIX',/sil
eline = sqrt(elo*ehi)

rebin = indgen(200-45) + 45

e_min = e_min(rebin)
e_max = e_max(rebin)
matrix = matrix(rebin,*)
;===================================================================


rate = rate(rebin)
stat_err = stat_err(rebin)

if pflag then begin

   ; Plot the original data

   makewin,0,tit='Original data'

   dataplot, sqrt(e_min*e_max),rate/(e_max-e_min),$
          stat_err/(e_max-e_min),/xbar,itype=3,$
          xtit='Energy [keV]'     ;,yr=[1.e-2,10]
endif


;   Define the spectral model

spectrcode = 'PL'

const = 9.7
param1 = 2.1
nh = 0.265 ; in units of 1.e22
sigma = 0.4
eflt = replicate(1.,500)

;   Generate the input spectrum

flux = arffit_photspec([const,param1,nh])
bcnts = flux * (ehi - elo) * arfr

c2base = chi2rel(matrix#(bcnts*eflt), rate, stat_err)
efltmin = eflt
print,'c2base = ', c2base


;* eflt = mod_array( eline, eflt, sig=sig, x_ch=e_ch)

nxl = 20
xl = xloggen(nxl, 2., 12.)
dxl = delta(xl)

c2min = c2base
fkmin = 1.0
; Find index of 'eline' for 3 keV
dummy = min(abs(eline - 3.0), i3kev)
; Define smallest slope of ARF below 3 keV
eta = 1.0

if pflag then begin
   makewin,1,tit='EFLT'
   makewin,2,tit='Data with model'
endif

repeat begin
   c2min_prev = c2min

   rxl = xl + (randomu(seed,nxl)-0.5)*0.9*dxl

   for ampl = -0.02, 0.02001, 0.04 do begin
      for i = 0, nxl-1 do begin
      
         eflt1 = add_peak_1d( eline, eflt, rxl(i), sigma*(rxl(i)/3.)^0.9, ampl)
         ; Ensure a minimum slope of 'eta':
         arfeflt = arfr * eflt1
         for j = i3kev, 1, -1 do begin
            arfeflt(j-1) = arfeflt(j-1) < arfeflt(j)*(eline(j-1)/eline(j))^eta
            eflt1(j-1) = arfeflt(j-1) / arfr(j-1)
         endfor
         c2 = chi2rel(matrix#(bcnts*eflt1), rate, stat_err, factor=fk)
         if c2 lt c2min then begin
            print,'i c2 fk = ', i, c2, fk
            c2min = c2
            imin = i
            efltmin = eflt1
            fkmin = fk
            if pflag then begin
               wset,1
               plot_oi, eline,eflt1,xtit='Energy [keV]',yr=[0.9,1.3],xr=[2,20]
               wset,2
               dataplot, sqrt(e_min*e_max),rate/(e_max-e_min),$
                   stat_err/(e_max-e_min),/xbar,itype=3,$
                   xtit='Energy [keV]'     ;,yr=[1.e-2,10]
               oplot, sqrt(e_min*e_max), fk*(matrix#(bcnts*eflt1))/(e_max-e_min),ps=10
            endif
         endif
         ;ans = ''
         ;read,'Next ...',ans
         ;* wait,0.1
      endfor
   endfor

   ; Start over with updated 'eflt'

   eflt = efltmin

endrep until (c2min_prev - c2min)/c2min_prev lt 1.e-3

countspec = fkmin * matrix#(bcnts*efltmin)

if pflag then begin
   wset,1
   plot_oi, eline, efltmin, xtit='Energy [keV]',yr=[0.9,1.3],xr=[2,20]
   wset,2
   dataplot, sqrt(e_min*e_max),rate/(e_max-e_min),$
       stat_err/(e_max-e_min),/xbar,itype=3,$
       xtit='Energy [keV]'     ;,yr=[1.e-2,10]
   oplot, sqrt(e_min*e_max), countspec/(e_max-e_min),ps=10
endif

wrmfitscols, new_eflt_file, 'EFLT', efltmin

k = total(countspec*rate/stat_err^2) / total(countspec^2/stat_err^2)



; Copy rmf_grp_file to new_rmf_grp_file

spawn,'cp '+rmf_grp_file+' '+new_rmf_grp_file

; Update ARF values in new_rmf_grp_file (ext#4, column 3)

newarf = fkmin * efltmin * arfr

fxbopen,unit,new_rmf_grp_file,4,access='RW'
for row = 1, 500 do fxbwrite,unit,newarf(row-1)/avgrat,3,row
fxbfinish, unit

; Copy crab_arf_file to new_arf_file
; This will generate the appropriate file structure

spawn,'cp '+crab_arf_file+' '+new_arf_file

; Update ARF values in new_arf_file

fxbopen,unit,new_arf_file,1,access='RW'
colnum = fxbcolnum(unit,'SPECRESP')
fxbwrite,unit,newarf,colnum,1
fxbfinish, unit

; Update ANCRFILE name in crab_spe_file

fxbopen,unit,crab_spe_file,1,access='RW'
colnum = fxbcolnum(unit,'ANCRFILE')
fxbwrite,unit,new_arf_file+'{1}',colnum,1
fxbfinish,unit

; Update RESPFILE to new_rmf_grp_file

fxhmodify,crab_spe_file,'RESPFILE',new_rmf_grp_file,'New RMF-GRP file',ext=1

print
print,'Now you are ready to do an XSPEC fit on '+crab_spe_file
print,'to check the effect of making a new EFLT array'
print


end
