function manip_lc, lc_in, len_min, len_avg, hori=hori
;
;  A function to manipulate a light curve in the sense that
;  stretches with zero values are converted to a linear connection
;  between the averages to the left and to the right of these
;  stretches. A stretch is here one or more consecutive zeros
;  as indicated with parameter 'len_min'.
;
;   2003-03-10/NJW
;
if n_params() lt 1 then begin
   print,'Syntax: new_lc = manip_lc(lc, len_min, len_avg[, /hori])'
   return,-1
endif

n_in = n_elements(lc_in)
in_stretch = 0
num_stretch = 0
first_start = 1
first_stop = 1

for i = 0L, n_in - 1 do begin
   ;
   ;  First cover case where value is zero
   ;  i.e. check for start of new stretch or
   ;  continuation
   ;
   if lc_in(i) eq 0 then begin
      if in_stretch then goto, next
      in_stretch = 1
      num_stretch = num_stretch + 1
      if first_start then begin
         start_pos = [i]
         first_start = 0
      endif else start_pos = [start_pos,i]
      goto, next
   endif
   ;
   ;  Then cover case where value is non-zero
   ;  i.e. check for stop of stretch or
   ;  continuation
   ;
   if not in_stretch then goto, next
   if first_stop then begin
      stop_pos = [i-1]
      first_stop = 0
   endif else stop_pos = [stop_pos,i-1]
   in_stretch = 0

   next:
endfor

; Check on finishing while still 'in stretch'

if in_stretch then stop_pos = [stop_pos,i-1]


; All consecutive stretches with zeros have
; been mapped now. Eliminate the shorter ones

i = 0L
while i lt num_stretch do begin
   len = stop_pos(i) - start_pos(i) + 1
   if len lt len_min then begin
      for j = i, num_stretch - 2 do begin
         start_pos(j) = start_pos(j+1)
         stop_pos(j) = stop_pos(j+1)
      endfor
      num_stretch = num_stretch - 1
   endif else i = i + 1
endwhile

print,'num_stretch = '+stri(num_stretch)
for i = 0L, num_stretch-1 do begin
   print,'start: '+stri(start_pos(i))+'   stop: '+stri(stop_pos(i))
endfor

if num_stretch eq 0 then return, lc_in

;  For remaining stretches the linear connection is to be made

lc = lc_in

for i = 0L, num_stretch-1 do begin

   ; Left anchor point

   if i eq 0 then begin
      if start_pos(i) eq 0 then left_value = 0.0 else begin
         i1 = start_pos(i) - len_avg
         if i1 lt 0 then i1 = 0
         left_value = avg(lc(i1:start_pos(i)-1))
      endelse
   endif else begin
      i1 = start_pos(i) - len_avg
      if i1 le stop_pos(i-1) then i1 = stop_pos(i-1)+1
      left_value = avg(lc(i1:start_pos(i)-1))
   endelse
      

   ; Right anchor point

   if i eq num_stretch-1 then begin
      if stop_pos(i) eq n_in-1 then right_value = 0.0 else begin
         i2 = stop_pos(i) + len_avg
         if i2 gt n_in-1 then i2 = n_in-1
         right_value = avg(lc(stop_pos(i)+1:i2))
      endelse
   endif else begin
      i2 = stop_pos(i) + len_avg
      if i2 ge start_pos(i+1) then i2 = start_pos(i+1)-1
      right_value = avg(lc(stop_pos(i)+1:i2))
   endelse

   ; Make linear connection

   if keyword_set(hori) then begin
      lc(start_pos(i):stop_pos(i)) = 0.5*( left_value + right_value )
   endif else begin
      slope = (right_value - left_value) / (stop_pos(i)+1 - start_pos(i)+1)
      lc(start_pos(i):stop_pos(i)) = left_value $
          + slope * (indgen(stop_pos(i) - start_pos(i) +1) + 1)
   endelse

endfor

return, lc

end
