pro mk_reg_input_srcs, radeclist_fits_file,$
       sxmin=sxmin, sxmax=sxmax, radsel=radsel, sxmintext=sxmintext
;
;   Make regions for SAOImage(ds9) image display with sources
;   from a FITS catalog with columns 'NAME|SOURCENAME','RA|RA_OBJ',
;     'DEC|DEC_OBJ', and (optionally 'INTENS'
;
;   2003-09-18/NJW   (updated)
;
; Read a file 'radec_list.fits' RA in dec. deg., DEC in dec.deg., NH,
;   Intensity code, Spectral type, Intensity, Sp.parm (a single)
;
;
if n_params() lt 1 then begin
   print,'Syntax: mk_reg_input_srcs, radeclist_fits_file_extno, $'
   print,'            [,sxmin=][,sxmax][,radsel=][,sxmintext=]'
   print,'  where  sxmin is'
   print,'         sxmax is'
   print,'        radsel is a radius selection 3-element array: [ra,dec,radius] in deg'
   print
   print,' Output file will be reg_????.reg'
   return
endif

get_exten_no, radeclist_fits_file, filename, extno

if not keyword_set(sxmintext) then sxmintext = 1.e-10

;  Read the input data

errmsg=''
rdfitscol,radeclist_fits_file,ra_deg,'RA',errmsg=errmsg
if errmsg ne '' then begin
   errmsg=''
   rdfitscol,radeclist_fits_file,ra_deg,'RA_OBJ',errmsg=errmsg
   if errmsg ne '' then begin ; give up
      print,'Not possible to get RA(_OBJ) '+errmsg
      return
   endif
endif
rdfitscol,radeclist_fits_file,dec_deg,'DEC',errmsg=errmsg
if errmsg ne '' then begin
   errmsg=''
   rdfitscol,radeclist_fits_file,dec_deg,'DEC_OBJ',errmsg=errmsg
   if errmsg ne '' then begin ; give up
      print,'Not possible to get DEC(_OBJ) '+errmsg
      return
   endif
endif
rdfitscol,radeclist_fits_file,intens,'INTENS',errmsg=errmsg
if errmsg ne '' then begin
   errmsg=''
   print,'No INTENS column; assign 1.e-9'
   ncat = n_elements(ra_deg)
   intens = fltarr(ncat) + 1.e-9
endif
rdfitscol,radeclist_fits_file,sourcename,'SOURCENAME',errmsg=errmsg
if errmsg ne '' then begin
   errmsg=''
   rdfitscol,radeclist_fits_file,sourcename,'NAME',errmsg=errmsg
   if errmsg ne '' then begin ; give up
      print,'Not possible to get (SOURCE)NAME '+errmsg
      sourcename = replicate('?',ncat)
      errmsg = ''
   endif
endif

if keyword_set(radsel) then begin
   sz = size(radsel)
   if sz(0) ne 1 or sz(1) ne 3 then begin
      print,'keyword ''radsel'' must be 3 element array'
      return
   endif
   r = arcdist(ra_deg,dec_deg,radsel(0),radsel(1),/deg)
   w = where( r lt radsel(2), nw)
   if nw le 0 then begin
      print,'No sources survived the radius selection'
      return
   endif
   ra_deg = ra_deg(w)
   dec_deg = dec_deg(w)
   intens = intens(w)
   sourcename = sourcename(w)
endif

nsrc = n_elements(ra_deg)

;
; Make logarithmic size so that sources with fx < 2.e-11
; are excluded; size 0.2 for fx = 2.e-11
;               size 3.0 for fx = 1.e-9
; and logarithmic distribution
;

size0 = 0.06; *10
size1 = 0.30; *10
fmin = 1.e-11
if keyword_set(sxmin) then fmin = sxmin
fmax = 2.e-9
if keyword_set(sxmax) then fmax = sxmax
bcoef = (size1 - size0)/alog10(fmax/fmin)
acoef = size0 - bcoef * alog10(fmin)

; Find allowed sources

is = where(intens ge fmin, n_is)
if n_is eq 0 then begin
   print,'MK_REG_INPUT_SRCS: Sorry, no sources >= ',fmin
   return
endif
ra_deg = ra_deg(is)
dec_deg = dec_deg(is)
intens = intens(is)
sourcename = sourcename(is)


; Compose new name for region file.

regname = get_next_filename('reg_????.reg')
   

openw,lun,regname,/get_lun

printf,lun,'#   Region file composed by mk_reg_input_srcs.pro '+systime()
printf,lun,'#   Source list: '+radeclist_fits_file
if keyword_set(radsel) then begin
   printf,lun,'#  Selection centered on RA '+stri(radsel(0))+' DEC '+stri(radsel(1))
   printf,lun,'#  with radius '+stri(radsel(2))+' deg'
endif else printf,lun,'# No selection'
printf,lun,'#'

for i = 0,n_is-1 do begin
   psz = acoef + bcoef * alog10(intens(i))
   printf,lun,'fk5;circle('+stri(ra_deg(i))+','+stri(dec_deg(i))+','+stri(psz)+')'
   if intens(i) gt sxmintext then $
   printf,lun,'fk5;text('+stri(ra_deg(i))+','+stri(dec_deg(i))+ $
           ') # text={'+stri(sourcename(i))+'}'
;physical;text(46,246) # color=black width=3 font="helvetica 14 normal" text={mosaic}
endfor

;  Text for field identification

;* printf,lun,'+text(5,195,"'+base+'")'

free_lun,lun

end

