pro mosaicq, h=h, imlist=imlist, outfile=outfile, diameter=diameter, exten=exten
;
;  A procedure to produce a mosaic image from a
;  number of FITS images
;
;  If keywords 'imlist' has been defined then the input
;  files will be taken from that, else an interactive
;  dialog will be startet
;
;  If keyword 'outfile' has been defined then this is
;  updated with images from 'image_list'
;
;  This version tries to work with implied do-loops as much
;  as possible
;
;  2003-09-16/NJW
;
if keyword_set(h) then begin
   print,'Syntax: mosaicq[, imlist=][, outfile=][, diameter=][,exten=]'
   print,'  imlist   string array with list of images (including extno)'
   print,'  outfile  if given then this file is updated'
   print,'  diameter is size of output image in degrees (default is 15 deg)'
   print,'  exten    is extension number of selected files (default is 1)'
   return
endif

if keyword_set(imlist) then begin
   image_list = imlist
endif else begin
   image_list = dialog_pickfile(path='/r6/jemx', $
           filter='*.fits',/multiple_files)
   if keyword_set(exten) then begin
      image_list = image_list+'+'+stri(exten)
   endif else begin
      image_list = image_list+'+1'
   endelse
   printl,image_list
   answer = ''
   read,'Continue ? ... ', answer
   if answer ne 'y' and answer ne '' then return
endelse
nlist = n_elements(image_list)

dr = !pi / 180
isub = 2
nsub = 2*isub + 1 ; produce an odd number
dsub = 1./nsub
reduc = dsub*dsub

if not keyword_set(diameter) then diameter = 15. ; degrees

; Walk through list to extract basic information

k = 0
for i = 0, nlist-1 do begin
   get_exten_no, image_list(i), filename, extno
   l = file_search(filename,/test_reg,count=nl)
   if nl ne 1 then begin
      print,'File '+filename+' is missing, skip and continue'
      continue
   endif
   hdr = headfits(filename,ext=extno)
   sz = size(hdr)
   if sz(sz(0)+1) ne 7 then begin
      print,'Header of '+image_list(i)+' not retrievable'
      continue
   endif
   if k eq 0 then begin
      filelist = [filename]
      extlist = [extno]
      crval1 = [fxpar(hdr,'crval1')]
      crval2 = [fxpar(hdr,'crval2')]
      crpix1 = [fxpar(hdr,'crpix1')]
      crpix2 = [fxpar(hdr,'crpix2')]
      cd1_1 = [fxpar(hdr,'cd1_1')]
      cd1_2 = [fxpar(hdr,'cd1_2')]
      cd2_1 = [fxpar(hdr,'cd2_1')]
      cd2_2 = [fxpar(hdr,'cd2_2')]
   endif else begin
      filelist = [filelist,filename]
      extlist = [extlist,extno]
      crval1 = [crval1,fxpar(hdr,'crval1')]
      crval2 = [crval2,fxpar(hdr,'crval2')]
      crpix1 = [crpix1,fxpar(hdr,'crpix1')]
      crpix2 = [crpix2,fxpar(hdr,'crpix2')]
      cd1_1 = [cd1_1,fxpar(hdr,'cd1_1')]
      cd1_2 = [cd1_2,fxpar(hdr,'cd1_2')]
      cd2_1 = [cd2_1,fxpar(hdr,'cd2_1')]
      cd2_2 = [cd2_2,fxpar(hdr,'cd2_2')]
   endelse
   k = k + 1
endfor

for i = 0, k-1 do begin
   print,'crval: ', crval1(i), crval2(i)
   ;print,'crpix: ', crpix1(i), crpix2(i)
   ;print,'cd1_1 cd1_2: ', cd1_1(i), cd1_2(i)
   ;print,'cd2_1 cd2_2: ', cd2_1(i), cd2_2(i)
endfor

avg_crval1 = avg(crval1)
avg_crval2 = avg(crval2)

; Define the output image

if keyword_set(outfile) then begin ; Get info from existing file

   hdr = headfits(outfile,exten=1)
   outimage = readfits( outfile, hdr, exten=1 )
   outexpos = readfits( outfile, exten=3 )

   crval1_o = fxpar(hdr,'CRVAL1')
   crval2_o = fxpar(hdr,'CRVAL2')
   crpix1_o = fxpar(hdr,'CRPIX1')
   crpix2_o = fxpar(hdr,'CRPIX2')
   ndimx    = fxpar(hdr,'NAXIS1')
   ndimy    = fxpar(hdr,'NAXIS2')
   cd1_1_o  = fxpar(hdr,'CD1_1')
   cd1_2_o  = fxpar(hdr,'CD1_2')
   cd2_1_o  = fxpar(hdr,'CD2_1')
   cd2_2_o  = fxpar(hdr,'CD2_2')

endif else begin  ;   Define the information

   crval1_o = avg_crval1
   crval2_o = avg_crval2
   cdelt = 0.03 ; degrees per pixel
   ndimx = fix(diameter / cdelt)
   ndimy = fix(diameter / cdelt)
   crpix1_o = float(ndimx)/2
   crpix2_o = float(ndimy)/2
   cdelt = diameter / ndimx ; degrees
   cd1_1_o = - cdelt
   cd1_2_o = 0.0
   cd2_1_o = 0.0
   cd2_2_o =   cdelt

   outimage = fltarr(ndimx,ndimy)
   outexpos = fltarr(ndimx,ndimy)

endelse

print,'Center for output image: ', crval1_o, crval2_o

cent_dist = arcdist( crval1, crval2, crval1_o, crval2_o, /deg)

cdmat_o = [[1,0,0],[0,cd1_1_o, cd2_1_o],[0,cd1_2_o, cd2_2_o]]
o_tamdc = invert(cdmat_o)
ca = cos(crval1_o*dr)
sa = sin(crval1_o*dr)
cd = cos(crval2_o*dr)
sd = sin(crval2_o*dr)
t1_o = [[ca,sa,0],[-sa,ca,0],[0,0,1]]
t2_o = [[cd,0,sd],[0,1,0],[-sd,0,cd]]
t_o = t1_o # t2_o
o_t = invert(t_o)

; Compose mosaic by looping over elementary images

for i = 0, k-1 do begin
   print,'Including: '+filelist(i)
   print,'Distance center to center = ', cent_dist(i),' deg'
   if cent_dist(i) gt diameter*0.5 + 2.5 then begin
      print,'Too far away - skip ...'
      continue
   endif

   ; transformation array
   cdmat = [[1.,0.,0.,],[0.,cd1_1(i),cd2_1(i)],[0.,cd1_2(i),cd2_2(i)]]
   ca = cos(crval1(i)*dr)
   sa = sin(crval1(i)*dr)
   cd = cos(crval2(i)*dr)
   sd = sin(crval2(i)*dr)
   t1 = [[ca,sa,0],[-sa,ca,0],[0,0,1]]
   t2 = [[cd,0,sd],[0,1,0],[-sd,0,cd]]
   t = t1 # t2
   o_tt = o_t # t
   im = readfits(filelist(i),exten=extlist(i),/silent)
   im = im * reduc
   szim = size(im)
   ; loop over individual pixels
   ;for p = 0, szim(1)-1 do begin
   ;for q = 0, szim(2)-1 do begin
   ;   for psub = -isub,isub do begin
   ;   for qsub = -isub,isub do begin
   n = long(szim(1))
   g = (findgen(n*nsub)-2.)*dsub
   dbl_loop,g,q,p
   ar = fltarr(3,n*n*nsub*nsub)
   ar(1,*) = p + 1. - crpix1(i)
   ar(2,*) = q + 1. - crpix2(i)
   yz = cdmat # ar
   ayz = atan(yz*dr)
   vi = sin(ayz)
   vi(0,*) = sqrt(1. - vi(1,*)^2 - vi(2,*)^2)
   vi_o = o_tt # vi
   ayz = asin(vi_o)
   yz = tan(ayz) / dr
   pix = o_tamdc # yz
   pix0 = pix(1,*) + crpix1_o - 1.
   pix1 = pix(2,*) + crpix2_o - 1.
   w = where( pix0 ge 0 and pix0 lt ndimx and $
              pix1 ge 0 and pix1 lt ndimy, nw)
            outimage(pix(0),pix(1)) = outimage(pix(0),pix(1)) $
   	         + im(p,q)
            outexpos(pix(0),pix(1)) = outexpos(pix(0),pix(1)) $
   	         + 1.0
         endif
endfor

if keyword_set(outfile) then begin

   header = hdr

endif else begin

   fxhmake_im,header,outimage,/extend
   fxaddpar,header,'EXTNAME','RAWMOSAIC','Simply added intensities'
   fxaddpar,header,'DATE',systime(),'Date of file creation'
   fxaddpar,header,'TELESCOP','INTEGRAL','Mission name'
   fxaddpar,header,'INSTRUME','JMX2','Instrument'
   fxaddpar,header,'CREATOR','mosaic.pro',$
   	'Executable that created this extension'
   ;fxaddpar,header,'REVOL',rev,'INTEGRAL revolution number'
   ;fxaddpar,header,'PID',pid,'Pointing identification'
   ;if keyword_set(emin) then begin
   ;   fxaddpar,header,'E_MIN',emin,'[keV] Lower energy limit'
   ;endif
   ;if keyword_set(emax) then begin
   ;   fxaddpar,header,'E_MAX',emax,'[keV] Upper energy limit'
   ;endif
   fxaddpar,header,'CTYPE1','RA---TAN','Projection name'
   fxaddpar,header,'CTYPE2','DEC--TAN','Projection name'
   fxaddpar,header,'CRVAL1',crval1_o,'[deg] Right Ascension'
   fxaddpar,header,'CRVAL2',crval2_o,'[deg] Declination'
   fxaddpar,header,'POSANGLE',0.0, $
   	'[deg] Position angle (same as roll angle)'
   fxaddpar,header,'IMATYPE','RAWMOSAIC','Simply added Intensity (counts)'
   fxaddpar,header,'CRPIX1',crpix1_o,'[pixel] Reference pixel axis 1'
   fxaddpar,header,'CRPIX2',crpix2_o,'[pixel] Reference pixel axis 2'
   fxaddpar,header,'CUNIT1','deg','Unit of axis 1'
   fxaddpar,header,'CUNIT2','deg','Unit of axis 2'
   fxaddpar,header,'CD1_1',cd1_1_o,'[deg] Transformation'
   fxaddpar,header,'CD1_2',cd1_2_o,'[deg] Transformation'
   fxaddpar,header,'CD2_1',cd2_1_o,'[deg] Transformation'
   fxaddpar,header,'CD2_2',cd2_2_o,'[deg] Transformation'
   fxaddpar,header,'NSUBDIV',nsub*nsub,'Number of pixel subdivisions'

endelse

for i = 0, k-1 do begin
   fxaddpar,header,'COMMENT','Includes '+filelist(i)+ $
	   '+'+stri(extlist(i))
endfor

; Write FITS file to local directory

mosname = get_next_filename('/r6/jemx/njw/mosaic/mosaic_???.fits')

fits_make_prim,mosname

fxaddpar,header,'IMATYPE','RAWMOSAIC','Simply added intensity (counts)'
fxaddpar,header,'EXTNAME','RAWMOSAIC','Simply added intensity (counts)'
writefits,mosname,outimage,header,/append

w = where( outexpos gt 0.0, nw )
if nw gt 0 then begin
   outimage(w) = outimage(w) / outexpos(w)
endif
fxaddpar,header,'IMATYPE','INTENSITY','Intensity (counts)'
fxaddpar,header,'EXTNAME','INTENSITY','Intensity (counts)'

writefits,mosname,outimage,header,/append

fxaddpar,header,'IMATYPE','EXPOSURE','Exposure (seconds)'
fxaddpar,header,'EXTNAME','EXPOSURE','Exposure (seconds)'

writefits,mosname,outexpos,header,/append

print,'Resulting FITS file: ', mosname

end
