pro mw, ra0, dec0, radius, ecode, outdir, rev=rev, tstart=tstart,$
    tstop=tstop, ori=ori, rmax=rmax, diameter=diameter, mwname=mwname, nlist=nlist, $
    logfile=logfile, n_inc=n_inc, maps=maps, r_w=r_w, pvsur=pvsur
;
;  This is a IDL wrapper for Jerome's mosaic_weight
;  program from 2004-02-06 (with option for searching for both
;    'EXPOSURE' and 'ONTIME' keywords)
;
;  ra0 and dec0 will be the center coordinates of the resulting image.
;  'radius' is the limit for including pointings
;  'rmax' is the radius limit in individual input images
;  'diameter' is the diameter of the output mosaic image
;            All radii and diameters are in degrees
;  

if n_params() lt 5 then begin
   print,'Syntax: mw, ra0, dec0, radius[deg], ecode, outdir'
   print
   print,'Parameters:'
   print,'       ra0       : RA [deg] for center of mosaic image'
   print,'       dec0      : DEC [deg] for center of mosaic image'
   print,'       radius    : [deg] radius for selection of pointings'
   print,'       ecode     : (integer) energy code as used by NL or CBJ'
   print,'       outdir    : (string) Directory where mosaic files are placed'
   print
   print,'Keywords:'
   print,' input      rev  :  (int) scalar or two-element array, revolution number'
   print,'                 or (string) such as ''45-48,52,55,58'''
   print,' input    tstart :  (float or double) IJD time start'
   print,' input    tstop  :  (float or double) IJD time stop '
   print,' input     ori   :  (string) ''C''(default) CBJ images 525 x 525 pixels'
   print,'                             ''c'' CBJ images 262 x 262 pixels'
   print,'                             ''nr'' raw NL images 255 x 255 pixels'
   print,'                             ''nn'' no sources NL images 255 x 255 pixels'
   print,'                             ''ns'' cleaned + sources NL images 255 x 255 pixels'
   print,' input    rmax   :  (float) max radius [deg] to use in input images
   print,' input  diameter :  (float) diameter [deg] of output mosaic image
   print,' input  maps     :  (string) 8 characters ''y'' or ''n'' whether to'
   print,'                             produce: exposure time file, raw intens.,'
   print,'                             eff. exp., var., sumw, sumw2,sig/exp,'
   print,'                             sqrt(var)/exp - default: ''nynnnnnn'''
   print,' input  r_w      :  (/r_w)   If the ranked_weights.txt file is to be kept'
   print,' input  pvsur    :  (/pvsur) Limits the SWIDs used to come from group PV or SURVEY
   print,' return  mwname  :  (string) resulting mosaic FITS file'
   print,' return  nlist   :  (int) resulting number of pointings'
   print,' return  n_inc   :  (int) resulting number of images included'
   print,' return logfile  :  (string) resulting logfile name'
   return
endif

; Define used rmax

if not keyword_set(rmax) then rmax = 5.0

; Define used diameter

if not keyword_set(diameter) then diameter = 15.0

; Look for required files in current directory
; If not present then setup a link

njwdir = '/home/njw/jemx/mosaic/src/'
f1name = 'cbj_vig.dat'
f2name = 'vignet_carl_2+.txt'
f3name = 'illum_k-2.txt'
f4name = 'JMX2-SKY.-IMA.tpl'
f5name = 'JMX2-SKY.-IMA-IDX.tpl'
f6name = 'vignet_262_2+.txt'
f7name = 'vignet_525_2+.txt'
f8name = 'bckim_1_5.txt'
if not file_test(f1name,/reg) then spawn,'ln -s '+njwdir+f1name
if not file_test(f2name,/reg) then spawn,'ln -s '+njwdir+f2name
if not file_test(f3name,/reg) then spawn,'ln -s '+njwdir+f3name
if not file_test(f4name,/reg) then spawn,'ln -s '+njwdir+f4name
if not file_test(f5name,/reg) then spawn,'ln -s '+njwdir+f5name
if not file_test(f6name,/reg) then spawn,'ln -s '+njwdir+f6name
if not file_test(f7name,/reg) then spawn,'ln -s '+njwdir+f7name
if not file_test(f8name,/reg) then spawn,'ln -s '+njwdir+f8name

if not file_test(f1name,/reg) then begin
   print,'Error - link to '+f1name+' failed'
   return
endif
if not file_test(f2name,/reg) then begin
   print,'Error - link to '+f2name+' failed'
   return
endif
if not file_test(f3name,/reg) then begin
   print,'Error - link to '+f3name+' failed'
   return
endif
if not file_test(f4name,/reg) then begin
   print,'Error - link to '+f4name+' failed'
   return
endif
if not file_test(f5name,/reg) then begin
   print,'Error - link to '+f5name+' failed'
   return
endif
if not file_test(f6name,/reg) then begin
   print,'Error - link to '+f6name+' failed'
   return
endif
if not file_test(f7name,/reg) then begin
   print,'Error - link to '+f7name+' failed'
   return
endif

;    Check which maps to produce

if not keyword_set(maps) then maps = 'nynnnnnn'
szmap = size(maps)
if szmap(0) ne 0 or szmap(1) ne 7 then begin
   print,'Error - bad keyword ''maps'''
   return
endif
len = strlen(maps)
if len lt 8 then begin
   for i = 0, 8 - len - 1 do maps = maps + 'n'
endif

; Define origin of images

nl_ext = 1 ; Must be set - no effect if Carl's images are chosen
origin = 2 ; Carl's new images, default
if keyword_set(ori) then begin
   if strmid(ori,0,1) eq 'c' then origin = 0
   if strmid(ori,0,1) eq 'n' or strmid(ori,0,1) eq 'N' then begin
      origin = 1
      if strlen(ori) ge 2 then begin
         if strmid(ori,1,1) eq 'n' then nl_ext = 2
         if strmid(ori,1,1) eq 's' then nl_ext = 3
      endif
   endif
endif


;    Create the parameter file: mosaic_weight.par


openw,lpar,'mosaic_weight.par',/get_lun

printf,lpar,'1 skymaps.lst'
printf,lpar,'2 '+stri(rmax)
printf,lpar,'3 '+stri(diameter)
printf,lpar,'31 '+stri(ra0)
printf,lpar,'32 '+stri(dec0)
printf,lpar,'101 '+stri(nl_ext)
for i = 0, 7 do begin
   if strmid(maps,i,1) eq 'y' then begin
      printf,lpar,stri(201+i)+' Y'
   endif else begin
      printf,lpar,stri(201+i)+' N'
   endelse
endfor

free_lun, lpar




loglist = strarr(2000)
l = 0
loglist(l) = '// IDL wrapper for mosaic_weight: mw run at '+systime()
l=l+1 & loglist(l) = '// ra0 = '+stri(ra0)
l=l+1 & loglist(l) = '// dec0 = '+stri(dec0)
l=l+1 & loglist(l) = '// radius = '+stri(radius)
l=l+1 & loglist(l) = '// rmax = '+stri(rmax)
l=l+1 & loglist(l) = '// diameter = '+stri(diameter)
l=l+1 & loglist(l) = '// ecode = '+stri(ecode)
l=l+1 & loglist(l) = '// nl_ext = '+stri(nl_ext)
l=l+1 & loglist(l) = '// maps = '+maps
if keyword_set(rev) then begin
   l=l+1
   loglist(l) = '// rev ='+stri(rev)
endif
if keyword_set(tstart) then begin
   l=l+1
   loglist(l) = '// tstart ='+stri(tstart)
endif
if keyword_set(tstop) then begin
   l=l+1
   loglist(l) = '// tstop ='+stri(tstop)
endif


n_included = 0

find_swid_radec, ra0, dec0, radius, rev=rev, tstart=tstart,$
   tstop=tstop, list=list, /nof, /sil

print,list
nlist = n_elements(list)
if nlist eq 1 and list(0) eq '' then nlist = 0

if nlist gt 0 then begin
   if keyword_set(pvsur) then begin ; Filter out all private data
      l=l+1 & loglist(l) = '// Using only PV and SURVEY data'
      group = swid_is_pvsurvey(list)
      w = where( group gt 0, nw )
      if nw gt 0 then list = list(w)
      nlist = nw
   endif else begin
      l=l+1 & loglist(l) = '// Using all available images'
   endelse
endif
   

print,nlist,' possible pointings (SWIDs)'

l=l+1 & loglist(l) = '// nlist = '+stri(nlist)
if nlist eq 0 then goto, udgang

nlin = (nlist-1)/5 + 1
k = 0
for i = 0, nlin-1 do begin
   left = nlist - k
   l=l+1 & loglist(l) = strjoin(list(k:(k+4) < (k + left - 1)),' ')
   k = k + 5
endfor

if origin eq 0 or origin eq 2 then begin

   ; Construct filenames for CBJ images

   capcmark = ''
   if origin eq 2 then capcmark = '1_5_'

   extno = 1
   for i = 0, nlist-1 do begin
      cbj_rev = stri(fix(strmid(list(i),0,4)))
      strrev = strmid(list(i),0,4)
      cbj_pid = stri(fix(strmid(list(i),4,4)))
      cbj_eco = stri(ecode)
      fname = '/r6/jemx/cbj/jmx2_images/'+strrev+$
       '/skyimage_'+capcmark+cbj_rev+'_'+cbj_pid+'_'+cbj_eco+'.fits'

      ; Avoid repetitions

      nw = 0
      if n_included ge 1 then w = where( fname eq list_incl, nw )

      if nw eq 0 then begin

         if file_test(fname,/reg) then begin
            l=l+1 & loglist(l) = 'Include: '+fname
            if n_included eq 0 then list_incl = [fname] $
                 else list_incl = [list_incl,fname]
            n_included = n_included + 1
            real_incl = list_incl
         endif else begin
            l=l+1 & loglist(l) = 'Missing: '+fname
            print,'Missing: '+fname
         endelse

      endif

   endfor

endif else begin

   ; Construct filenames for NL images

   extno = nl_ext
   for i = 0, nlist-1 do begin
      strrev = strmid(list(i),0,4)
      nl_pid = strmid(list(i),5,3)
      nl_seq = stri(fix(strmid(list(i),10,1))-1)
      nl_eco = stri(ecode)
      pthname = '/r6/jemx/nlwork/jemximages/'+strrev+'/'
      bname = 'skyima2_'+nl_eco+'_'+strrev+'_'+nl_pid+nl_seq+'.fits'
      fname = pthname+bname
      fnamegz = fname+'.gz'
      if file_test(fname,/reg) then begin
         l=l+1 & loglist(l) = 'Include: '+fname
         if n_included eq 0 then begin
             list_incl = [fname]
             real_incl = [fname]
         endif else begin
             list_incl = [list_incl,fname]
             real_incl = [real_incl,fname]
         endelse
         n_included = n_included + 1
      endif else begin
         if file_test(fnamegz,/reg) then begin
            spawn,'cp '+fnamegz+' /pool1'
            spawn,'gunzip /pool1/'+bname+'.gz'
            l=l+1 & loglist(l) = 'Include: '+fnamegz
            if n_included eq 0 then begin
                list_incl = [fname]
                real_incl = ['/pool1/'+bname]
            endif else begin
                list_incl = [list_incl,fname]
                real_incl = [real_incl,'/pool1/'+bname]
            endelse
            n_included = n_included + 1
         endif else begin
            l=l+1 & loglist(l) = 'Missing: '+fname
            print,'Missing: '+fname
         endelse
      endelse
   endfor
   write_slist,'list_incl',list_incl
   write_slist,'real_incl',real_incl
endelse

if n_included gt 0 then begin
   
   ;  Limit the number of input files to 1000

   if n_included gt 1000 then begin
      for m = 1000, n_included-1 do begin
         l=l+1 & loglist(l) = 'Exclude: '+list_incl(m)
      endfor
      list_incl = list_incl(0:1000-1)
      real_incl = real_incl(0:1000-1)
      n_included = 1000
   endif

   ; Create the skymaps.lst

   openw,lst,'skymaps.lst',/get_lun
   printf,lst,n_included
   if origin eq 0 then printlf,lst,list_incl else printlf,lst,real_incl
   free_lun,lst

   ; Create the tmpfil for header updating

   openw,tmp,'tmpfil_mw',/get_lun
   printlf,tmp,'COMMENT Includes '+list_incl+'+'+stri(extno)
   free_lun,tmp

   ; Remove old files

   mname = 'MOSAIC_'+stri(n_included)+'.fits'
   if file_test(mname,/reg) then spawn,'\rm '+mname
   otname = 'OBS_TIME_'+stri(n_included)+'.fits'
   if file_test(otname,/reg) then spawn,'\rm '+otname

   ; Ready to run 'mosaic_weight'

   found_exe = 0
   if !version.os eq 'sunos' then begin
      found_exe = 1
      spawn,'/home/njw/jemx/mosaic/src/mosaic_weight'
   endif
   if !version.os eq 'linux' then begin
      if getenv('HOST') eq 'lin6' then begin
         found_exe = 1
         spawn,'/users/njw/bin/mosaic_weight'
      endif
   endif

   if not found_exe then begin
      print,'mw error: No executable of mosaic_weight found for current OS'
      return
   endif

   mwname = get_next_filename(app_slash(outdir)+'mw_????.fits')
   otwname = mwname
   rwwname = mwname
   pos = strpos(otwname,'mw',/reverse_search)
   strnum = strmid(mwname,pos+3,4)
   strput,otwname,'ot',pos
   strput,rwwname,'rw',pos
   len = strlen(rwwname)
   rwwname = strmid(rwwname,0,len-4)+'txt'

   ; update the FITS header with included files
   spawn,'fmodhead '+mname+'+1 tmpfil=tmpfil_mw'
   spawn,'mv '+mname+' '+mwname
   print,'Resulting file is: '+mwname
   l=l+1 & loglist(l) = 'Resulting file is: '+mwname

   if strmid(maps,0,1) eq 'y' then begin

      spawn,'mv '+otname+' '+otwname
      print,'Obs time file is:  '+otwname
      l=l+1 & loglist(l) = 'Obs time file is:  '+otwname

   endif

   ; Check presence of 'ranked_weights.txt'

   if file_test('ranked_weights.txt',/reg) then begin

      if keyword_set(r_w) then begin
         spawn,'mv ranked_weights.txt '+rwwname
         print,'Ranked weights in '+rwwname
      endif else file_delete,'ranked_weights.txt'

   endif else print,'No ranked_weights.txt file was made'


endif else begin

   print,'Sorry, no images could be included'
   mwname = '***'

endelse

udgang:

n_inc = n_included
l=l+1 & loglist(l) = '// n_included = '+stri(n_included)

; Write log file to disk if some files are included

if n_included gt 0 then begin

   logfile = get_next_filename('mw_????.log')
   logfile = app_slash(outdir)+'mw_'+strnum+'.log'
   loglist = loglist(0:l)
   write_slist,logfile,loglist

   print,'Log file: '+logfile

endif else begin

   logfile = 'none'

endelse

;  Clean up

if file_test(f1name,/symlink) then file_delete,f1name
if file_test(f2name,/symlink) then file_delete,f2name
if file_test(f3name,/symlink) then file_delete,f3name
if file_test(f4name,/symlink) then file_delete,f4name
if file_test(f5name,/symlink) then file_delete,f5name
if file_test(f6name,/symlink) then file_delete,f6name
if file_test(f7name,/symlink) then file_delete,f7name
if file_test(f8name,/symlink) then file_delete,f8name
if file_test('list_incl',/reg) then file_delete,'list_incl'
if file_test('real_incl',/reg) then file_delete,'real_incl'
if file_test('tmpfil_mw',/reg) then file_delete,'tmpfil_mw'


end
