function predict_arf_v2, full_cor_file_in, raw_file_in, ener=ener, $
     chat=chat, gain=gain, pl=pl, radec=radec, esens=esensi, $
     scecor=scecor, upfcor=upfcor

;   2005-11-07/NJW made from predict_arf_nrt.pro
;   2005-11-16/NJW updated to include the UPF function
;   2006-02-02/NJW updated to require keyword for SCE and UPF correction
;
;   Prediction of ARF based on SB's sensitivity curve
;   and SCE (SL Collection Efficiency) and UPF (Unknown Physical Filter)
;
;
;  Define the local gain as the number of linPHA channels per keV:
;
;     local_gain = linPHA / Energy
;
;  The local gain is both a function of detector position and of time
;
;  Define the global gain (global_gain) as
;
;     global_gain = average[ linPHA / (Energy * f_spag) ]
;
;  where f_spag is the local spatial gain factor and the average
;  is taken over a suitable area of the detector
;
;  The global gain is a function of time i.e. 'global' refers to the
;  detector surface

if n_params() lt 2 then begin
   print,'Syntax: arf = predict_arf_v2( full_cor_file, raw_file[,gain=][,ener=]$'
   print,'     [,/chat][,/pl][,esens=][,radec=][,/scecor][,/upfcor])'
   print,'     Keyword ''gain'' returns the average gain'
   print,'     Keyword ''ener'' returns ARF energy array'
   print,'     Keyword ''chat'' activates extra test output'
   print,'     Keyword ''pl''   activates a plot of the gain evolution'
   print,'     Keyword ''esens'' for file with electronic sensitivity curve'
   print,'     Keyword ''radec'' (2 elem array) if source pos is known'
   print,'     Keyword ''scecor'' for doing the SCE correction'
   print,'     Keyword ''upfcor'' for doing the UPF correction'
   return,-1
endif

; full_cor_file might be gzipped
full_cor_file = gz_proxy_file(full_cor_file_in)
if not file_test(full_cor_file,/reg) then begin
   print,full_cor_file,' is missing'
   return, -1
endif

; raw_file might be gzipped
raw_file = gz_proxy_file(raw_file_in)
if not file_test(raw_file,/reg) then begin
   print,raw_file,' is missing'
   return, -1
endif

if keyword_set(radec) then begin
   szra = size(radec)
   if szra(0) ne 1 or szra(1) ne 2 then begin
      print,'Error, the radec given should be a 2-element array'
      return,-1
   endif
endif

fhdr = headfits( full_cor_file, ext=1 )
rhdr = headfits( raw_file, ext=1 )
fswid = stri(fxpar(fhdr, 'SWID'))
rswid = stri(fxpar(rhdr, 'SWID'))
finstr = stri(fxpar(fhdr, 'INSTRUME'))
rinstr = stri(fxpar(rhdr, 'INSTRUME'))

if fswid ne rswid then begin
   print,'Mismatching SWIDs: ', fswid+'  '+rswid
   return, -1
endif
if finstr ne rinstr then begin
   print,'Mismatching INSTRUMEs: ', finstr+'  '+rinstr
   return, -1
endif

sjmx = strmid(finstr,3,1)
jemxnum = fix(sjmx)

;  Open a log file

logname = get_next_filename('predict_arf_v2_????.log')
openw,lut,logname,/get_lun
printf,lut,'// '+systime()
printf,lut,'// full_cor = '+full_cor_file_in
printf,lut,'// raw = '+raw_file_in
printf,lut,'// INSTRUME = '+finstr
if keyword_set(scecor) then printf,lut,'// SCE correction done' $
           else printf,lut,'// No SCE correction'
if keyword_set(upfcor) then printf,lut,'// UPF correction done' $
           else printf,lut,'// No UPF correction'

;  Get instances of IMOD and RMF

spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' imod', insta_imod
spawn,'get_insta_swijd -swid '+fswid+' '+sjmx+' rmf', insta_rmf
insta_imod = insta_imod(0)
insta_rmf = insta_rmf(0)

printf,lut,'// IMOD = '+insta_imod
printf,lut,'//  RMF = '+insta_rmf

;  Get the events

is_event_file = 0
errmsg = ''
extnam = finstr+'-FULL-RAW'
rdfitscol,raw_file+'['+finstr+'-FULL-RAW]',rawx,'RAWX',errmsg=errmsg,nrows=nrows
if errmsg ne '' then begin
   print,'This is an events file'
   is_event_file = 1
   errmsg = ''
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',rawx,'RAWX',nrows=nrows,errmsg=errmsg
   if errmsg ne '' then begin
      print,'Well, it wasn''t an events file after all - I give up!'
      return,-1
   endif
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',rawy,'RAWY',/sil
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',pha,'PHA',/sil
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',pi,'PI',/sil
   rdfitscol,raw_file+'['+finstr+'-FULL-ALL]',status,'STATUS',/sil
endif else begin
   rdfitscol,raw_file+'['+finstr+'-FULL-RAW]',rawy,'RAWY',/sil
   rdfitscol,raw_file+'['+extnam+'-FULL-RAW]',pha,'PHA',/sil
endelse

printf,lut,'// num_events_total = ', nrows

if not is_event_file then begin
   ;* rdfitscol,full_cor_file+'+1',time,'TIME',/sil
   rdfitscol,full_cor_file+'+1',pi,'PI',/sil
   rdfitscol,full_cor_file+'+1',status,'STATUS',/sil
endif

tmp = where( pha lt 0, ntmp)
if ntmp gt 0 then begin
   print,stri(ntmp)+' PHA values less than zero in raw or event file!' 
   printf,lut,stri(ntmp)+' PHA values less than zero in raw or event file!' 
   pha(tmp) = 0
endif




;  Define 'flag' array to make sure that each detector position is only
;  counted once when making the average

flag = intarr(256,256)

; If keyword 'radec' has been given then limit the detector area

if keyword_set(radec) then begin
   printf,lut,'// ra_src = ', radec(0)
   printf,lut,'// dec_src = ', radec(1)
   get_pointing_for_swid, fswid, ra_scx, dec_scx, roll
   printf,lut,'// ra_scx = ', ra_scx
   printf,lut,'// dec_scx = ', dec_scx
   printf,lut,'// roll = ', roll
   flag = 1 - j_src_on_det( jemxnum, ra_scx, dec_scx, roll, radec(0), radec(1) )
   ; 'flag' signals pixel that is not to be used (again)
   savename = get_next_filename('pav2_flag_????.fits')
   fxhmake, hdr, flag
   fxaddpar, hdr, 'ATTACHED', logname, 'connection'
   fxaddpar, hdr, 'DATE', systime(), 'Time of creation'
   fxaddpar, hdr, 'ORIGIN', 'predict_arf_v2', 'Creating procedure'
   fxaddpar, hdr, 'SWID', fswid, 'Science Window identifier'
   fxaddpar, hdr, 'RA_SRC', radec(0), 'RA of source'
   fxaddpar, hdr, 'DEC_SRC', radec(1), 'DEC of source'
   fxaddpar, hdr, 'RA_SCX', ra_scx, 'RA of pointing'
   fxaddpar, hdr, 'DEC_SCX', dec_scx, 'DEC of pointing'
   fxaddpar, hdr, 'ROLL', roll, 'Roll of pointing'
   writefits, savename, flag, hdr
   printf,lut,'// flag_file = '+savename
endif

;* rstab,'sens_old_rejection.scm',xsens, sens,/sil
;* rstab,'sens_new_rejection.scm',xsens, sens,/sil

;  Read electronic sensitivity curve which is a function of linear PHA

;* rstab,'sens_new_r1.scm',xsens, sens,/sil
esens_file = 'sens_trial.scm'
if keyword_set(esens) then esens_file = esens
if not file_test(esens_file,/reg) then begin
   print,'File '+esens_file+' was not found'
   return,-1
endif
printf,lut,'// esens_file = '+esens_file
rstab,esens_file,xsens, sens,/sil

;  Read the curtain function

rstab,'curtain.scm', xcurt, xicurt,/sil
printf,lut,'// curtain_file = curtain.scm'

;  Get the PI energy boundaries

ebm_get_child_by_name,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+ $
      '_imod_grp_'+insta_imod+'.fits+1', $
      finstr+'-FBDS-MOD', dol_fbds
rdfitscol, dol_fbds, e_min,'e_min',/sil
rdfitscol, dol_fbds, e_max,'e_max',/sil

;  Get the standard ARF shape as well as the 500 energy values

;* ebm_get_child_by_name,'/r6/jemx/ic/rmf_grp/'+strlowcase(finstr)+ $
;*       '_rmf_grp_'+insta_rmf+'.fits+1', $
;*       finstr+'-AXIS-ARF', dol_axis
;* rdfitscol, dol_axis, energ_lo,'energ_lo',/sil,nrows=n_arf
;* rdfitscol, dol_axis, energ_hi,'energ_hi',/sil
;* rdfitscol, dol_axis, specresp,'specresp',/sil
;* energy_arf = sqrt(energ_lo*energ_hi)
;* ener = energy_arf
;* 
;* ;  Get the basic quantum efficiency, at low energy mostly from Be absorption
;* 
;* ebm_get_child_by_name,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+ $
;*       '_imod_grp_'+insta_imod+'.fits+1', $
;*       finstr+'-QEFF-MOD', dol_qeff
;* rdfitscol, dol_qeff, energy_qeff,'energy',/sil
;* rdfitscol, dol_qeff, quanteff,'quanteff',/sil
;* 
;* qarf = interpol( quanteff, energy_qeff, energy_arf )

; Newer version:

rstab,strlowcase(finstr)+'_basic_arf.scm', energy_arf, qarf
ener = energy_arf

eta_sum = fltarr(n_elements(energy_arf))

;  Get the PHA rebinning table i.e. from linear PHA to PHA in telemetry

rdfitscol,'/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+'_imod_grp_'+insta_imod+ $
      '.fits['+finstr+'-FULB-MOD]',fulb,'channel',/sil

;  Get the spatial gain array with two components

spag = readfits('/r6/jemx/ic/imod_grp/'+strlowcase(finstr)+ $
      '_imod_grp_'+insta_imod+'.fits',ext=24)
print,'size(spag):', size(spag)

;  Make an event selection based on STATUS flag

status_limit = 16L
repeat begin
   status_limit = 2 * status_limit
   w = where( status le status_limit, nw )
   print,stri(nw)+' were selected from '+stri(nrows)+ $
            ' events. Status_limit = '+stri(status_limit)
endrep until nw gt 0 or status_limit ge 32768

printf,lut,'// num_events_selected = ', nw
printf,lut,'// status_limit = ', status_limit

rawx = rawx(w)
rawy = rawy(w)
pha = pha(w)
pi = pi(w)
status = status(w)
lin_pha = fltarr(nw)
energy = fltarr(nw)

; Since the global gain is a function of time an array
; is made

global_gain_arr = fltarr(nw)

n_used = 0L

;   Loop over the selected events

for i = 0L, nw-1 do begin

   xi = xicurt(rawx(i))
   lin_pha_min = fulb((pha(i) > 1) - 1) + 1
   lin_pha_max = fulb(pha(i))
   if pha(i) le 128 then lin_pha(i) = lin_pha_max else $
      lin_pha(i) = lin_pha_min + $
      float(fix(randomu(seed)*(lin_pha_max-lin_pha_min)+0.5))

   ;  'lin_pha' is (close to) the signal as delivered from the DFEE

   ;  assign an energy between PI limits

   energy(i) = e_min(pi(i)) + randomu(seed)*(e_max(pi(i)) - e_min(pi(i)))

   f_spag = ((1. + 0.01*spag(rawx(i),rawy(i),0))*spag(rawx(i),rawy(i),1))
   slow = lin_pha(i) / (1. - xi)
   global_gain_arr(i) = lin_pha(i) / (energy(i) * f_spag)
   eta = interpol(sens,xsens,slow)
   if keyword_set(chat) then begin
      if i mod 7345L eq 0 then begin
         print,'i = ',i,' w(i) = ', w(i)
         print,'RAWX,RAWY = ', rawx(i), rawy(i)
         print,'PHA,PI,ENERGY = ', pha(i), pi(i), energy(i)
         print,'SPAG level 0 = ', spag(rawx(i),rawy(i),0)
         print,'SPAG level 1 = ', spag(rawx(i),rawy(i),1)
         print,'Curtain xi = ', xi
         print,'Linear PHA between ', lin_pha_min, lin_pha_max
         print,'Chosen Linear PHA = ', lin_pha(i)
         print,'Local gain = ', lin_pha(i) / energy(i)
         print,'Global gain = ', global_gain_arr(i)
         print,'Slow signal = ', slow
         print,'Sensitivity = ', eta
      endif
   endif

   ; sum up the sensitivities

   if not flag(rawx(i),rawy(i)) then begin
      lin_pha_arr = energy_arf * f_spag * global_gain_arr(i)
      slow_arr = lin_pha_arr / (1. - xi)
      eta_arr = interpol(sens,xsens,slow_arr)
      eta_sum = eta_sum + eta_arr
      n_used = n_used + 1
      flag(rawx(i),rawy(i)) = 1
   endif

endfor

gain = avg(global_gain_arr)
print,'Average gain is '+string(gain,format='(f6.2)')+' chan/keV'
printf,lut,'// average_gain = '+string(gain,format='(f6.2)')+' ; chan/keV'

if keyword_set(pl) then begin
   plot,global_gain_arr,ps=3,yr=[0,40],xtit='Event number',$
        ytit='Gain [PHA/keV]',title=rinstr+'  '+rswid
   plotsign
   plotname,'predict_arf_v2'
endif

eta_avg = eta_sum / n_used

;* arf = 90. * arf / n_used ; Use 90 cm2 as a preliminary value

if keyword_set(scecor) then begin

   ;  Correct for the SCE (SL Collection Efficiency)

   rdfitscol,strlowcase(finstr)+'_sce_archive_0001.fits+1',sce_gain,'GAIN'
   rdfitscol,strlowcase(finstr)+'_sce_archive_0001.fits+1',sce,'SCE'
   rdfitscol,strlowcase(finstr)+'_sce_archive_0001.fits+1',sce_energy,'ENERGY'

   ; use average gain even though it might not be entirely appropriate

   ; Interpolation algorithm:
   w = where( sce_gain gt gain, nw )
   n = n_elements(sce_gain)
   if nw gt 0 then i = (w(0)-1) > 0 else i = n - 2
   fac = (gain - sce_gain(i)) / (sce_gain(i+1) - sce_gain(i))
   sce_ip1 = interpol(sce(*,i+1),sce_energy(*,i+1), energy_arf)
   sce_i   = interpol(sce(*,i),sce_energy(*,i), energy_arf)
   sce_int = fac * (sce_ip1 - sce_i) + sce_i

endif else sce_int = 1.0

; Get the unknown physical filter (UPF)

if keyword_set(upfcor) then begin

   rstab,'jmx1_upf_interpol.scm',e_upf, upf
   upf_int = interpol(upf, e_upf, energy_arf)
   printf,lut,'// upf_file = jmx1_upf_interpol.scm'

endif else upf_int = 1.0

;* arf = qarf * sce_int * eta_avg * upf_int

; Get the Kapton filter transmission

rstab,'kapton12_5mu.scm', e_kap, t_kap
printf,lut,'// kapton_file = kapton12_5mu.scm'
t_kap_int = interpol( t_kap, e_kap, energy_arf )

arf = qarf * sce_int * upf_int * eta_avg * t_kap_int
;      The detector area is included in 'qarf'

print,'Log file: '+logname
free_lun,lut

return, arf

end
