function psf, im, nstep,  r, area, eec, b=b, silent=silent, loga=loga
;
; Return Point Spread Function assuming azimuthal symmetry
; for peak in image 'im'.
; If the keyword 'b' is given then only the area where the
; b-array (of same size as 'im') is one is used
;
;  result = psf(im, nstep, > r[, area[, eec]])
;
; where nstep is the number of annuli
;       r    is array with radii to largest distance
;       area is array with area of each annulus in pixels^2
;       eec is array with accumulated number of counts
;       result is (eec(n) - eec(n-1)) / area(n)
;
np = n_params()
if np lt 3 then begin
   print,'Syntax: res = psf(im,nstep,r [,area] [,eec] [,b=] [,/silent] [,/loga]'
   return, -1.
endif
sz = size(im)
if( sz(0) ne 2 ) then begin
   print,'not a 2D image - terminate'
   return,-1.
endif
barr = replicate(1,sz(1),sz(2))
if keyword_set(b) then begin
   barr = b
   sb = size(barr)
   if( (sz(1) ne sb(1)) or (sz(2) ne sb(2)) ) then begin
      print,'B array has different dimensions from image - terminate'
      stop
   endif
endif

verb = 1
if keyword_set(silent) then verb = 0
;
; get centroide of peak
;
centr = peakb(im,barr)
if verb then print,'Centroide : ', centr
tt = total(im)
if verb then print,'Total number of counts : ', tt
;
; make array of distances
;
if verb then print,' Making array of distances ...'
dist_arr = distances(sz(1),sz(2),centr(0),centr(1))
rmin = min(dist_arr, j)
;  find the next-smallest distance
aux = dist_arr
rmax = max(dist_arr)
aux(j) = rmax
rmin = min(aux)
area = fltarr(nstep)
eec = area
res = area
if keyword_set(loga) then r = [0.,xloggen(nstep-1, rmin*1.1, rmax)]$
   else r = xlingen(nstep,0.,rmax)
if verb then print,' Accumulate arrays ...'
for i = 1, nstep-1 do begin
   ireg = where( dist_arr le r(i) )
   area(i) = n_elements(ireg)
   eec(i) = total(im(ireg))
   da = area(i) - area(i-1)
   de = eec(i) - eec(i-1)
   if da gt 0 then res(i) = de/da else res(i) = res(i-1)
   if verb then print, nstep, i, r(i),$
         area(i), eec(i), res(i)
endfor
res(0) = res(1)
eec = eec / tt
return, res
end
