pro rdfitscolc, filename_extno, col_data, col_id, silent=silent, hdr=hdr,$
                errmsg=errmsg, trace=trace, nrows=nrows
;
; Read a FITS binary extension table column
;
;  990211/NJW
;  Updated 000807/NJW
;  Updated 010313/NJW to use cache storage to save time for re-reading
;
;   Data is returned in 'col_data'
;   The col_id can be a column number or name
;

if n_params() lt 3 then begin
   print,'Syntax: RDFITSCOLC, filename_extno, col_data, col_id, silent=,hdr='
   print,'      col_id can be a number or a name'
   print,'      This version uses a cache system'
   return
endif

errmsg = ''
vb = 1 - keyword_set(silent)
tr = keyword_set(trace)
sz = size(col_id)
if sz(sz(0)+1) eq 7 then begin
   col_idstr = strupcase(col_id)
   col_idnum = 0
endif else begin
   col_idstr = ' '
   col_idnum = fix(col_id)
endelse

if !version.os eq 'sunos' then begin
   cache_dir = '/pool14/'
endif
if !version.os eq 'linux' then begin
   cache_dir = '/users/njw/cache/'
endif
if !version.os eq 'Win32' then begin
   cache_dir = 'c:\work\'
endif

; Check if filename is complete

if strmid(filename_extno,0,1) ne '/' then begin ; expand
   fullfilename_extno = getenv('PWD')+'/'+filename_extno
endif else fullfilename_extno = filename_extno

; Get last serial number in 'cache_dir'

list_serial = findfile(cache_dir+'cache_last_serial.number',count=n_list_serial)
last_serial = 0L
if n_list_serial eq 1 then begin
   if tr then print,'##0## open(r) file: ',cache_dir+'cache_last_serial.number'
   openr,lun,cache_dir+'cache_last_serial.number',/get_lun
   readf,lun,last_serial
   if tr then print,'##1## got last_serial = ', last_serial
   free_lun,lun
endif else begin
   if tr then print,'##2## open(w) file: ',cache_dir+'cache_last_serial.number'
   openw,lun,cache_dir+'cache_last_serial.number',/get_lun
   printf,lun,last_serial,format='(i4.4)'
   if tr then print,'##3## wrote last_serial = ', last_serial
   free_lun,lun
endelse
   
;  Find the cache file candidates

found = 0
list_i = findfile(cache_dir+'rfc_cache_????.iden',count=n_list_i)
if tr then print,'##4## Found ', n_list_i,' cache id files'
if n_list_i gt 0 then begin ; There are some cache id files
                            ; Look for current name and column
   i = 0
   repeat begin
      if tr then print,'##5## repeat loop, i = ', i
      if tr then print,'##6## open(r) id file: ', list_i(i)
      openr,lun,list_i(i),/get_lun
      file_extno = ''
      readf,lun,file_extno
      colname = ''
      readf,lun,colname
      colnumber = 0
      readf,lun,colnumber
      free_lun,lun
      if tr then print,'##7## file/col: ', file_extno,'  ', colname

      if fullfilename_extno eq file_extno and $
            (col_idstr eq colname or col_idnum eq colnumber ) then begin

         found = 1
         ; extract serial number
         str_serial = str_reverse(strmid(str_reverse(list_i(i)),5,4))
         if tr then print,'##8## Match! serial number = ', str_serial
      endif
      i = i + 1
   endrep until (found or i eq n_list_i)

endif

if found then begin ; check for existence of data file (idlsave)

   datname = cache_dir+'rfc_cache_'+str_serial+'.idlsav'
   if tr then print,'##9## Check for existence of data file: ', datname
   l = findfile(datname, count=n_l)
   if n_l ne 1 then found = 0
   if tr then begin
      if found then print,'##10## It''s there' else print,'##3## It''s missing!'
   endif

endif

if found then begin

   if vb then print,'Restoring column data from '+datname+' ...'
   restore,file=datname
   nrows = n_elements(col_data)

endif else begin

   get_exten_no, fullfilename_extno, filename, extno
   if not file_test( filename, /reg ) then begin
      errmsg = 'File not found'
      print,errmsg
      return
   endif
   errmsg = ''
   fxbopen,unit,filename,extno,hdr,errmsg=errmsg
   if errmsg ne '' then begin
      print,errmsg
      fxbclose,unit
      return
   endif
   if tr then print,'##11## opened ', filename,' with extno = ', extno

   errmsg = ''
   col_number = fxbcolnum( unit, col_id, errmsg=errmsg)
   if errmsg ne '' then begin
      print,errmsg
      fxbclose,unit
      return
   endif
   if tr then print,'##12## The column number is ', col_number

   if col_number lt 10 then begin
      sc = 'TTYPE'+string(col_number,format='(i1)')
   endif else begin
      sc = 'TTYPE'+string(col_number,format='(i2)')
   endelse
   col_name = strtrim(fxpar(hdr,sc),2)
   nrows = fxpar(hdr,'NAXIS2')

   if vb then begin
      print,'Reading file '+fullfilename_extno
      print,'column named: '+fxpar(hdr,sc)
      print,'with ',nrows,' rows'
   endif

   fxbread,unit,col_data,col_number
   fxbclose,unit

   ; Save column data into cache area for quick retrieval next time

   ; 1) update file with last serial number

   last_serial = last_serial + 1L
   str_serial = string(last_serial,format='(i4.4)')
   openw,lun,cache_dir+'cache_last_serial.number',/get_lun
   printf,lun,str_serial
   free_lun,lun

   ; 2) write data identification file

   openw,lun,cache_dir+'rfc_cache_'+str_serial+'.iden',/get_lun
   printf,lun,fullfilename_extno
   printf,lun,col_name
   printf,lun,col_number
   free_lun,lun

   ; 3) save column data in IDL format

   save,file=cache_dir+'rfc_cache_'+str_serial+'.idlsav',col_data, hdr

endelse



end
