pro read_catalog, dol_of_cat_in, num_entries_out, chat=chat
;
;  Read an X-ray source catalog and store it in a common area
;
;  NJW/010126  Read a catalog in format like DSRI_CAT.fits
;  NJW/2005-10-27  Updated to version 2 for gnrl_refr_cat_latest.fits
;  NJW/2006-01-10  Updated to also have the entry: catalogue
;
;      Some of the 'not so important' columns may be missing
;      in which case some dummy information is produced
;
;      The general common: cat_com_v2, cat, num_entries_cat, dol_of_cat
;
;      is filled with data


common cat_com_v2, cat, num_entries_cat, dol_of_cat

source = {   name:'', ra_obj:0.0, dec_obj:0.0, spe_modl:'', $
             spe_npar:0, spe_pars:fltarr(10), $
             intens:0.0, category:'', catalogue:''}

if n_params() lt 2 then begin
   print,'Syntax: read_catalog, <dol_of_cat>, >num_entries'
   return
endif

get_exten_no, dol_of_cat_in, file_of_cat_in, extno
if not file_test( file_of_cat_in ) then begin
   print,'READ_CATALOG: Requested catalog: '+file_of_cat_in+' does not exist ...'
   num_entries_out = -1
   return
endif

dol_of_cat = dol_of_cat_in

; Read the contents of the database

if keyword_set(chat) then print,'READ_CATALOG: Reading '+dol_of_cat+' ...'

;  RA(_OBJ) and DEC(_OBJ) are required columns

errmsg = ''
rdfitscol,dol_of_cat,ra,'RA_OBJ',err=errmsg,/sil
if errmsg ne '' then begin ; Try RA
   errmsg = ''
   rdfitscol,dol_of_cat,ra,'RA',err=errmsg,/sil
   if errmsg ne '' then begin
      print,'READ_CATALOG: Neither RA nor RA_OBJ column found'
      print, errmsg
      return
   endif
endif

rdfitscol,dol_of_cat,dec,'DEC_OBJ',err=errmsg,/sil
if errmsg ne '' then begin ; Try DEC
   errmsg = ''
   rdfitscol,dol_of_cat,dec,'DEC',err=errmsg,/sil
   if errmsg ne '' then begin
      print,'READ_CATALOG: Neither DEC nor DEC_OBJ column found'
      print, errmsg
      return
   endif
endif

num_entries = n_elements(ra)
num_entries_cat = num_entries
num_entries_out = num_entries

; The remaining columns may be omitted


rdfitscol,dol_of_cat,name,'NAME',err=errmsg,/sil
if errmsg ne '' then begin ;  Try 'SOURCENAME' in stead
   errmsg = ''
   rdfitscol,dol_of_cat,name,'SOURCENAME',err=errmsg,/sil
endif
if errmsg ne '' then begin
   name = replicate('?NAME',num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with ?NAME'
   errmsg = ''
endif

rdfitscol,dol_of_cat,intens,'INTENS',err=errmsg,/sil
if errmsg ne '' then begin
   intens = replicate(1.e-13,num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with 1.e-13'
   errmsg = ''
endif

rdfitscol,dol_of_cat,spe_modl,'SPE_MODL',err=errmsg,/sil
if errmsg ne '' then begin
   spe_modl = replicate('wabs*powerlaw',num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with wabs*powerlaw'
   errmsg = ''
endif

rdfitscol,dol_of_cat,spe_npar,'SPE_NPAR',err=errmsg,/sil
if errmsg ne '' then begin
   spe_npar = replicate(2,num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with 3'
   errmsg = ''
endif

rdfitscol,dol_of_cat,spe_pars_orig,'SPE_PARS',err=errmsg,/sil
spe_pars = fltarr(10,num_entries)
if errmsg ne '' then begin
   deflt = [0.1,1.,1.,0,0,0,0,0,0,0]
   for i = 0L, num_entries-1 do spe_pars(*,i) = deflt
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with 0.1 1 1 0 0 ...'
   errmsg = ''
endif else begin
   for i = 0L, num_entries-1 do spe_pars(*,i) = spe_pars_orig(0:9,i)
endelse

rdfitscol,dol_of_cat,class,'CLASS',err=errmsg,/sil
if errmsg ne '' then begin
   category = replicate('?CLASS',num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with ?CLASS'
   errmsg = ''
endif else begin
   category = 'class'+string(class,'(i6)')
endelse

rdfitscol,dol_of_cat,catalogue,'CATALOGUE',err=errmsg,/sil
if errmsg ne '' then begin
   catalogue = replicate('?CATLG',num_entries)
   if keyword_set(chat) then print,errmsg,'READ_CATALOG:  substituted with ?CATLG'
   errmsg = ''
endif

cat = replicate( source, num_entries )

for i = 0, num_entries-1 do begin
   cat(i).name       = name(i)
   cat(i).ra_obj     = ra(i)
   cat(i).dec_obj    = dec(i)
   cat(i).spe_modl   = spe_modl(i)
   cat(i).spe_npar   = spe_npar(i)
   cat(i).spe_pars(*)  = spe_pars(*,i)
   cat(i).intens     = intens(i)
   cat(i).category   = category(i)
   cat(i).catalogue   = catalogue(i)
endfor

end
