pro rebin_eb2eb, eblo_in, ebhi_in, rate_in, stat_err_in, $
                 eblo_out, ebhi_out, rate_out, stat_err_out, $
                 chatter=chatter, qerr=qerr, intpol=intpol, farray=farray
;
;  Rebin a spectrum from a given set of energy boundaries to a
;  new set of energy boundaries
;
;  Assume that there are no holes between the channels
;     i.e.  ebhi(i) = eblo(i+1)
;
;    2005-11-08/NJW
;    2006-01-31/NJW updated to make interpolation when output bins
;        are less than half the size of the input bins
;
if n_params() lt 8 then begin
   print
   print,'Syntax: rebin_eb2eb, eblo_in, ebhi_in, rate_in, stat_err_in, $'
   print,'                   eblo_out, ebhi_out, >rate_out, >stat_err_out $'
   print,'                   [, chatter=][,/qerr][,intpol=]
   print
   print,' The spectra must be given _per bin_ for this to function correctly'
   print
   print,'  Keyword ''chatter'' should be given as 3 for full debugging output'
   print,'          ''qerr'' forces the errors to be added in quadrature'
   print,'                   This is to be preferred when smaller bins are combined to larger ones'
   print,'          ''intpol'' to be set for linear interplation and set larger than 1'
   print,'                   for logarithmic interpolation. Only useful when larger bins'
   print,'                   are subdivided to smaller bins'
   print
   return
endif

if not keyword_set(chatter) then chatter = 0

n_in = n_elements(eblo_in)
if n_elements(ebhi_in) ne n_in then print,'Error #1'
if n_elements(rate_in) ne n_in then print,'Error #2'
if n_elements(stat_err_in) ne n_in then print,'Error #3'
for i = 0, n_in-2 do begin
   if abs(ebhi_in(i)-eblo_in(i+1)) gt 1.e-6 then print,'Warning #101'
endfor
n_out = n_elements(eblo_out)
if n_elements(ebhi_out) ne n_out then print,'Error #4'
for i = 0, n_out-2 do begin
   if abs(ebhi_out(i)-eblo_out(i+1)) gt 1.e-6 then print,'Warning #102'
endfor

;  Loop over output channels

rate_out = fltarr(n_out)
stat_err_out = fltarr(n_out)
farray = fltarr(n_in, n_out)

for i = 0, n_out-1 do begin

   if chatter gt 2 then $
     print,'---- out chan#'+stri(i)+'  '+stri(eblo_out(i))+' - '+stri(ebhi_out(i))
   blo = eblo_out(i)
   bho = ebhi_out(i)

   for j = 0, n_in-1 do begin

      bli = eblo_in(j)
      bhi = ebhi_in(j)
      if bli ge bho then break
      if bhi le blo then continue
      if bhi gt bho and bli lt blo then icase = 0
      if bhi gt bho and bli ge blo then icase = 1
      if bhi le bho and bli ge blo then icase = 2
      if bhi le bho and bli lt blo then icase = 3
      case icase of
            0  : fraction = (bho - blo) / (bhi - bli)
            1  : fraction = (bho - bli) / (bhi - bli)
            2  : fraction = 1.
            3  : fraction = (bhi - blo) / (bhi - bli)
      endcase
      if chatter gt 2 then $
         print,'     in chan#'+stri(j)+'  '+stri(eblo_in(j))+ $
	     ' - '+stri(ebhi_in(j))+' frac: '+stri(fraction)

      rate_out(i) = rate_out(i) + rate_in(j)*fraction
      if keyword_set(qerr) then $
         stat_err_out(i) = stat_err_out(i) + stat_err_in(j)^2*fraction^2 $
      else $
         stat_err_out(i) = stat_err_out(i) + stat_err_in(j)*fraction

      farray(j,i) = fraction

   endfor

   ; Now all contributions have now been localized

endfor

if keyword_set(qerr) then stat_err_out = sqrt(stat_err_out)

; If interpolation has been required then reshape the output spectrum
; while maintaining the number of counts from each input bin

if keyword_set(intpol) then begin

   ;  Define generic shape of input spectrum

   e_in = 0.5*(eblo_in + ebhi_in)
   e_out = 0.5*(eblo_out + ebhi_out)
   f_out = interpol(rate_in, e_in, e_out)
   for j = 0, n_in-1 do begin
      idx = where( farray(j,*) gt 0.0, n_idx)
      if n_idx gt 0 then $
      rate_out(idx) = f_out(idx) * total(rate_out(idx)) / total(f_out(idx))
   endfor

endif

end
