pro rebin_spectrum, e1_in, e2_in, rate_in, rate_err_in, rebin_array_in, $
                    e1new, e2new, ratenew, rate_errnew, perkev=perkev
;
;  Rebin a spectrum. e1 and e2 are the input energy boundaries
;  The resulting bins are in e1new and e2new with ratenew and rate_errnew
;
;    NJW/2002-07-29
;    2005-12-21/NJW updated to accept rate and rate_err as per keV
;
;  The default assumption is that rate and rate_err are given
;  per bin.
;
;  The keyword 'perkev' indicates that the input as well as the
;  resulting arrays are per keV.
;
;  The rebinning array 'rebin_array' must be set up as follows:
;  Each element contains the number of original bins contained
;  in the final bin. If the first element is less than zero
;  then the absolute number indicates the number of bins to skip
;
if n_params() ne 9 then begin
   print, 'Syntax: rebin_spectrum, e1, e2, rate, rate_err, rb $'
   print,'        >e1new, >e2new, >r_new, >r_err_new, [, /perkev]'
   print
   print,'         ''rb'' is the rebinning array'
   print
   return
endif

e1 = e1_in
e2 = e2_in
rate = rate_in
rate_err = rate_err_in
n_e1 = n_elements(e1)
n_e2 = n_elements(e2)
n_rate = n_elements(rate)
n_rate_err = n_elements(rate_err)
rebin_array = rebin_array_in

if n_e1 ne n_e2 or n_e1 ne n_rate or n_rate_err ne n_e1 then begin
   print,'REBIN_SPECTRUM error: problem with array dimensions'
   return
end
if rebin_array(0) eq 0 then begin
   print,'REBIN_SPECTRUM error: rebin_array(0) may not be zero'
   return
endif

if keyword_set(perkev) then begin
   rate = rate * ( e2 - e1 )
   rate_err = rate_err * ( e2 - e1 )
endif

;  Skip beginning ?

n = n_e1
if rebin_array(0) lt 0 then begin
   nskip = abs(rebin_array(0))
   b = indgen(n_e1 - nskip) + nskip
   e1 = e1(b)
   e2 = e2(b)
   rate = rate(b)
   rate_err = rate_err(b)
   rebin_array = rebin_array(1:*)
   n = n_elements(b)
endif

nnew = n_elements(rebin_array)
e1new = fltarr(nnew)
e2new = fltarr(nnew)
ratenew = fltarr(nnew)
rate_errnew = fltarr(nnew)

j = 0
for i = 0, nnew-1 do begin

   for k = 0, rebin_array(i)-1 do begin
      if k eq 0 then e1new(i) = e1(j)
      e2new(i) = e2(j)
      ratenew(i) = ratenew(i) + rate(j)
      rate_errnew(i) = sqrt(rate_errnew(i)^2 + rate_err(j)^2)
      j = j + 1
      if j eq n then begin
          i = i + 1
          goto, label1
      endif
   endfor

endfor

label1:

e1new = e1new(0:i-1)
e2new = e2new(0:i-1)
ratenew = ratenew(0:i-1)
rate_errnew = rate_errnew(0:i-1)

if keyword_set(perkev) then begin
   ratenew = ratenew / ( e2new - e1new )
   rate_errnew = rate_errnew / ( e2new - e1new )
endif

return

end
