pro rscol, filename, col_dat, col_def, str=str, int=int, long=long, dble=dble,$
          silent=silent, block=block, col_num=col_num, nrows=nrows
;
; A procedure in the S-table package
;   (Others are: rstab, rtab, wtab, wstab)
;
; This procedure will read a column from an ASCII file
; organized as a table
; If the column should be read as a string then
; the keyword 'str' must be set.
; Default type is float. Keywords exist for int, long and double
;
;   IDL> rscol,'mytable',col,col_definiton[,/str][,/int][,/long] $
;           [,/dble][,/silent][,block=][nrows=]
;
; 'rscol' will create the array 'col_dat'
; with length equal to the number of rows in the table.
;
;  The columns can be numbers or character strings separated with
;  blanks. The first column must always start with one of +-.0123456789
;  The column asked for must be a number.
;
;  First column number is 1
;
;  Keyword 'silent' for silent i.e. no output to screen
;
; The lines that start with non-digit characters are interpreted as
; comment lines and will be displayed but skipped in
; the reading of the data
;
; If the 'block' keyword is given with a block number
; then the file will be scanned for existence of lines:
;       // block = nnn
; and only the table rows between where nnn matches the
; given block number and the next '// block = xxx'
;
; Other procedures required: n_columns.pro
;                            keyword_setx.pro
;                            comblock.pro
;
if n_params() lt 3 then begin
   print,'RSCOL, (input)filename, (return)col_array, (input)column_definition'
   print,'    column_definition is either a number or a name specified'
   print,'    with keyword ''colname'''
   print,'Keywords:    /str    - a string is returned'
   print,'             /int    - a short integer is returned'
   print,'             /long   - a long integer is returned'
   print,'             /dble   - a double is returned'
   print,'             /silent - no extra messages'
   print,'             block=  - read only this block of rows'
   print,'             nrows=  - returns number of data lines'
   retall
endif

vb = 1 - keyword_set(silent) ; keyword for verbosity

; Define type of returned array

type = 4 ; float
if keyword_set(str) then type = 7 ; string
if keyword_set(long) then type = 3 ; long integer
if keyword_set(dble) then type = 5 ; double
if keyword_set(int) then type = 2 ; short integer

; If column designator is a string then find the matching
; number

szcf = size(col_def)
if szcf(0) ne 0 then begin
   print,'The column definition must be a scalar'
   retall
endif
if szcf(1) eq 7 then begin
   ; get the column names
   colnames = comgets(filename,'colname',ncols)
   if ncols eq 0 then begin
      print,'Error - no keyword ''colname'' found'
      retall
   endif
   match = 0
   for i = 0, ncols-1 do begin
      if col_def eq colnames(i) then begin
         col_number = i + 1
         match = 1
      endif
   endfor
   if not match then begin
      print,'Error: no such column name has been found'
      retall
   endif
endif else begin
   col_number = fix(col_def)
endelse
col_num = col_number

; Initiate use of block (i.e. only take data from a
; specified block

useblock = 0
if keyword_setx(block) then begin
   useblock = 1
   nblock = block
   comblock,filename,nblock,ln_begin,ln_end
   if ln_begin eq -1 then begin
      print,'%RSTAB Warning: Requested block not found'
      print,'                 Proceed reading all data'
      useblock = 0
   endif
endif

; See if requested column number is less than or equal to
; the number of columns

if vb then print,'RSCOL: at least ', col_number, $
          ' columns expected in '+filename
nc = n_columns(filename)
if vb then print,nc,' columns found in '+filename

if col_number gt nc then begin
   print,'% RSCOL: only ', nc,' columns found in file'
   stop
endif

; Prepare reading the file

nmax = 3000L
cur_size = nmax

openr,lun,filename,/get_lun
case type of
   2 : col_dat = intarr(nmax)
   3 : col_dat = lonarr(nmax)
   4 : col_dat = fltarr(nmax)
   5 : col_dat = dblarr(nmax)
   7 : col_dat = strarr(nmax)
endcase
x = fltarr(nc)
s = ' '
kount = 0L
linenumber = 0L

while not eof(lun) do begin

  readf,lun,s
  pos = strpos(s,';')
  if pos ge 0 then ss = strmid(s,0,pos) else ss = s
  st = strtrim(ss,2) ; remove leading and trailing blanks
  if strlen(st) gt 0 then begin
     if( strpos('+-.0123456789',strmid(st,0,1)) lt 0 ) then begin
           if vb then print,s
     endif else begin
        su = strcompress(st) ; reduce all space to single spaces
        elmnts = strsplit(su,' ',/extract) ; separate into columns
        if useblock then begin
           if linenumber ge ln_begin and linenumber le ln_end then begin
              case type of
                   2 : col_dat(kount) = fix(elmnts(col_number-1))
                   3 : col_dat(kount) = long(elmnts(col_number-1))
                   4 : col_dat(kount) = float(elmnts(col_number-1))
                   5 : col_dat(kount) = double(elmnts(col_number-1))
                   7 : col_dat(kount) = elmnts(col_number-1)
              endcase
              kount = kount + 1L
           endif
        endif else begin
              case type of
                   2 : col_dat(kount) = fix(elmnts(col_number-1))
                   3 : col_dat(kount) = long(elmnts(col_number-1))
                   4 : col_dat(kount) = float(elmnts(col_number-1))
                   5 : col_dat(kount) = double(elmnts(col_number-1))
                   7 : col_dat(kount) = elmnts(col_number-1)
              endcase
           kount = kount + 1L
        endelse

        ; Check if expansion is needed

        if kount eq cur_size then begin
    
           case type of
              2 : col_dat = [col_dat, intarr(nmax)]
              3 : col_dat = [col_dat, lonarr(nmax)]
              4 : col_dat = [col_dat, fltarr(nmax)]
              5 : col_dat = [col_dat, dblarr(nmax)]
              7 : col_dat = [col_dat, strarr(nmax)]
           endcase
           cur_size = cur_size + nmax

        endif
     endelse
   endif
   linenumber = linenumber + 1
endwhile
if vb then print,'Size of table = ',kount
nrows = kount
col_dat = col_dat(0:kount-1)
close,lun
free_lun,lun
end

