pro rstab,filename,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,$
          c11,c12,c13,c14,c15,silent=silent,block=block,nrows=nrows
;
; A procedure in the S-table package
;   (Others are: rtab, wtab, wstab)
;
;  ~1996/NJW
;   2005-03-21/NJW Update to eliminate hard limit on number of rows
;
; This procedure will read an ASCII file organized as a table
; with up to 15 columns. The actual number of columns must be
; specified in the call e.g.
;
;              IDL> rstab,'mytable',x,y,z[,/silent]
;
; if 'mytable' contains 3 columns. 'rstab' will create the arrays
; with length equal to the number of rows in the table
;  Keyword 'silent' for silent i.e. no output to screen
;
; The lines that start with non-digit characters are interpreted as
; comment lines and will be displayed but skipped in
; the reading of the data
;
; If the 'block' keyword is given with a block number
; then the file will be scanned for existence of lines:
;       // block = nnn
; and only the table rows between where nnn matches the
; given block number and the next '// block = xxx'
;
; Other procedures required: n_columns.pro
;                            keyword_setx.pro
;                            comblock.pro
;

if n_params() lt 2 then begin
        print,'Syntax: rstab,filename,c1,c2,...[,/silent][,block=][,nrows=]'
        print,'           Up to and including 15 columns of data'
        print,'           ''silent'' inhibits extra terminal output'
        print,'           ''block''  defines the block number for row section'
        print,'           ''nrows''  returns the number of rows read'
        return
end

vb = 1 - keyword_set(silent) ; keyword for verbosity

; Initiate use of block (i.e. only take data from a
; specified block

useblock = 0
if keyword_setx(block) then begin
   useblock = 1
   nblock = block
   comblock,filename,nblock,ln_begin,ln_end
   if ln_begin eq -1 then begin
      print,'%RSTAB Warning: Requested block not found'
      print,'                 Proceed reading all data'
      useblock = 0
   endif
endif

; See if requested number of columns matches the actual number

nmax = 2000L
n = n_params() - 1
if vb then print,'RSTAB: ', n,' columns expected in '+filename
nc = n_columns(filename)

; Check for data

if nc eq -1 then begin
   nrows = 0
   print,'% RSTAB: no data rows in file'
   return
endif

; Check for correct number of columns

if n ne nc then begin
   print,'% RSTAB: ', nc,' columns found in file'
   return
endif

; Prepare reading the file

openr,lun,filename,/get_lun
c = fltarr(nmax,n)
x = fltarr(n)
s = ' '
i = 0L
linenumber = 0L
while not eof(lun) do begin
  readf,lun,s
  pos = strpos(s,';')
  if pos ge 0 then ss = strmid(s,0,pos) else ss = s
  st = strtrim(ss,1) ; remove leading blanks
  if strlen(st) gt 0 then begin
     if( strpos('+-.0123456789',strmid(st,0,1)) lt 0 ) then begin
           if vb then print,s
     endif else begin
        if useblock then begin
           if linenumber ge ln_begin and linenumber le ln_end then begin
              reads,s,x
              for j = 0, n-1 do c(i,j) = x(j)
              i = i + 1
           endif
        endif else begin
           reads,s,x
           for j = 0, n-1 do c(i,j) = x(j)
           i = i + 1
        endelse
     endelse
   endif
   linenumber = linenumber + 1
   if i eq nmax then begin
           nmax = 2*nmax
           if vb then print,'Expanding to ',nmax
           expand_array, c, nmax, 1
   endif
endwhile
if vb then print,'Size of table = ',i
nrows = i ;  Report back to caller

; Shrink arrays to absolutely necessary

if nrows gt 0 then begin
   c = c(0:i-1,*)
   c1 = c(*,0)
   if( n gt 1 ) then c2 = c(*,1)
   if( n gt 2 ) then c3 = c(*,2)
   if( n gt 3 ) then c4 = c(*,3)
   if( n gt 4 ) then c5 = c(*,4)
   if( n gt 5 ) then c6 = c(*,5)
   if( n gt 6 ) then c7 = c(*,6)
   if( n gt 7 ) then c8 = c(*,7)
   if( n gt 8 ) then c9 = c(*,8)
   if( n gt 9 ) then c10 = c(*,9)
   if( n gt 10 ) then c11 = c(*,10)
   if( n gt 11 ) then c12 = c(*,11)
   if( n gt 12 ) then c13 = c(*,12)
   if( n gt 13 ) then c14 = c(*,13)
   if( n gt 14 ) then c15 = c(*,14)
endif

free_lun,lun
end
