pro rstrtab,filename,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,$
          c11,c12,c13,c14,c15,coltyp=coltyp,silent=silent,block=block
;
; A procedure in the S-table package
;   (Others are: rstab, rtab, wtab, wstab)
;
;  ~1997/NJW
;
; This procedure will read an ASCII file organized as a table
; with up to 15 columns. The actual number of columns must be
; specified in the call e.g.
;
;              IDL> rstrtab,'mytable',x,y,z[,coltyp=][,/silent]
;
; if 'mytable' contains 3 columns. 'rstrtab' will create the arrays
; with length equal to the number of rows in the table
;  Keyword 'silent' for silent i.e. no output to screen
;
; The lines that start with non-digit characters are interpreted as
; comment lines and will be displayed but skipped in
; the reading of the data
;
; If the 'block' keyword is given with a block number
; then the file will be scanned for existence of lines:
;       // block = nnn
; and only the table rows between where nnn matches the
; given block number and the next '// block = xxx'
;
; This version includes the option to use columns with strings
;    The rules are:  The first column must be a number
;                    The strings may not contain blanks
;                    The strings may not contain semicolon ';'
;
;    The keyword 'coltyp' must be a string with a least as many
;        characters as there are columns. Example:
;           'nsnn' for a table where column 1, 3, and 4 are numbers,
;           and column 2 contains strings
;
; Other procedures required: n_columns.pro
;                            keyword_setx.pro
;                            comblock.pro
;

if n_params() lt 1 then begin
   print,'RSTRTAB, filename, c1, c2, ... (max 15)[,coltyp=][,/silent][,block=]'
   print,'        A single character gives each column type: ''n'' or ''s'''
   print,'        The default column type is numerical (n)'
   return
endif



vb = 1 - keyword_set(silent) ; keyword for verbosity

; Initiate use of block (i.e. only take data from a
; specified block

useblock = 0
if keyword_setx(block) then begin
   useblock = 1
   nblock = block
   comblock,filename,nblock,ln_begin,ln_end
   if ln_begin eq -1 then begin
      print,'%RSTAB Warning: Requested block not found'
      print,'                 Proceed reading all data'
      useblock = 0
   endif
endif

; See if requested number of columns matches the actual number

nmax = 30000
n = n_params() - 1
if vb then print,'RSTAB: ', n,' columns expected in '+filename
nc = n_columns(filename)
if n ne nc then begin
   print,'% RSTAB: ', nc,' columns found in file'
   return
endif 

; If the keyword 'coltyp' has been set then check, else define

column_type = replicate(1,n) ; 1 for number, 0 for string
column_indx = indgen(n)
i_str = 0
i_num = 0
n_strings = 0
if keyword_set(coltyp) then begin
   n_coltyp = strlen(coltyp)
   print,' --- num of chars in ''coltyp''', n_coltyp
   if n_coltyp ne n then begin
      print,'% RSTAB: keyword ''coltyp'' has bad string length'
      return
   endif
   for i = 0,n_coltyp-1 do begin
      if strmid(coltyp,i,1) eq 's' then begin
         column_type(i) = 0
         column_indx(i) = i_str
         i_str = i_str + 1
         print,' --- coltyp ', i, ' is s'
         n_strings = n_strings + 1
      endif else begin
         column_indx(i) = i_num
         i_num = i_num + 1
      endelse
   endfor
endif

n_numbers = n - n_strings
if n_numbers le 0 then begin
   print,'% RSTAB: warning, no columns with numbers found'
;   return
endif

; Prepare reading the file

openr,lun,filename,/get_lun
if n_numbers gt 0 then c = fltarr(nmax,n_numbers)
if n_strings gt 0 then cstr = strarr(nmax,n_strings)
; x = fltarr(n_numbers)
; if n_strings gt 0 then xstr = strarr(n_strings)
s = ' '
i = 0
linenumber = 0
while not eof(lun) and i lt nmax do begin
  readf,lun,s
  pos = strpos(s,';')
  if pos ge 0 then ss = strmid(s,0,pos) else ss = s
  st = strtrim(strcompress(ss),2) ; Change multiple blanks to one,
                                  ; remove leading and trailing blanks
  if strlen(st) gt 0 then begin
     ;   check for comment
     if( strpos('+-.0123456789',strmid(st,0,1)) lt 0 ) then begin
           if vb then print,s
     endif else begin
        ;  this line contains data to be read
        sep = strsplit(st,' ',/extract)
        n_sep = n_elements(sep)
        if n_sep ne n then begin
           print,'% RSTRTAB : problem with line: ', s
           return
        endif

        i_num = 0
        i_str = 0
        if useblock then begin
           if linenumber ge ln_begin and linenumber le ln_end then begin
              for j = 0, n-1 do begin
                 if column_type(j) eq 0 then begin
                    cstr(i,i_str) = sep(j)
                    i_str = i_str + 1
                 endif else begin
                    c(i,i_num) = float(sep(j))
                    i_num = i_num + 1
                 endelse
              endfor
              i = i + 1
           endif
        endif else begin
           for j = 0, n-1 do begin
              if column_type(j) eq 0 then begin
                 cstr(i,i_str) = sep(j)
                 i_str = i_str + 1
              endif else begin
                 c(i,i_num) = float(sep(j))
                 i_num = i_num + 1
              endelse
           endfor
           i = i + 1
        endelse
     endelse
   endif
   linenumber = linenumber + 1
endwhile
if vb then print,'Size of table = ',i

if n_numbers gt 0 then c = c(0:i-1,*)
if n_strings gt 0 then cstr = cstr(0:i-1,*)

i_num = 0
i_str = 0

if column_type(0) eq 0 then c1 = cstr(*,column_indx(0)) else c1 = c(*,column_indx(0))
if( n gt 1 ) then begin
   if column_type(1) eq 0 then c2 = cstr(*,column_indx(1)) else c2 = c(*,column_indx(1))
endif
if( n gt 2 ) then begin
   if column_type(2) eq 0 then c3 = cstr(*,column_indx(2)) else c3 = c(*,column_indx(2))
endif
if( n gt 3 ) then begin
   if column_type(3) eq 0 then c4 = cstr(*,column_indx(3)) else c4 = c(*,column_indx(3))
endif
if( n gt 4 ) then begin
   if column_type(4) eq 0 then c5 = cstr(*,column_indx(4)) else c5 = c(*,column_indx(4))
endif
if( n gt 5 ) then begin
   if column_type(5) eq 0 then c6 = cstr(*,column_indx(5)) else c6 = c(*,column_indx(5))
endif
if( n gt 6 ) then begin
   if column_type(6) eq 0 then c7 = cstr(*,column_indx(6)) else c7 = c(*,column_indx(6))
endif
if( n gt 7 ) then begin
   if column_type(7) eq 0 then c8 = cstr(*,column_indx(7)) else c8 = c(*,column_indx(7))
endif
if( n gt 8 ) then begin
   if column_type(8) eq 0 then c9 = cstr(*,column_indx(8)) else c9 = c(*,column_indx(8))
endif
if( n gt 9 ) then begin
   if column_type(9) eq 0 then c10 = cstr(*,column_indx(9)) else c10 = c(*,column_indx(9))
endif
if( n gt 10 ) then begin
   if column_type(10) eq 0 then c11 = cstr(*,column_indx(10)) else c11 = c(*,column_indx(10))
endif
if( n gt 11 ) then begin
   if column_type(11) eq 0 then c12 = cstr(*,column_indx(11)) else c12 = c(*,column_indx(11))
endif
if( n gt 12 ) then begin
   if column_type(12) eq 0 then c13 = cstr(*,column_indx(12)) else c13 = c(*,column_indx(12))
endif
if( n gt 13 ) then begin
   if column_type(13) eq 0 then c14 = cstr(*,column_indx(13)) else c14 = c(*,column_indx(13))
endif
if( n gt 14 ) then begin
   if column_type(14) eq 0 then c15 = cstr(*,column_indx(14)) else c15 = c(*,column_indx(14))
endif
close,lun
free_lun,lun
end
