pro rstab,filename,c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,$
          c11,c12,c13,c14,c15,silent=silent,block=block,nrows=nrows
;
; A procedure in the S-table package
;   (Others are: rtab, wtab, wstab)
;
;  ~1996/NJW
;   2005-03-21/NJW Update to eliminate hard limit on number of rows
;
; This procedure will read an ASCII file organized as a table
; with up to 15 columns. The actual number of columns must be
; specified in the call e.g.
;
;              IDL> rstab,'mytable',x,y,z[,/silent]
;
; if 'mytable' contains 3 columns. 'rstab' will create the arrays
; with length equal to the number of rows in the table
;  Keyword 'silent' for silent i.e. no output to screen
;
; The lines that start with non-digit characters are interpreted as
; comment lines and will be displayed but skipped in
; the reading of the data
;
; If the 'block' keyword is given with a block number
; then the file will be scanned for existence of lines:
;       // block = nnn
; and only the table rows between where nnn matches the
; given block number and the next '// block = xxx'
;
; Other procedures required: n_columns.pro
;                            keyword_setx.pro
;                            comblock.pro
;

if n_params() lt 2 then begin
        print,'Syntax: rstab,filename,c1,c2,...[,/silent][,block=][,nrows=]'
        print,'           Up to and including 15 columns of data'
        print,'           ''silent'' inhibits extra terminal output'
        print,'           ''block''  defines the block number for row section'
        print,'           ''nrows''  returns the number of rows read'
        return
end

vb = 1 - keyword_set(silent) ; keyword for verbosity

; Initiate use of block (i.e. only take data from a
; specified block

useblock = 0
if keyword_setx(block) then begin
   useblock = 1
   nblock = block
   comblock,filename,nblock,ln_begin,ln_end
   if ln_begin eq -1 then begin
      print,'%RSTAB Warning: Requested block not found'
      print,'                 Proceed reading all data'
      useblock = 0
   endif
endif

; See if requested number of columns matches the actual number

nmax = 2000L
n = n_params() - 1
if vb then print,'RSTAB: ', n,' columns expected in '+filename
nc = n_columns(filename)

; Check for data

if nc eq -1 then begin
   nrows = 0
   print,'% RSTAB: no data rows in file'
   return
endif

; Check for correct number of columns

if n ne nc then begin
   print,'% RSTAB: ', nc,' columns found in file'
   return
endif

; Prepare reading the file

openr,lun,filename,/get_lun
c = fltarr(nmax,n)
x = fltarr(n)
s = ' '
i = 0L
linenumber = 0L
while not eof(lun) do begin
  readf,lun,s
  pos = strpos(s,';')
  if pos ge 0 then ss = strmid(s,0,pos) else ss = s
  st = strtrim(ss,1) ; remove leading blanks
  if strlen(st) gt 0 then begin
     if( strpos('+-.0123456789',strmid(st,0,1)) lt 0 ) then begin
           if vb then print,s
     endif else begin
        if useblock then begin
           if linenumber ge ln_begin and linenumber le ln_end then begin
              reads,s,x
              for j = 0, n-1 do c(i,j) = x(j)
              i = i + 1
           endif
        endif else begin
           reads,s,x
           for j = 0, n-1 do c(i,j) = x(j)
           i = i + 1
        endelse
     endelse
   endif
   linenumber = linenumber + 1
   if i eq nmax then begin
           nmax = 2*nmax
           if vb then print,'Expanding to ',nmax
           expand_array, c, nmax, 1
   endif
endwhile
if vb then print,'Size of table = ',i
nrows = i ;  Report back to caller

; Shrink arrays to absolutely necessary

if nrows gt 0 then begin
   c = c(0:i-1,*)
   c1 = c(*,0)
   if( n gt 1 ) then c2 = c(*,1)
   if( n gt 2 ) then c3 = c(*,2)
   if( n gt 3 ) then c4 = c(*,3)
   if( n gt 4 ) then c5 = c(*,4)
   if( n gt 5 ) then c6 = c(*,5)
   if( n gt 6 ) then c7 = c(*,6)
   if( n gt 7 ) then c8 = c(*,7)
   if( n gt 8 ) then c9 = c(*,8)
   if( n gt 9 ) then c10 = c(*,9)
   if( n gt 10 ) then c11 = c(*,10)
   if( n gt 11 ) then c12 = c(*,11)
   if( n gt 12 ) then c13 = c(*,12)
   if( n gt 13 ) then c14 = c(*,13)
   if( n gt 14 ) then c15 = c(*,14)
endif

free_lun,lun
end
pro wtab, filename, c1,c2,c3,c4,c5,c6,c7,c8,c9,c10,c11,c12,c13,c14,c15,fmt=fmt,app=app
;
; Write a table in ASCII with a maximum of 15 columns as indicated
; The format may be given (but without parentheses) as a keyword: fm
;
; Niels J. Westergaard, around 1990, Danish Space Research Institute
;
;   2005-09-21/NJW  Updated with info when no parameters given
;
n = n_params() - 1
if n lt 1 then begin
   print,'Syntax: wtab, filename, c1, ...[,fmt=][,/app]'
   return
endif
if n gt 15 then begin
   print,'%WTAB: number of columns must not exceed 15'
   retall
endif
form = '(e14.5,$)'
if keyword_set(fmt) then form = '('+fmt+',$)'
print, 'WTAB: ', n_elements(c1),' rows and', n,' columns'
if keyword_set(app) then begin
   openw,lun,filename,/get_lun,/append
endif else begin
   openw,lun,filename,/get_lun
endelse
for i = 0L, n_elements(c1) - 1 do begin
   printf,lun,format=form,c1(i)
   if( n gt 1 ) then printf,lun,format=form,c2(i)
   if( n gt 2 ) then printf,lun,format=form,c3(i)
   if( n gt 3 ) then printf,lun,format=form,c4(i)
   if( n gt 4 ) then printf,lun,format=form,c5(i)
   if( n gt 5 ) then printf,lun,format=form,c6(i)
   if( n gt 6 ) then printf,lun,format=form,c7(i)
   if( n gt 7 ) then printf,lun,format=form,c8(i)
   if( n gt 8 ) then printf,lun,format=form,c9(i)
   if( n gt 9 ) then printf,lun,format=form,c10(i)
   if( n gt 10 ) then printf,lun,format=form,c11(i)
   if( n gt 11 ) then printf,lun,format=form,c12(i)
   if( n gt 12 ) then printf,lun,format=form,c13(i)
   if( n gt 13 ) then printf,lun,format=form,c14(i)
   if( n gt 14 ) then printf,lun,format=form,c15(i)
   printf,lun
endfor
close,lun
free_lun,lun
end
pro rscol, filename, col_dat, col_def, str=str, int=int, long=long, dble=dble,$
          silent=silent, block=block, col_num=col_num
;
; A procedure in the S-table package
;   (Others are: rstab, rtab, wtab, wstab)
;
; This procedure will read a column from an ASCII file
; organized as a table
; If the column should be read as a string then
; the keyword 'str' must be set.
; Default type is float. Keywords exist for int, long and double
;
;   IDL> rscol,'mytable',col,col_definiton[,/str][,/int][,/long][,/dble][,/silent][,block=]
;
; 'rscol' will create the array 'col_dat'
; with length equal to the number of rows in the table.
;
;  The columns can be numbers or character strings separated with
;  blanks. The first column must always start with one of +-.0123456789
;  The column asked for must be a number.
;
;  First column number is 1
;
;  Keyword 'silent' for silent i.e. no output to screen
;
; The lines that start with non-digit characters are interpreted as
; comment lines and will be displayed but skipped in
; the reading of the data
;
; If the 'block' keyword is given with a block number
; then the file will be scanned for existence of lines:
;       // block = nnn
; and only the table rows between where nnn matches the
; given block number and the next '// block = xxx'
;
; Other procedures required: n_columns.pro
;                            keyword_setx.pro
;                            comblock.pro
;
if n_params() lt 3 then begin
   print,'RSCOL, (input)filename, (return)col_array, (input)column_definition'
   print,'    column_definition is either a number or a name specified'
   print,'    with keyword ''colname'''
   print,'Keywords:    /str    - a string is returned'
   print,'             /int    - a short integer is returned'
   print,'             /long   - a long integer is returned'
   print,'             /dble   - a double is returned'
   print,'             /silent - no extra messages'
   print,'             block=  - read only this block of rows'
   retall
endif

vb = 1 - keyword_set(silent) ; keyword for verbosity

; Define type of returned array

type = 4 ; float
if keyword_set(str) then type = 7 ; string
if keyword_set(long) then type = 3 ; long integer
if keyword_set(dble) then type = 5 ; double
if keyword_set(int) then type = 2 ; short integer

; If column designator is a string then find the matching
; number

szcf = size(col_def)
if szcf(0) ne 0 then begin
   print,'The column definition must be a scalar'
   retall
endif
if szcf(1) eq 7 then begin
   ; get the column names
   colnames = comgets(filename,'colname',ncols)
   if ncols eq 0 then begin
      print,'Error - no keyword ''colname'' found'
      retall
   endif
   match = 0
   for i = 0, ncols-1 do begin
      if col_def eq colnames(i) then begin
         col_number = i + 1
         match = 1
      endif
   endfor
   if not match then begin
      print,'Error: no such column name has been found'
      retall
   endif
endif else begin
   col_number = fix(col_def)
endelse
col_num = col_number

; Initiate use of block (i.e. only take data from a
; specified block

useblock = 0
if keyword_setx(block) then begin
   useblock = 1
   nblock = block
   comblock,filename,nblock,ln_begin,ln_end
   if ln_begin eq -1 then begin
      print,'%RSTAB Warning: Requested block not found'
      print,'                 Proceed reading all data'
      useblock = 0
   endif
endif

; See if requested column number is less than or equal to
; the number of columns

if vb then print,'RSCOL: at least ', col_number, $
          ' columns expected in '+filename
nc = n_columns(filename)
if vb then print,nc,' columns found in '+filename

if col_number gt nc then begin
   print,'% RSCOL: only ', nc,' columns found in file'
   stop
endif

; Prepare reading the file

nmax = 3000L
cur_size = nmax

openr,lun,filename,/get_lun
case type of
   2 : col_dat = intarr(nmax)
   3 : col_dat = lonarr(nmax)
   4 : col_dat = fltarr(nmax)
   5 : col_dat = dblarr(nmax)
   7 : col_dat = strarr(nmax)
endcase
x = fltarr(nc)
s = ' '
kount = 0L
linenumber = 0L

while not eof(lun) do begin

  readf,lun,s
  pos = strpos(s,';')
  if pos ge 0 then ss = strmid(s,0,pos) else ss = s
  st = strtrim(ss,2) ; remove leading and trailing blanks
  if strlen(st) gt 0 then begin
     if( strpos('+-.0123456789',strmid(st,0,1)) lt 0 ) then begin
           if vb then print,s
     endif else begin
        su = strcompress(st) ; reduce all space to single spaces
        elmnts = strsplit(su,' ',/extract) ; separate into columns
        if useblock then begin
           if linenumber ge ln_begin and linenumber le ln_end then begin
              case type of
                   2 : col_dat(kount) = fix(elmnts(col_number-1))
                   3 : col_dat(kount) = long(elmnts(col_number-1))
                   4 : col_dat(kount) = float(elmnts(col_number-1))
                   5 : col_dat(kount) = double(elmnts(col_number-1))
                   7 : col_dat(kount) = elmnts(col_number-1)
              endcase
              kount = kount + 1L
           endif
        endif else begin
              case type of
                   2 : col_dat(kount) = fix(elmnts(col_number-1))
                   3 : col_dat(kount) = long(elmnts(col_number-1))
                   4 : col_dat(kount) = float(elmnts(col_number-1))
                   5 : col_dat(kount) = double(elmnts(col_number-1))
                   7 : col_dat(kount) = elmnts(col_number-1)
              endcase
           kount = kount + 1L
        endelse

        ; Check if expansion is needed

        if kount eq cur_size then begin
    
           case type of
              2 : col_dat = [col_dat, intarr(nmax)]
              3 : col_dat = [col_dat, lonarr(nmax)]
              4 : col_dat = [col_dat, fltarr(nmax)]
              5 : col_dat = [col_dat, dblarr(nmax)]
              7 : col_dat = [col_dat, strarr(nmax)]
           endcase
           cur_size = cur_size + nmax

        endif
     endelse
   endif
   linenumber = linenumber + 1
endwhile
if vb then print,'Size of table = ',kount
col_dat = col_dat(0:kount-1)
close,lun
free_lun,lun
end

function n_columns, file_name
;
; Return the number of columns in the first line that starts with
; a number i.e. not a comment
;
ncol = -1     ; to be returned if no numbers found
openr,lun,file_name,/get_lun
i = 1
while i eq 1 and not eof(lun) do begin
   line = ' '
   readf,lun,line
   pos = strpos(line,';')
   if pos ge 0 then line = strmid(line,0,pos)
                            ; remove ';' and what follows (comments)
   line = strcompress(line) ; multiple blanks and tabs to single blanks
   line = strtrim(line,2)   ; remove leading and trailing blanks
   len = strlen(line)
   if len gt 0 then begin     ; skip if no characters on line
      cc = strmid(line,0,1)
      if cc eq '.' and len gt 1 then cc = strmid(line,1,1)
      n = strpos('+-0123456789',cc)
      if n ge 0 then begin            ; here we have a number
         for j = 0, len-1 do begin
            cc = strmid(line,j,1)
            if cc eq ',' or cc eq ';' then strput,line,' ',j
         endfor
         line = strtrim(line,2)
         len = strlen(strcompress(line))
         len0= strlen(strcompress(line,/remove_all))
         ncol = len - len0 + 1
         i = 0  ; terminate while loop
      endif
   endif
endwhile
close,lun
free_lun,lun
return,ncol
end

