pro sky2pts, fits_name, out_name, ert=ert, block=block, red=red

; used for converting a FITS celestial image file with proper
; celestial coordinate keywords CRVAL, CRPIX, CDELT, CROTA2
; to a table of (RA,DEC) + intensity for every pixel
;
; It is assumed that the unit in the image is CTS per PIXEL
; and that the conversion countrate to flux is 2.7e-11

; Keywords : ert   error tracking
;            block blocking of image (to avoid too many lines in output)
;            red   reduce fov in a simple way

; analyze keywords

if keyword_set(ert) then et = 1 else et = 0 ; error-track
if keyword_set(block) then blo = block else blo = 1 ; blocking
if keyword_set(red) then rd = red else rd = 0 ; reduction

if n_params() eq 2 then res_name = out_name else res_name = 'output'
print,'The output will appear in ', res_name
openw,lun,res_name,/get_lun

; Include standard routines

astrolib

deg2rad = 0.0174533
; *** cts2flux = 2.7e-11
cts2flux = 2.7e-11

im = readfits(fits_name)
im = im * 1.0
szim = size(im)
if et then print,'image size ', szim(1:2)
if blo ne 1 then im = rebin(im,szim(1)/blo,szim(2)/blo)

;  NOTE: the pixel values do not change since they represent
;        surface brightness

szim = size(im)
if et then print,'image size ', szim(1:2)
   
; Get header information

hdr = headfits(fits_name)

;... get transformation coordinates

extast,hdr,cd,crpix,crval,noparams
cdelt = blo * [cd(0,0), cd(1,1)]
crpix = crpix / blo
tobs = sxpar(hdr,'XS-LIVTI',fits_name)
if et then begin
   print,' crpix = ', crpix
   print,' crval = ', crval
   print,' noparams = ', noparams
   print,' cdelt = ',cdelt
endif

;... get the celestial coordinate transformation

ra_x = crval(0) * deg2rad
dc_x = crval(1) * deg2rad
;* teta = 0.0   ; roll (assumed to be zero from 'revtrans')
teta_deg = -90.
teta = teta_deg * deg2rad
s_sc = fltarr(3)

; There is some confusion about the definition of the roll angle
; and the coordinate systems. For an image given in S/C coordinates
; that is upright i.e. North is positive Z, East is positive Y
; the roll (here 'teta') must be -90 deg

; Pointing in EQ system
 
x_eq = [cos(ra_x)*cos(dc_x), sin(ra_x)*cos(dc_x), sin(dc_x)]
 
; S/C Y-axis direction in EQ system
 
y_eq = [sin(ra_x)*sin(teta)-sin(dc_x)*cos(ra_x)*cos(teta),$
        -cos(ra_x)*sin(teta)-sin(dc_x)*sin(ra_x)*cos(teta),$
        cos(dc_x)*cos(teta)]
 
; S/C Z-axis direction in EQ system
 
z_eq = [sin(ra_x)*cos(teta)+sin(dc_x)*cos(ra_x)*sin(teta),$
        -cos(ra_x)*cos(teta)+sin(dc_x)*sin(ra_x)*sin(teta),$
        -cos(dc_x)*sin(teta)]
 
; Define transformation matrix
 
omega = fltarr(3,3)
omega(*,0) = x_eq
omega(*,1) = y_eq
omega(*,2) = z_eq
 
;... Go through all the pixels and derive coordinates


for i = rd, szim(1)-1-rd do begin
   for j = rd, szim(2)-1-rd do begin
   
      y_sc = (i - crpix(0))*cdelt(0)*deg2rad
      z_sc = (j - crpix(1))*cdelt(1)*deg2rad
   
      ; Go back again to RA and Dec
    
      s_sc(1) = tan(y_sc)
      s_sc(2) = tan(z_sc)
      s_sc(0) = sqrt(1 - s_sc(1)^2 - s_sc(2)^2)
    
      s_eq = omega # s_sc
    
      ra = atan(s_eq(1), s_eq(0))
      if ra lt 0 then ra = ra + 2*!pi
      dc = asin(s_eq(2))
      if im(i,j) gt 0 then $
         printf,lun,adstring(ra/deg2rad, dc/deg2rad),$
                 im(i,j)*blo*blo*cts2flux/tobs,$
                 format='(a," 1.0E+21   SX    TB",e9.1,"   2.0")'
   endfor
endfor
close,lun
free_lun,lun

end
