pro skyplot, fits_name_extno,thr=thr, ul=ul, nk=nk, ert=ert, noc=noc,$
             wno=wno,pts=pts,psz=psz,auto=auto,f1=f1,f2=f2,resim=resim

; used for plotting a FITS file with proper celestial coordinate
; keywords CRVAL, CRPIX, CDELT, CROTA2
;
;  The image extension is given as  myimage.fits[2] or
;                                   myimage.fits+2
;
;  If no extension number is given, it is assumed that the primary
;  array is requested.

; Keywords : thr   data (lower) threshold
;            ul    upper limit [both of these set for nicer plotting]
;            nk    number of foldings with quasi gaussian kernel
;            ert   error tracking
;            noc   set if no contours are wanted
;            wno   window number for plotting on screen
;            pts   list of (RA,DEC) in degrees to be added on plot
;                  as points: Dim pts(2,n) where n is the number
;                  of points,  pts(0,*) the RA values, pts(1,*) DEC
;            psz   array with point sizes, Dim n
;            auto  automatic setting of thr, ul, and nk

if n_params() eq 0 then begin
   print,'Syntax: SKYPLOT, fits_name+extno, thr=, ul=, nk=$'
   print,'                 ert=, noc=, who=, pts=, psz=,auto='
   print
   print,'   Keyword      Effect'
   print,'  thr          Threshold'
   print,'  ul           Upper limit'
   print,'  nk           Number of foldings with Gaussian kernel'
   print,'  ert          Error tracking'
   print,'  wno          Window number for plotting on screen'
   print,'  pts          List of sources to add'
   print,'  psz          List of point sizes (for source list)'
   print,'  auto         For automatic setting of thr, ul, and nk'
   print,'  f1           Fraction of pixels above ul'
   print,'  f2           Fraction of pixels above thr'
   print,'  resim        Returns resulting image'
   return
endif

; Reset axis styles

!x.style = 1
!y.style = 17

; analyze keywords

if keyword_set(ert) then et = 1 else et = 0 ; error-track
if keyword_set(wno) then wo = wno else wo = 0 ; error-track
n_pts = 0
if keyword_set(pts) then begin
   ra_pts = pts(0,*)
   dc_pst = pts(1,*)
   n_pts = n_elements(ra_pts)
endif
n_psz = 0
if keyword_set(psz) then begin
   if n_pts eq 0 then begin
      print,'SKYPLOT error, no points have been defined'
      retall
   endif
   n_psz = n_elements(psz)
   if n_pts ne n_psz then begin
      print,'SKYPLOT error, number of point sizes does not match pts'
      retall
   endif
endif
if n_pts gt 0 and n_psz eq 0 then psz = replicate(1.,n_pts)

if keyword_set(thr) then begin
   threshold = thr
   kthr = 1
endif else kthr = 0
if keyword_set(ul) then begin
   upper_limit = ul
   kul = 1
endif else kul = 0
if keyword_set(nk) then begin
   msmooth = nk
   knk = 1
endif else knk = 0

kauto = 0
if keyword_set(auto) then begin
   kauto = 1
endif

; Find the image extension number

get_exten_no, fits_name_extno, fits_name, extno

; Include standard routines

astrolib

deg2rad = 0.0174533

im = readfits(fits_name, hdr, exten_no=extno)
szim = size(im)
nelem = szim(szim(0)+2)
xsize = szim(1)
ysize = szim(2)
if et then print,'image size ', xsize, ysize

; Make suitable background to subtract

im_no_extremes = rem_extremes(im, nsig=3)
bkg_image = gauss_convol_2d(im_no_extremes, 2.5, val='s', ks=15)

; Define best fitting factor

dum = bkg_subtract(im_no_extremes, bkg_image, kres=k)

; Use this factor to get resulting image

im_no_bkg = bkg_subtract(im, bkg_image, fac=k)

; Determine max and min of image

maxim,im_no_bkg,mx,ixmx,iymx
mn = min(im_no_bkg)

to_ps = 0
if !d.name eq 'PS' then to_ps = 1



; If keyword 'auto' has been set then override anything else

if not kauto then begin

; Plot histogram of image values to set the threshold

if kthr+kul lt 2 then begin
   set_plot,'X'
   window,wo
   step = (mx-mn)/100.
   h = histogram(im_no_bkg,binsize=step)
   hmx = max(h,indx)
   h(indx) = 0
   x = xlingen(n_elements(h),mn,mx)
   plot,x,h,psym=10
   if kthr eq 0 then begin
      threshold = 0.0
      print,'Use cursor to select lower threshold ...'
      cursor,threshold,yc,/data
      print,'You selected ', threshold, '  as threshold'
      wait,0.3
   endif
   if kul eq 0 then begin
      print,'Use cursor to select upper limit ...'
      cursor,upper_limit,yc,/data
      print,'You selected ', upper_limit, '  as upper limit'
   endif
   wdelete,wo
endif

endif else begin  ;   Start making automatic choices

   ; Automatic setting of threshold and upper limit
   
   ; Set upper limit so that 1% of the image values
   ;   exceed the upper limit
   ;
   ; Set threshold so that 80% of the image values are
   ;   below the threshold
   
   ; Binary search for upper limit
   
   ul2 = mx
   ul1 = mn
   fr1 = 0.0002
   if keyword_set(f1) then fr1 = f1
   n_wanted = fr1*nelem
   
   while ul2-ul1 gt 0.01*(mx-mn) do begin
      upper_limit = 0.5*(ul1 + ul2)
      dum = where(im_no_bkg gt upper_limit, n_above)
      if n_above ge n_wanted then ul1 = upper_limit $
                   else ul2 = upper_limit
   endwhile
   print,'Upper limit became ', upper_limit
   
   ; Binary search for threshold
   
   ul2 = mx
   ul1 = mn
   fr2 = 0.01
   if keyword_set(f2) then fr2 = f2
   n_wanted = fr2*nelem
   
   while ul2-ul1 gt 0.01*(mx-mn) do begin
      threshold = 0.5*(ul1 + ul2)
      dum = where(im_no_bkg gt threshold, n_above)
      if n_above ge n_wanted then ul1 = threshold $
                   else ul2 = threshold
   endwhile
   print,'Threshold became ', threshold

endelse




if to_ps then set_plot,'PS'
im_no_bkg = im_no_bkg < upper_limit
im_no_bkg = (im_no_bkg > threshold) - threshold

; Ask for folding with kernel

kernel = [[0.2,.3,.2],[0.3,1.,.3],[0.2,.3,.2]]
print,'Peak of ', mx, ' at (x,y)pix = ', ixmx, iymx

if not kauto then begin

   if knk eq 0 then begin
      msmooth = 0
      read,'Enter number of kernel foldings : ', msmooth
   endif

endif else begin

   msmooth = 1

endelse

for i = 0, msmooth-1 do im_no_bkg = convol(im_no_bkg,kernel)

maxim,im_no_bkg,mx,ixmx,iymx
print,'Peak of ', mx, ' at (x,y)pix = ', ixmx, iymx

resim = im_no_bkg ; if requested by 'resim' keyword


;- old version:  hdr = headfits(fits_name)

;... get transformation coordinates

extast,hdr,cd,crpix,crval,noparams
cdelt = [cd(0,0), cd(1,1)]
if et then begin
   print,' crpix = ', crpix
   print,' crval = ', crval
   print,' noparams = ', noparams
   print,' cdelt = ',cdelt
endif

;... get the celestial coordinates of the four corners of the image

ra_x = crval(0) * deg2rad
dc_x = crval(1) * deg2rad
;* teta = 0.0   ; roll (assumed to be zero from 'revtrans')
teta_deg = -90.
teta = teta_deg * deg2rad
s_sc = fltarr(3)

; There is some confusion about the definition of the roll angle
; and the coordinate systems. For an image given in S/C coordinates
; that is upright i.e. North is positive Z, East is positive Y
; the roll (here 'teta') must be -90 deg

; Pointing in EQ system
 
x_eq = [cos(ra_x)*cos(dc_x), sin(ra_x)*cos(dc_x), sin(dc_x)]
 
; S/C Y-axis direction in EQ system
 
y_eq = [sin(ra_x)*sin(teta)-sin(dc_x)*cos(ra_x)*cos(teta),$
        -cos(ra_x)*sin(teta)-sin(dc_x)*sin(ra_x)*cos(teta),$
        cos(dc_x)*cos(teta)]
 
; S/C Z-axis direction in EQ system
 
z_eq = [sin(ra_x)*cos(teta)+sin(dc_x)*cos(ra_x)*sin(teta),$
        -cos(ra_x)*cos(teta)+sin(dc_x)*sin(ra_x)*sin(teta),$
        -cos(dc_x)*sin(teta)]
 
; Define transformation matrix
 
omega = fltarr(3,3)
omega(*,0) = x_eq
omega(*,1) = y_eq
omega(*,2) = z_eq
 
;... Do the four corners : lo left, lo right, up left, up right
;     and center

ycorner = [0,szim(1)-1,0,szim(1)-1]
zcorner = [0,0,szim(2)-1,szim(2)-1]
ycor = fltarr(2)
zcor = fltarr(2)

for corner = 0, 3 do begin
   y_sc = (ycorner(corner) - crpix(0))*cdelt(0)*deg2rad
   z_sc = (zcorner(corner) - crpix(1))*cdelt(1)*deg2rad

   ; Go back again to RA and Dec
 
; Here 'atan' has been changed to 'tan'  NJW 970107
   s_sc(1) = tan(y_sc)
   s_sc(2) = tan(z_sc)
   s_sc(0) = sqrt(1 - s_sc(1)^2 - s_sc(2)^2)
 
   s_eq = omega # s_sc
 
   ra = atan(s_eq(1), s_eq(0))
   if ra lt 0 then ra = ra + 2*!pi
   dc = asin(s_eq(2))
   if et then begin
      print,corner, '  in EQ RA, DEC = ',ra/deg2rad, dc/deg2rad
      print,'        or ',adstring(ra/deg2rad, dc/deg2rad)
   endif
   if corner eq 0 then ycor(0) = ra/deg2rad
   if corner eq 0 then zcor(0) = dc/deg2rad
   if corner eq 1 then ycor(1) = ra/deg2rad
   if corner eq 2 then zcor(1) = dc/deg2rad
endfor

;... plot contour plot

wsize = 500
if !d.name eq 'X' then window,wo,xs=wsize,ys=wsize
!p.position = [0.125,0.15,0.875,0.9]
xfsize = wsize * (!p.position(2)-!p.position(0))
yfsize = wsize * (!p.position(3)-!p.position(1))
xfoffset = wsize * !p.position(0)
yfoffset = wsize * !p.position(1)

xtics = 8
ytics = 8

pixx = xsize / xtics
pixy = ysize / ytics

tics,ycor(0),ycor(1),xsize,pixx,raincr,/RA
;* print,' raincr = ', raincr, '  pixx = ', pixx
tics,zcor(0),zcor(1),ysize,pixy,decincr
;* print,' decincr = ', decincr, '  pixy = ', pixy
tic_one,ycor(0),pixx,raincr,botmin,xtic1,/RA
;* print,' botmin = ', botmin,' xtic1 = ', xtic1
tic_one,zcor(0),pixy,decincr,leftmin,ytic1
;* print,' leftmin = ', leftmin,' ytic1 = ', ytic1
nx = fix((xsize-1-xtic1)/pixx)
ny = fix((ysize-1-ytic1)/pixy)
;* print,'nx ny = ', nx,ny
ra_grid = (botmin + findgen(nx+1)*raincr/4.)*deg2rad
dec_grid = (leftmin + findgen(ny+1)*decincr/60.)*deg2rad
;* print,'ra_grid : ', ra_grid
;* print,'dec_grid : ', dec_grid
ticlabels,botmin,nx+1,raincr,xlab,/RA
ticlabels,leftmin,ny+1,decincr,ylab
;* print,'xlab : ', xlab
;* print,'ylab : ', ylab

xpos = fltarr(nx+1)
ypos = fltarr(ny+1)
for i = 0, nx do begin
   ad2yz, ra_grid(i)/deg2rad, zcor(0), omega, crpix, cdelt, y, z
   xpos(i) = y
endfor
for i = 0, ny do begin
   ad2yz, ycor(0), dec_grid(i)/deg2rad, omega, crpix, cdelt, y, z
   ypos(i) = z
endfor
;* print,'xpos = ', xpos
;* print,'ypos = ', ypos

blankx = replicate(' ',nx+1)
blanky = replicate(' ',ny+1)

levs = xlingen(4,0.6*mx,0.95*mx)
if keyword_set(noc) then levs = [1.1*mx, 1.2*mx]
contour,im_no_bkg, xticks=nx, yticks=ny, xtickv=xpos, ytickv=ypos,$
            xtickname=blankx,ytickname=blanky, ticklen=0, $
            levels=levs,title=fits_name_extno,$
            xtitle='RA (2000)',ytitle='DEC (2000)'
if !d.name eq 'X' then tvscl,congrid(im_no_bkg,xfsize,yfsize),xfoffset,yfoffset
if !d.name eq 'PS' then tvscl,-im_no_bkg,!x.window(0),!y.window(0),$
                        xsize=!x.window(1)-!x.window(0),$
                        ysize=!y.window(1)-!y.window(0),/norm
contour,im_no_bkg, xticks=nx, yticks=ny, xtickv=xpos, ytickv=ypos,$
            xtickname=blankx,ytickname=blanky, ticklen=0, $
            levels=levs,title=fits_name_extno,$
            xtitle='RA (2000)',ytitle='DEC (2000)',/noerase

;... add tick labels

for i = 0, nx do begin
   xyouts,xpos(i),-ysize/25.,xlab(i),align=0.5
endfor
for j = 0, ny do begin
   xyouts,xsize*1.025,ypos(j),ylab(j),align=0.0
endfor

;... add grid

xp = fltarr(nx+3,ny+3)
yp = fltarr(nx+3,ny+3)
rag = [ycor(0),ra_grid/deg2rad,ycor(1)]
decg = [zcor(0),dec_grid/deg2rad,zcor(1)]
for i = 0, nx+2 do begin
   for j = 0, ny+2 do begin
      ad2yz, rag(i), decg(j), omega, crpix, cdelt, y, z
      xp(i,j) = y
      yp(i,j) = z
   endfor
   if i gt 0 and i lt nx+2 then oplot,xp(i,*),yp(i,*),li=2
endfor
for j = 1, ny+1 do oplot,xp(*,j),yp(*,j),li=2

;... add extra points

if n_pts gt 0 then begin
   for i = 0, n_pts-1 do begin
      ad2yz, pts(0,i), pts(1,i), omega, crpix, cdelt, y, z
      oplot,[y],[z],psym=7,symsize=psz(i)
   endfor
endif

end
