pro skyplot_auto, fits_name, nk=nk, ert=ert, noc=noc,$
             wno=wno,pts=pts,psz=psz, nos=nos, bias=bias

; used for plotting a FITS file with proper celestial coordinate
; keywords CRVAL, CRPIX, CDELT, CROTA2

; Keywords :
;            nk    number of foldings with quasi gaussian kernel
;            ert   error tracking
;            noc   set if no contours are wanted
;            nos   set if no squeeze is wanted
;            wno   window number for plotting on screen
;            pts   list of (RA,DEC) in degrees to be added on plot
;                  as points: Dim pts(2,n) where n is the number
;                  of points,  pts(0,*) the RA values, pts(1,*) DEC
;            psz   array with point sizes, Dim n

; Reset axis styles

!x.style = 1
!y.style = 17

; analyze keywords

if keyword_set(ert) then et = 1 else et = 0 ; error-track
if keyword_set(wno) then wo = wno else wo = 0 ; error-track
n_pts = 0
if keyword_set(pts) then begin
   ra_pts = pts(0,*)
   dc_pst = pts(1,*)
   n_pts = n_elements(ra_pts)
endif
n_psz = 0
if keyword_set(psz) then begin
   if n_pts eq 0 then begin
      print,'SKYPLOT error, no points have been defined'
      retall
   endif
   n_psz = n_elements(psz)
   if n_pts ne n_psz then begin
      print,'SKYPLOT error, number of point sizes does not match pts'
      retall
   endif
endif
if n_pts gt 0 and n_psz eq 0 then psz = replicate(1.,n_pts)

;  Set background color

colorbias = 0.1
if keyword_set(bias) then colorbias = bias


sznk = size(nk)
if sznk(0)+sznk(1) ne 0 then begin
   msmooth = nk
   knk = 1
endif else knk = 0

; Include standard routines

astrolib

deg2rad = 0.0174533

im = readfits(fits_name)
szim = size(im)
xsize = szim(1)
ysize = szim(2)
if et then print,'image size ', xsize, ysize
maxim,im,mx,ixmx,iymx
mn = min(im)
nelem = szim(szim(0)+2)

; Automatic setting of threshold and upper limit

; Set upper limit so that 1% of the image values
;   exceed the upper limit
;
; Set threshold so that 60% of the image values are
;   below the threshold

; Binary search for upper limit

ul2 = mx
ul1 = mn
n_wanted = nelem/1000

while ul2-ul1 gt 0.01*(mx-mn) do begin
   upper_limit = 0.5*(ul1 + ul2)
   dum = where(im gt upper_limit, n_above)
   if n_above ge n_wanted then ul1 = upper_limit $
                else ul2 = upper_limit
endwhile
print,'Upper limit became ', upper_limit

; Binary search for threshold

ul2 = mx
ul1 = mn
n_wanted = 0.70 * nelem

while ul2-ul1 gt 0.01*(mx-mn) do begin
   threshold = 0.5*(ul1 + ul2)
   dum = where(im gt threshold, n_above)
   if n_above ge n_wanted then ul1 = threshold $
                else ul2 = threshold
endwhile
print,'Threshold became ', threshold

;* im = im < upper_limit
if not keyword_set(nos) then im = squeeze(im, upper_limit)
im = (im > threshold) - threshold

; Ask for folding with kernel

kernel = [[0.2,.3,.2],[0.3,1.,.3],[0.2,.3,.2]]
print,'Peak of ', mx, ' at (x,y)pix = ', ixmx, iymx
if knk eq 0 then begin
   msmooth = 0
   read,'Enter number of kernel foldings : ', msmooth
endif
for i = 0, msmooth-1 do im = convol(im,kernel)
maxim,im,mx,ixmx,iymx
print,'Peak of ', mx, ' at (x,y)pix = ', ixmx, iymx
hdr = headfits(fits_name)

;... get transformation coordinates

extast,hdr,cd,crpix,crval,noparams
cdelt = [cd(0,0), cd(1,1)]
if et then begin
   print,' crpix = ', crpix
   print,' crval = ', crval
   print,' noparams = ', noparams
   print,' cdelt = ',cdelt
endif

;... get the celestial coordinates of the four corners of the image

ra_x = crval(0) * deg2rad
dc_x = crval(1) * deg2rad
;* teta = 0.0   ; roll (assumed to be zero from 'revtrans')
teta_deg = -90.
teta = teta_deg * deg2rad
s_sc = fltarr(3)

; There is some confusion about the definition of the roll angle
; and the coordinate systems. For an image given in S/C coordinates
; that is upright i.e. North is positive Z, East is positive Y
; the roll (here 'teta') must be -90 deg

; Pointing in EQ system
 
x_eq = [cos(ra_x)*cos(dc_x), sin(ra_x)*cos(dc_x), sin(dc_x)]
 
; S/C Y-axis direction in EQ system
 
y_eq = [sin(ra_x)*sin(teta)-sin(dc_x)*cos(ra_x)*cos(teta),$
        -cos(ra_x)*sin(teta)-sin(dc_x)*sin(ra_x)*cos(teta),$
        cos(dc_x)*cos(teta)]
 
; S/C Z-axis direction in EQ system
 
z_eq = [sin(ra_x)*cos(teta)+sin(dc_x)*cos(ra_x)*sin(teta),$
        -cos(ra_x)*cos(teta)+sin(dc_x)*sin(ra_x)*sin(teta),$
        -cos(dc_x)*sin(teta)]
 
; Define transformation matrix
 
omega = fltarr(3,3)
omega(*,0) = x_eq
omega(*,1) = y_eq
omega(*,2) = z_eq
 
;... Do the four corners : lo left, lo right, up left, up right
;     and center

ycorner = [0,szim(1)-1,0,szim(1)-1]
zcorner = [0,0,szim(2)-1,szim(2)-1]
ycor = fltarr(2)
zcor = fltarr(2)

for corner = 0, 3 do begin
   y_sc = (ycorner(corner) - crpix(0))*cdelt(0)*deg2rad
   z_sc = (zcorner(corner) - crpix(1))*cdelt(1)*deg2rad

   ; Go back again to RA and Dec
 
; Here 'atan' has been changed to 'tan'  NJW 970107
   s_sc(1) = tan(y_sc)
   s_sc(2) = tan(z_sc)
   s_sc(0) = sqrt(1 - s_sc(1)^2 - s_sc(2)^2)
 
   s_eq = omega # s_sc
 
   ra = atan(s_eq(1), s_eq(0))
   if ra lt 0 then ra = ra + 2*!pi
   dc = asin(s_eq(2))
   if et then begin
      print,corner, '  in EQ RA, DEC = ',ra/deg2rad, dc/deg2rad
      print,'        or ',adstring(ra/deg2rad, dc/deg2rad)
   endif
   if corner eq 0 then ycor(0) = ra/deg2rad
   if corner eq 0 then zcor(0) = dc/deg2rad
   if corner eq 1 then ycor(1) = ra/deg2rad
   if corner eq 2 then zcor(1) = dc/deg2rad
endfor

;... plot contour plot

wsize = 500
if !d.name eq 'X' then window,wo,xs=wsize,ys=wsize
!p.position = [0.125,0.15,0.875,0.9]
xfsize = wsize * (!p.position(2)-!p.position(0))
yfsize = wsize * (!p.position(3)-!p.position(1))
xfoffset = wsize * !p.position(0)
yfoffset = wsize * !p.position(1)

xtics = 8
ytics = 8

pixx = xsize / xtics
pixy = ysize / ytics

tics,ycor(0),ycor(1),xsize,pixx,raincr,/RA
;* print,' raincr = ', raincr, '  pixx = ', pixx
tics,zcor(0),zcor(1),ysize,pixy,decincr
;* print,' decincr = ', decincr, '  pixy = ', pixy
tic_one,ycor(0),pixx,raincr,botmin,xtic1,/RA
;* print,' botmin = ', botmin,' xtic1 = ', xtic1
tic_one,zcor(0),pixy,decincr,leftmin,ytic1
;* print,' leftmin = ', leftmin,' ytic1 = ', ytic1
nx = fix((xsize-1-xtic1)/pixx)
ny = fix((ysize-1-ytic1)/pixy)
;* print,'nx ny = ', nx,ny
ra_grid = (botmin + findgen(nx+1)*raincr/4.)*deg2rad
dec_grid = (leftmin + findgen(ny+1)*decincr/60.)*deg2rad
;* print,'ra_grid : ', ra_grid
;* print,'dec_grid : ', dec_grid
ticlabels,botmin,nx+1,raincr,xlab,/RA
ticlabels,leftmin,ny+1,decincr,ylab
;* print,'xlab : ', xlab
;* print,'ylab : ', ylab

xpos = fltarr(nx+1)
ypos = fltarr(ny+1)
for i = 0, nx do begin
   ad2yz, ra_grid(i)/deg2rad, zcor(0), omega, crpix, cdelt, y, z
   xpos(i) = y
endfor
for i = 0, ny do begin
   ad2yz, ycor(0), dec_grid(i)/deg2rad, omega, crpix, cdelt, y, z
   ypos(i) = z
endfor
;* print,'xpos = ', xpos
;* print,'ypos = ', ypos

blankx = replicate(' ',nx+1)
blanky = replicate(' ',ny+1)

levs = xlingen(4,0.6*mx,0.95*mx)
if keyword_set(noc) then levs = [1.1*mx, 1.2*mx]
contour,im, xticks=nx, yticks=ny, xtickv=xpos, ytickv=ypos,$
            xtickname=blankx,ytickname=blanky, ticklen=0, $
            levels=levs,title=fits_name,$
            xtitle='RA (2000)',ytitle='DEC (2000)'
;* old > if !d.name eq 'X' then tvscl,congrid(im,xfsize,yfsize),xfoffset,yfoffset
if !d.name eq 'X' then $
     colorplot,congrid(im,xfsize,yfsize),colorbias,xfoffset,yfoffset
;* old > if !d.name eq 'PS' then tvscl,-im,!x.window(0),!y.window(0),$
;* old >                         xsize=!x.window(1)-!x.window(0),$
;* old >                         ysize=!y.window(1)-!y.window(0),/norm
if !d.name eq 'PS' then colorplot,im,colorbias,!x.window(0),!y.window(0),$
                        xsize=!x.window(1)-!x.window(0),$
                        ysize=!y.window(1)-!y.window(0),/norm
contour,im, xticks=nx, yticks=ny, xtickv=xpos, ytickv=ypos,$
            xtickname=blankx,ytickname=blanky, ticklen=0, $
            levels=levs,title=fits_name,$
            xtitle='RA (2000)',ytitle='DEC (2000)',/noerase

;... add tick labels

for i = 0, nx do begin
   xyouts,xpos(i),-ysize/25.,xlab(i),align=0.5
endfor
for j = 0, ny do begin
   xyouts,xsize*1.025,ypos(j),ylab(j),align=0.0
endfor

;... add grid

icol = !d.n_colors - 1
xp = fltarr(nx+3,ny+3)
yp = fltarr(nx+3,ny+3)
rag = [ycor(0),ra_grid/deg2rad,ycor(1)]
decg = [zcor(0),dec_grid/deg2rad,zcor(1)]
for i = 0, nx+2 do begin
   for j = 0, ny+2 do begin
      ad2yz, rag(i), decg(j), omega, crpix, cdelt, y, z
      xp(i,j) = y
      yp(i,j) = z
   endfor
   if i gt 0 and i lt nx+2 then oplot,xp(i,*),yp(i,*),li=2,color=icol
endfor
for j = 1, ny+1 do oplot,xp(*,j),yp(*,j),li=2,color=icol

;... add extra points

if n_pts gt 0 then begin
   def_sym,'c'
   for i = 0, n_pts-1 do begin
      ad2yz, pts(0,i), pts(1,i), omega, crpix, cdelt, y, z
      oplot,[y],[z],psym=8,symsize=psz(i),color=icol
   endfor
endif

end
