pro skypos_fits, arg1, pix1, pix2, coord1, coord2,dir=dir, hdr=hdr, r1=r1,r2=r2, silent=silent
;
;  Get sky position for given position in FITS image
;  pix1 and pix2 are pixel numbers starting count with 1 (dir = 'sky')
;  i.e. 1 larger than the output of 'find_strongest_source'
;  or pixel numbers from coordinates (dir = 'pix')
;
;  First argument is either a FITS DOL or a FITS header in which case the
;      keyword 'hdr' must be set
;
;   2003-09-16/NJW
;   2003-12-17/NJW Updated to derive pixel numbers from coords
;
if n_params() lt 5 then begin
   print,'Syntax: skypos_fits, file_ext, pix1, pix2, coord1, coord2[,dir=][,/hdr][,r1][,r2][,/silent]'
   return
endif

if keyword_set(silent) then vb = 0 else vb = 1
if not keyword_set(dir) then dir = 'sky'
if dir ne 'sky' and dir ne 'pix' then begin
   print,'Keyword ''dir'' MUST have value ''sky'' or ''pix'''
   return
endif

if not keyword_set(hdr) then begin ; 1. argument is a DOL

   file_ext = arg1

   get_exten_no, file_ext, file, extno

   if file_test(file,/reg) eq 0 then begin
      print,'file '+file+' not found!'
      return
   endif

   hdr = headfits(file, exten=extno)
   sz = size(hdr)
   if sz(sz(0)+1) ne 7 then begin
      print,'Requested extension ('+stri(extno)+') not found!'
      return
   endif

endif else hdr = arg1

dr = !pi / 180

nms = 0
crval1 = fxpar(hdr,'crval1',count=nm)
nms = nms + nm
crval2 = fxpar(hdr,'crval2',count=nm)
nms = nms + nm
crpix1 = fxpar(hdr,'crpix1',count=nm)
nms = nms + nm
if keyword_set(r1) then crpix1 = r1
crpix2 = fxpar(hdr,'crpix2',count=nm)
nms = nms + nm
if keyword_set(r2) then crpix2 = r2
cd1_1  = fxpar(hdr,'cd1_1',count=nm)
nms = nms + nm
cd1_2  = fxpar(hdr,'cd1_2',count=nm)
nms = nms + nm
cd2_1  = fxpar(hdr,'cd2_1',count=nm)
nms = nms + nm
cd2_2  = fxpar(hdr,'cd2_2',count=nm)
nms = nms + nm
if nms ne 8 then begin
   print,'A required keyword is missing!'
   return
endif

det = cd1_1 * cd2_2 - cd1_2 * cd2_1

posangle = atan(cd1_2,cd1_1) / dr

cdmat = [[cd1_1,cd2_1],[cd1_2,cd2_2]]

if vb then begin
   print,'CRVAL1 CRVAL2 : ', crval1, crval2
   print,'Derived POSANGLE : ', posangle
   print,'CRPIX1 CRPIX2 : ', crpix1, crpix2
   print,'CD1_1 CD1_2 : ', cd1_1, cd1_2
   print,'CD2_1 CD2_2 : ', cd2_1, cd2_2
endif
;
;
a0 = crval1 * dr
ca0 = cos(a0)
sa0 = sin(a0)
d0 = crval2 * dr
cd0 = cos(d0)
sd0 = sin(d0)

; transformation matrices

t1 = [[ca0,sa0,0],[-sa0,ca0,0],[0,0,1]]
t2 = [[cd0,0,sd0],[0,1,0],[-sd0,0,cd0]]

t = t1#t2    ;* #t3

if dir eq 'sky' then begin

   ; star position in image (y,z) expressed in degrees
   yz = cdmat # [pix1-crpix1,pix2-crpix2]
   y = atan(yz(0) * dr)
   z = atan(yz(1) * dr)
   ; unit vector expressed in image system
   vi = [0,sin(y),sin(z)]
   vi(0) = sqrt(1. - vi(1)^2 - vi(2)^2)
   
   ; star position in eq
   
   ve = t # vi
   
   ; derive RA and Dec
   
   d = asin(ve(2))
   a = zero2pi(atan(ve(1),ve(0)))
   ddeg = d / dr
   adeg = a / dr
   
   if vb then print,'RA: ', adeg,'  Dec: ',ddeg
   
   coord1 = adeg
   coord2 = ddeg

endif else begin

   arad = coord1 * dr
   drad = coord2 * dr

   ve = [cos(arad)*cos(drad),sin(arad)*cos(drad),sin(drad)]
   vi = invert( t ) # ve

   yz = tan(asin(vi(1:2))) / dr

   p = invert(cdmat) # yz + [crpix1,crpix2]

   pix1 = p(0)
   pix2 = p(1)

endelse

return
end

