pro substitute_arf, arfdir, datadir, filelist=filelist, txtfile=txtfile
;
;  Change the ARF in jmxi_srcl_arf.fits+1
;
;   2003-05-07/NJW
;   2004-03-24/NJW updated to no longer deal with 'eflt'
;   2004-05-13/NJW searches directory tree and gets information
;                  from the jmxi_srcl_spe.fits files
;   2004-05-18/NJW & Ruben Farinelli: The spectral files can now
;   Version c      be obtained from a text file by using keyword 'txtfile='
;                  and a detailed log file is produced
;

if n_params() lt 2 then begin
   print,'Syntax: substitute_arf, arfdir, datadir[,filelist=][,txtfile=]'
   print,'  where ''arfdir'' is the directory where jmx1_all_arf_0003.fits'
   print,'                  and jmx2_all_arf_0003.fits can be found'
   print,'       ''datadir'' is the top directory for the search for'
   print,'       files with name ''jmx?_srcl_spe.fits'' where the spectra are found'
   print
   print,' If the keyword ''filelist'' is given it is assumed that it is a string'
   print,'       array that contains a list of all spectral files where ARF update'
   print,'       is requested. The path given in ''datadir'' will be disregarded.'
   print
   print,' If the keyword ''txtfile'' is given it is assumed that it is the name'
   print,'       of a text file that contains a list of spectral files. Also here'
   print,'       the path in ''datadir'' is disregarded.'
   print
   print, 'Examples'
   print, 'substitute_arf, ''/userdir/'', ''/REP_BASE_PROD/obs/og_id/scw/''
   print, 'to substitute all ARF under a same OG'
   print
   print,'substitute_arf, ''/userdir'', ''datadir'', $'
   print,'       filelist=[''/my_source/obs1/res/jmx1_srcl_spe.fits'', $'
   print,'                 ''/my_source/obs2/res/jmx2_srcl_spe.fits'']'
   print
   print,'substitute_arf, ''/userdir'', ''datadir'', $'
   print,'       txtfile=''/userdir/arf_repair/test/list_of_files''
   print, 'where list_of_files has the content:'
   print, '/vesta/INTEGRAL/.../scw/004600200010/jmx2/res/jmx1_srcl_spe.fits'
   print, '/vesta/INTEGRAL/.../scw/004600210010/jmx2/res/jmx1_srcl_spe.fits'
   print
   print,' It is strongly recommended to make sure that you can (easily)'
   print,' recreate the relevant files before running this procedure'
   print
  return
endif


all_arf_file = [arfdir+'/jmx1_all_arf_0003.fits',arfdir+'/jmx2_all_arf_0003.fits']

if not file_test(all_arf_file(0),/reg) or not file_test(all_arf_file(1),/reg) then begin
   print,'No action - ARF catalogs not found where expected'
   return
endif

; Get the validity times in the ARF catalogs and the new ARF arrays

; JMX1
rdfitscol,all_arf_file(0)+'+1',tstart1,'TSTART',/sil
rdfitscol,all_arf_file(0)+'+1',tstop1,'TSTOP',/sil
rdfitscol,all_arf_file(0)+'+1',comm1,'COMMENT',/sil
rdfitscol,all_arf_file(0)+'+1',arfnew1,'AXIS_ARF',/sil
rdfitscol,all_arf_file(0)+'+1',creator1,'CREATOR',/sil
print,'size of arfnew1:',size(arfnew1)

creator1 = strtrim(creator1,2)
w351_1 = where( creator1 eq 'j_src_spectra 3.5.1', n_w351_1)
w400_1 = where( creator1 eq 'j_src_spectra 4.0.0', n_w400_1)

; JMX2
rdfitscol,all_arf_file(1)+'+1',tstart2,'TSTART',/sil
rdfitscol,all_arf_file(1)+'+1',tstop2,'TSTOP',/sil
rdfitscol,all_arf_file(1)+'+1',comm2,'COMMENT',/sil
rdfitscol,all_arf_file(1)+'+1',arfnew2,'AXIS_ARF',/sil
rdfitscol,all_arf_file(1)+'+1',creator2,'CREATOR',/sil
print,'size of arfnew2:',size(arfnew2)

creator2 = strtrim(creator2,2)
w351_2 = where( creator2 eq 'j_src_spectra 3.5.1', n_w351_2)
w400_2 = where( creator2 eq 'j_src_spectra 4.0.0', n_w400_2)


;  Open the log file

openw,log,'substitute_arf.log',/get_lun
printf,log,'Run of substitute_arf Version c    '+systime()
printf,log

; If no keyword 'filelist' is given then build the list from
; the top directory: datadir


if not keyword_set(filelist) and not keyword_set(txtfile) then begin

  printf,log,'Search for ''jmx?_srcl_spe.fits'' files below directory'
  printf,log,'      '+datadir

  list_of_files = file_search( datadir, 'jmx?_srcl_spe.fits', count=nlist )
   if nlist eq 0 then begin
      print,'No jmx?_srcl_spe.fits file found under '+datadir
      print,'substitute_arf exits without changing anything'
      printf,log,'No files found, exit'
      free_lun,log
      return
   endif else printf,log,nlist,' files found'

endif

if keyword_set(filelist) then begin

   printf,log,'The spectral files are given by keyword ''filelist'''
   nlist = n_elements(filelist)
   list_of_files = filelist
   printf,log,nlist,' files given'

endif

if keyword_set(txtfile) then begin

   printf,log,'The spectral files are given in text file:'
   printf,log,'      '+txtfile
   list_of_files = read_slist( txtfile, count=nlist )
   if nlist eq 0 then begin
      print,'No lines in file: '+txtfile+', exit'
      printf,log,'No lines in file: '+txtfile+', exit'
      free_lun,log
    endif

endif


;  LOOP OVER ALL SPECTRAL FILES

for loop = 0, nlist-1 do begin

   print,'Started on '+list_of_files(loop)
   printf,log
   printf,log,'Spectral file: '+list_of_files(loop)
   printf,log

   if not file_test(list_of_files(loop),/reg) then begin
      print,list_of_files(loop)+' was not found - skip!'
      printf,log,list_of_files(loop)+' was not found - skip!'
      printf,log
      continue
   endif

   ; Get the number of spectra first

   specdol = list_of_files(loop)+'+1'
   rdfitscol,specdol,anchor,'ANCRFILE',/sil,nrows=nspctr

   if nspctr eq 0 then begin
      print,'No spectra found in file: '+list_of_files(loop)
      print,'skip and continue'
      printf,log,'No spectra found, skip'
      continue
   endif

   ; Get directory of spectral file

   pos = strpos(list_of_files(loop),'/',/reverse_search)
   if pos lt 0 then begin
      curdir = './'
   endif else begin
      curdir = strmid(list_of_files(loop),0,pos+1)
   endelse

   ; Get extra information

   hdr = headfits(list_of_files(loop),ext=1)
   ijd_start = fxpar(hdr,'TSTART')
   ijd_stop  = fxpar(hdr,'TSTOP')
   sz1 = size(ijd_start)
   sz2 = size(ijd_stop)
   if sz1(sz1(0)+1) eq 7 or sz2(sz2(0)+1) eq 7 then begin
      read,'TSTART/TSTOP not defined, enter IJD for observation: ... ', ijd
   endif else ijd = 0.5*(ijd_start + ijd_stop)
   print,'IJD = ', ijd
   creator = strtrim(fxpar(hdr,'CREATOR'),2)
   print,'      '+creator+' was the creator'
   instrume = strtrim(fxpar(hdr,'INSTRUME'),2)
   jmxname = strlowcase(instrume)

   printf,log,'From header: '+instrume+', IJD = ', ijd
   printf,log,'            creator: '+creator

   if creator ne 'j_src_spectra 3.5.1' and creator ne 'j_src_spectra 4.0.0' then begin
      print,'Warning! Spectra in file: '+list_of_files(loop)
      print,'    have been extracted with '+creator
      print,'    and not with j_src_spectra 3.5.1 or 4.0.0 as expected.'
      print,'Results are therefore unreliable but substitution continues ...'
      printf,log,'Warning! Inappropriate spectral extraction, but continues'
   endif

   ; Determine the actual period for new ARF

   if instrume eq 'JMX1' then begin

      ; Use only rows where creator is correct

      if creator eq 'j_src_spectra 3.5.1' then begin
         w = w351_1
         ngt1 = n_w351_1
      endif else begin
         w = w400_1
         ngt1 = n_w400_1
      endelse 

      for i = 0, ngt1-1 do begin
         if ijd ge tstart1(w(i)) and ijd lt tstop1(w(i)) then break
      endfor
      if i eq ngt1 then begin
         print,'Warning! No ARF replacement for IJD = ',ijd
         print,'   from spectral file: '+list_of_files(loop)
         printf,log,'Warning! No ARF replacement for current IJD, skip'
         continue
      endif
      print,'JMX1 selection new ARF # ', w(i)
      arfnew = arfnew1(*,w(i))
   endif else begin         ;         JMX2

      ; Use only rows where creator is correct

      if creator eq 'j_src_spectra 3.5.1' then begin
         w = w351_2
         ngt2 = n_w351_2
      endif else begin
         w = w400_2
         ngt2 = n_w400_2
      endelse 

      for i = 0, ngt2-1 do begin
         if ijd ge tstart2(w(i)) and ijd lt tstop2(w(i)) then break
      endfor
      if i eq ngt2 then begin
         print,'Warning! No ARF replacement for IJD = ',ijd
         print,'   from spectral file: '+list_of_files(loop)
         printf,log,'Warning! No ARF replacement for current IJD, skip'
         continue
      endif
      print,'JMX2 selection new ARF # ', w(i)
      arfnew = arfnew2(*,w(i))
   endelse

   printf,log,'Substitute ARF from catalog row# ',w(i)+1


   rdfitscol,specdol,rowid,'ROWID',/sil

   ; Strip off the curly brackets if found

   for i = 0, nspctr-1 do begin

      printf,log,'   spectrum# ',i+1,'  source ID: '+rowid(i)

      anc = strtrim(anchor(i),2)
      len = strlen(anc)
      printf,log,'   anchor file: '+anc

      if strmid(anc,len-1,1) eq '}' then begin
         pos = strpos(anc,'{')
         if pos gt 0 then begin
            anc = strmid(anc,0,pos)
         endif else begin
            print,'Strange name for anchorfile: '+anc
            print,' given in spectral file: '+list_of_files(loop)
         endelse
      endif
      anchor(i) = curdir+anc  ; Compose full path+name

      if i gt 0 then begin ; Compare the names - the same name is expected

         if anchor(0) ne anchor(i) then begin
            print,'Warning! the anchor files (ARF files) are different'
            print,'for spectral file: '+list_of_files(loop)
            print,'The current version of substitute_arf does not handle'
            print,'that situation so results are unpredictable'
            printf,log,'Warning! the anchor files (ARF files) are different'
         endif
      endif
   endfor

   ; Read the relevant ARF
   
   
   ; Open the local ARF and check whether this is the first
   ; ARF substitution

   local_arf_file = anchor(0)
   loc_ext = 1

   if not file_test(local_arf_file,/reg) then begin
      print,'ARF file - '+local_arf_file+' - was not found'
      print,'skip and continue'
      printf,log,'ARF file - '+local_arf_file+' - was not found, skip'
      continue
   endif

   hdr = headfits(local_arf_file,exten=loc_ext)
   nspectra = fxpar(hdr,'NAXIS2')
   arfsubs = fxpar(hdr,'ARFSUBS') ; ARF substitutions
   keystr = replicate('VFACT',nspectra)
   vignfac = fltarr(nspectra)
   for i = 1, nspectra do begin
      keystr(i-1) = keystr(i-1)+string(i,format='(i3.3)')
   endfor

   fxbopen, unit, local_arf_file, loc_ext, access='RW'

   if arfsubs eq 0 then begin  ; this is first update

      ;   determine the various vignetting factors
      for i = 1, nspectra do begin
         fxbread,unit,arf, 4, i
         fac = arf(324) / 45.125
         ; 45.125 is the AXIS_ARF value in jmxi_rmf_grp_0005.fits
         ; @ 25 keV (energ_lo(324))
         vignfac(i-1) = fac
      endfor
   endif else begin
      for i = 1, nspectra do begin
         vignfac(i-1) = fxpar(hdr,keystr(i-1))
      endfor
   endelse
      
   
   for i = 0, nspectra-1 do begin
         fxbwrite, unit, arfnew*vignfac(i), 4, i+1
   endfor
   
   fxbfinish, unit

   ; If this is a first substitution then update the header

   if arfsubs eq 0 then begin
      fxhmodify,local_arf_file,exten=loc_ext,'ARFSUBS',1,'ARF substitution has been done'
      for i = 1, nspectra do begin
         fxhmodify,local_arf_file,exten=loc_ext,keystr(i-1),$
              vignfac(i-1),'Vignetting factor for this spectrum'
      endfor
   endif


endfor

free_lun, log

end

