function tbget, h, tab, field, rows, nulls, CONTINUE=continue, NOSCALE = noscale
;+
; NAME:
;	TBGET 
; PURPOSE:
;	Function to return value(s) from specified column in a FITS binary
;	table
;
; CALLING SEQUENCE
;	values = TBGET( h, tab, field, [ rows, nulls, /NOSCALE, /CONTINUE ] )
;
; INPUTS:
;	h - FITS binary table header, e.g. as returned by READFITS()
;	tab - FITS binary table array, e.g. as returned by READFITS()
;	field - field name or number, scalar
;
; OPTIONAL INPUTS:
;	rows -  scalar or vector giving row number(s)
;		Row numbers start at 0.  If not supplied or set to
;		-1 then values for all rows are returned
;
; OPTIONAL KEYWORD INPUT:
;	NOSCALE - If this keyword is set and nonzero, then the TSCALn and
;		TZEROn keywords will *not* be used to scale to physical values
;		Default is to perfrom scaling
;	CONTINUE - On the first call to TBGET() the FITS header is interrogated
;		(using TBINFO) to get the column names, formats, units etc.
;		If /CONTINUE is set, then TBINFO is not called again, but
;		the same values (stored in a common block) from the previous
;		call are used again.   Used to improve efficiency.
; OUTPUTS:
;	the values for the row are returned as the function value.
;	Null values are set to 0 or blanks for strings.
;
; OPTIONAL OUTPUT:
;	nulls - null value flag of same length as the returned data.
;		Only used for integer data types, B, I, and J
;		It is set to 1 at null value positions and 0 elsewhere.
;		If supplied then the optional input, rows, must also
;		be supplied.
;
; COMMON BLOCKS:
;	TB_COMMON - stores the values returned by TBINFO
; NOTES:
;	(1) If the column is variable length ('P') format, then TBGET() will 
;	return the longword array of pointers into the heap area.   TBGET() 
;	currently lacks the ability to actually extract the data from the 
;	heap area.  
; PROCEDURE CALLS:
;	DATATYPE(), IEEE_TO_HOST, TBINFO, TBSIZE 
; HISTORY:
;       Written  W. Landsman        February, 1991
;       Work for string and complex   W. Landsman         April, 1993
;	Default scaling by TSCALn, TZEROn, Added /NOSCALE keyword,
;	Fixed nulls output, return longword pointers for variable length
;		binary tables,     W. Landsman  December 1996
;-
;------------------------------------------------------------------
 On_error,2

 common tb_common, tbcol, width, idltype, numval, tunit, tnull, tform, ttype, $
	maxval, tscale, tzero
	

 if N_params() LT 3 then begin
    print, $
 'Syntax - values = TBGET(h, tab, field, [ rows, nulls, /NOSCALE, /NOCONTIN ])'
    return, -1
 endif

 !ERR = 0 			;no error yet

; get size of table

 tbsize, h, tab, ncols, nrows, tfields

; get characteristics of specified field

 if ( not keyword_set(CONTINUE) ) or ( N_elements(tbcol) EQ 0 ) then $
    tbinfo, h, tbcol, width, idltype, numval, $ 
            tunit, tnull, tform, ttype, maxval, tscale, tzero

 case datatype(field) of

 'STR': begin
      i = where( strupcase(ttype) EQ strupcase(field), Nfound)
      if Nfound EQ 0 then $ 
         message,'Field ' + field + ' not found in header'
      end

 'UND':message,'First parameter must be field name or number'
 
 ELSE: begin
      i = field-1
      if (i LT 0 ) or (i GT tfields) then $
            message,'Field number must be between 1 and ' +strtrim(tfields,2)
      end

 endcase

 i = i(0)

  if tform(i) EQ 'P' then message, /INF, $ 
           'Variable Length column - returning array of pointers'

; if rows not supplied then return all rows

 if N_params() LT 4 then rows = -1

; determine if scalar supplied

 row = rows
 s =size(row) & ndim = s(0)   
 if ndim EQ 0 then begin		;scalar?
        if row LT 0 then begin	; -1 get all rows
		ndim = 1
		row = lindgen(nrows)
	   end else begin
		row = lonarr(1) + row
	end
 end

 nrow = N_elements(row)

; check for valid row numbers

 if (min(row) LT 0) or (max(row) GT (nrows-1)) then $
	message,'Row numbers must be between 0 and '+ strtrim(nrows-1,2)

; get column

 if ndim EQ 0 then begin					;scalar?
	d = tab(tbcol(i):tbcol(i)+numval(i)*width(i)-1,row(0))
    end else begin					;vector
	d = tab(tbcol(i):tbcol(i)+numval(i)*width(i)-1,row)
 end
 n = N_elements(d)
 Nnull = 0

; convert data to the correct type

 case idltype(i) of

 1:  begin
     temp = byte( d, 0, numval(i), nrow)
     if tform(i) EQ 'L' then begin
       d = strarr( numval(i), nrow )
       for j = 0, numval(i)*nrow-1 do d(j) = string( temp(j) )
     endif else if tnull(i) NE 0 then nullval = where(d EQ tnull(i), Nnull)
     end

 2:  begin
     byteorder,d,/NtoHS
     d = fix(d,0, numval(i), nrow)
     if tnull(i) NE 0 then nullval = where(d EQ tnull(i), Nnull)
     end
 
 3:  begin
     byteorder,d,/NtoHL
     d = long( d, 0, numval(i), nrow)
     if tnull(i) NE 0 then nullval = where(d EQ tnull(i), Nnull)
     end

 4:  begin
     d = float( d, 0, numval(i), nrow)
     ieee_to_host, d
     end

 5:  begin
     d = double( d, 0, numval(i), nrow)
     ieee_to_host, d
     end

 6:  begin
     d = complex( d, 0, numval(i), nrow)
     ieee_to_host, d
     end

 7:  d = string(d)

 endcase

 if not keyword_set(NOSCALE) then begin
	if ( (tscale(i) NE 1.0) or (tzero(i) NE 0.0) ) then $
		d = d*tscale(i) + tzero(i)
 endif

 if N_params() EQ 5 then begin
	 nulls = bytarr(n)
	 if Nnull GT 0 then begin
		nulls(nullval) = 1b
		d(nullval) = 0
	endif
 endif	

; Extract correct rows if vector supplied

 if N_elements(d) EQ 1 then return, d(0) else return, reform(d)

 end
